@@ -3,7 +3,7 @@ require 'benchmark'
 def bench_rescue(bm)
   bm.report("control") { 1_000_000.times { foo1 } }
   bm.report("one rescue") { 1_000_000.times { foo2a } }
-  bm.report("one rescue + raise") { 1_000_000.times { foo2a } }
+  bm.report("one rescue + raise") { 1_000_000.times { foo2b } }
   bm.report("five rescues") { 1_000_000.times { foo3a } }
   bm.report("five rescues + raise") { 1_000_000.times { foo3b } }
 end
@@ -15,5 +15,8 @@ def foo3a; rescue NameError; rescue NameError; rescue NameError; rescue NameErro
 def foo3b; raise; rescue NameError; rescue NameError; rescue NameError; rescue NameError; rescue RuntimeError; end
 
 if $0 == __FILE__
-  Benchmark.bmbm(40) {|bm| bench_rescue(bm)}
+  TIMES = (ARGV[0] || 1).to_i
+  TIMES.times {
+    Benchmark.bm(40) {|bm| bench_rescue(bm)}
+  }
 end
