@@ -67,6 +67,18 @@ describe "Java String and primitive-typed methods" do
     CoreTypeMethods.setBooleanTrue(true).should == "true"
     CoreTypeMethods.setBooleanFalse(false).should == "false"
 
+    CoreTypeMethods.setByte(nil).should == "0"
+    CoreTypeMethods.setShort(nil).should == "0"
+    CoreTypeMethods.setChar(nil).should == "\000"
+    CoreTypeMethods.setInt(nil).should == "0"
+    CoreTypeMethods.setLong(nil).should == "0"
+
+    CoreTypeMethods.setFloat(nil).should == "0.0"
+    CoreTypeMethods.setDouble(nil).should == "0.0"
+
+    CoreTypeMethods.setBooleanTrue(nil).should == "false"
+    CoreTypeMethods.setBooleanFalse(nil).should == "false"
+
     CoreTypeMethods.setBigInteger(1234567890123456789012345678901234567890).should ==
       "1234567890123456789012345678901234567890"
 
@@ -90,18 +102,28 @@ describe "Java String and primitive-typed methods" do
 
     CoreTypeMethods.setBooleanTrueObj(true).should == "true"
     CoreTypeMethods.setBooleanFalseObj(false).should == "false"
+
+    CoreTypeMethods.setByteObj(nil).should == "null"
+    CoreTypeMethods.setShortObj(nil).should == "null"
+    CoreTypeMethods.setCharObj(nil).should == "null"
+    CoreTypeMethods.setIntObj(nil).should == "null"
+    CoreTypeMethods.setLongObj(nil).should == "null"
+
+    CoreTypeMethods.setFloatObj(nil).should == "null"
+    CoreTypeMethods.setDoubleObj(nil).should == "null"
+
+    CoreTypeMethods.setBooleanTrueObj(nil).should == "null"
+    CoreTypeMethods.setBooleanFalseObj(nil).should == "null"
     
     CoreTypeMethods.setNull(nil).should == "null"
   end
   
   it "should raise errors when passed values can not be precisely coerced" do
-    pending("precision failure does not raise error") do
-      lambda { CoreTypeMethods.setByte(1 << 8) }.should raise_error(TypeError)
-      lambda { CoreTypeMethods.setShort(1 << 16) }.should raise_error(TypeError)
-      lambda { CoreTypeMethods.setChar(1 << 16) }.should raise_error(TypeError)
-      lambda { CoreTypeMethods.setInt(1 << 32) }.should raise_error(TypeError)
-      lambda { CoreTypeMethods.setLong(1 << 64) }.should raise_error(TypeError)
-    end
+    lambda { CoreTypeMethods.setByte(1 << 8) }.should raise_error(RangeError)
+    lambda { CoreTypeMethods.setShort(1 << 16) }.should raise_error(RangeError)
+    lambda { CoreTypeMethods.setChar(1 << 16) }.should raise_error(RangeError)
+    lambda { CoreTypeMethods.setInt(1 << 32) }.should raise_error(RangeError)
+    lambda { CoreTypeMethods.setLong(1 << 64) }.should raise_error(RangeError)
   end
   
   it "should select the most narrow and precise overloaded method" do
@@ -119,26 +141,28 @@ describe "Java String and primitive-typed methods" do
     pending "passing null to overloaded methods randomly selects from them" do
       CoreTypeMethods.getType(nil).should == "CharSequence"
     end
+
+    CoreTypeMethods.getType(BigDecimal.new('1.1')).should == "BigDecimal"
   end
 end
 
 describe "Java Object-typed methods" do
-  it "should coerce primitive Ruby types to the narrowest possible boxed Java type" do
+  it "should coerce primitive Ruby types to a single, specific Java type" do
     CoreTypeMethods.getObjectType("foo").should == "class java.lang.String"
 
-    CoreTypeMethods.getObjectType(0).should == "class java.lang.Byte"
-    CoreTypeMethods.getObjectType(java::lang::Byte::MAX_VALUE).should == "class java.lang.Byte"
-    CoreTypeMethods.getObjectType(java::lang::Byte::MIN_VALUE).should == "class java.lang.Byte"
-    CoreTypeMethods.getObjectType(java::lang::Byte::MAX_VALUE + 1).should == "class java.lang.Short"
-    CoreTypeMethods.getObjectType(java::lang::Byte::MIN_VALUE - 1).should == "class java.lang.Short"
+    CoreTypeMethods.getObjectType(0).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Byte::MAX_VALUE).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Byte::MIN_VALUE).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Byte::MAX_VALUE + 1).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Byte::MIN_VALUE - 1).should == "class java.lang.Long"
 
-    CoreTypeMethods.getObjectType(java::lang::Short::MAX_VALUE).should == "class java.lang.Short"
-    CoreTypeMethods.getObjectType(java::lang::Short::MIN_VALUE).should == "class java.lang.Short"
-    CoreTypeMethods.getObjectType(java::lang::Short::MAX_VALUE + 1).should == "class java.lang.Integer"
-    CoreTypeMethods.getObjectType(java::lang::Short::MIN_VALUE - 1).should == "class java.lang.Integer"
+    CoreTypeMethods.getObjectType(java::lang::Short::MAX_VALUE).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Short::MIN_VALUE).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Short::MAX_VALUE + 1).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Short::MIN_VALUE - 1).should == "class java.lang.Long"
 
-    CoreTypeMethods.getObjectType(java::lang::Integer::MAX_VALUE).should == "class java.lang.Integer"
-    CoreTypeMethods.getObjectType(java::lang::Integer::MIN_VALUE).should == "class java.lang.Integer"
+    CoreTypeMethods.getObjectType(java::lang::Integer::MAX_VALUE).should == "class java.lang.Long"
+    CoreTypeMethods.getObjectType(java::lang::Integer::MIN_VALUE).should == "class java.lang.Long"
     CoreTypeMethods.getObjectType(java::lang::Integer::MAX_VALUE + 1).should == "class java.lang.Long"
     CoreTypeMethods.getObjectType(java::lang::Integer::MIN_VALUE - 1).should == "class java.lang.Long"
 
@@ -147,13 +171,13 @@ describe "Java Object-typed methods" do
     CoreTypeMethods.getObjectType(java::lang::Long::MAX_VALUE + 1).should == "class java.math.BigInteger"
     CoreTypeMethods.getObjectType(java::lang::Long::MIN_VALUE - 1).should == "class java.math.BigInteger"
 
-    CoreTypeMethods.getObjectType(java::lang::Float::MAX_VALUE).should == "class java.lang.Float"
-    CoreTypeMethods.getObjectType(java::lang::Float::MIN_VALUE).should == "class java.lang.Float"
-    CoreTypeMethods.getObjectType(-java::lang::Float::MAX_VALUE).should == "class java.lang.Float"
-    CoreTypeMethods.getObjectType(-java::lang::Float::MIN_VALUE).should == "class java.lang.Float"
+    CoreTypeMethods.getObjectType(java::lang::Float::MAX_VALUE).should == "class java.lang.Double"
+    CoreTypeMethods.getObjectType(java::lang::Float::MIN_VALUE).should == "class java.lang.Double"
+    CoreTypeMethods.getObjectType(-java::lang::Float::MAX_VALUE).should == "class java.lang.Double"
+    CoreTypeMethods.getObjectType(-java::lang::Float::MIN_VALUE).should == "class java.lang.Double"
 
-    CoreTypeMethods.getObjectType(java::lang::Float::NaN).should == "class java.lang.Float"
-    CoreTypeMethods.getObjectType(0.0).should == "class java.lang.Float"
+    CoreTypeMethods.getObjectType(java::lang::Float::NaN).should == "class java.lang.Double"
+    CoreTypeMethods.getObjectType(0.0).should == "class java.lang.Double"
 
     CoreTypeMethods.getObjectType(java::lang::Double::MAX_VALUE).should == "class java.lang.Double"
     CoreTypeMethods.getObjectType(java::lang::Double::MIN_VALUE).should == "class java.lang.Double"
@@ -370,62 +394,65 @@ end
 
 describe "Java primitive-box-typed interface methods" do
   it "should coerce to Ruby types for the implementer" do
-    pending "We do not coerce numerics to boxed Java numerics (yet?)" do
-      impl = Class.new {
-        attr_accessor :result
-        include ValueReceivingInterface
+    impl = Class.new {
+      attr_accessor :result
+      include ValueReceivingInterface
 
-        def receiveByte(obj)
-          self.result = obj
-          obj
-        end
+      def receiveByte(obj)
+        self.result = obj
+        obj
+      end
 
-        %w[Short Char Int Long Float Double True False].each do |type|
-          alias_method "receive#{type}".intern, :receiveByte
-        end
-      }
+      alias_method :receiveByteObj, :receiveByte
 
-      vri = impl.new
-      vri_handler = ValueReceivingInterfaceHandler.new(vri);
+      %w[Short Char Int Long Float Double True False].each do |type|
+        alias_method "receive#{type}".intern, :receiveByte
+        alias_method "receive#{type}Obj".intern, :receiveByte
+      end
+    }
 
-      obj = 1
+    vri = impl.new
+    vri_handler = ValueReceivingInterfaceHandler.new(vri);
 
-      vri_handler.receiveByteObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Fixnum
+    obj = 1
 
-      vri_handler.receiveShortObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Fixnum
+    vri_handler.receiveByteObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Fixnum
 
+    vri_handler.receiveShortObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Fixnum
+
+    pending "char appears to be getting signed/unsigned-garbled" do
       vri_handler.receiveCharObj(obj).should == obj
       vri.result.should == obj
       vri.result.class.should == Fixnum
-
-      vri_handler.receiveIntObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Fixnum
-
-      vri_handler.receiveLongObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Fixnum
-
-      vri_handler.receiveFloatObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Float
-
-      vri_handler.receiveDoubleObj(obj).should == obj
-      vri.result.should == obj
-      vri.result.class.should == Float
-
-      vri_handler.receiveTrueObj(true).should == true
-      vri.result.should == true
-      vri.result.class.should == TrueClass
-
-      vri_handler.receiveFalseObj(false).should == false
-      vri.result.should == false
-      vri.result.class.should == FalseClass
     end
+
+    vri_handler.receiveIntObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveLongObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Fixnum
+
+    vri_handler.receiveFloatObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Float
+
+    vri_handler.receiveDoubleObj(obj).should == obj
+    vri.result.should == obj
+    vri.result.class.should == Float
+
+    vri_handler.receiveTrueObj(true).should == true
+    vri.result.should == true
+    vri.result.class.should == TrueClass
+
+    vri_handler.receiveFalseObj(false).should == false
+    vri.result.should == false
+    vri.result.class.should == FalseClass
   end
 end
 
