@@ -57,7 +57,7 @@ public final class IncludedModuleWrapper extends RubyClass {
     private final RubyModule delegate;
 
     public IncludedModuleWrapper(Ruby runtime, RubyClass superClass, RubyModule delegate) {
-        super(runtime, superClass, delegate.generation, false);
+        super(runtime, superClass, false);
         this.delegate = delegate;
         this.metaClass = delegate.metaClass;
         delegate.addIncludingHierarchy(this);
@@ -150,12 +150,12 @@ public final class IncludedModuleWrapper extends RubyClass {
         return delegate.id();
     }
 
-    //
-    // VARIABLE TABLE METHODS - pass to delegate
-    //
-    @Override
-    public InstanceVariableTable getVariables() {
-        return delegate.getVariables();
+    protected synchronized Map<String, IRubyObject> getClassVariables() {
+        return delegate.getClassVariables();
+    }
+
+    protected Map<String, IRubyObject> getClassVariablesForRead() {
+        return delegate.getClassVariablesForRead();
     }
 
     @Override
@@ -169,7 +169,7 @@ public final class IncludedModuleWrapper extends RubyClass {
     }
 
     @Override
-    protected IRubyObject variableTableFetch(String name) {
+    protected Object variableTableFetch(String name) {
         return delegate.variableTableFetch(name);
     }
 
@@ -199,32 +199,10 @@ public final class IncludedModuleWrapper extends RubyClass {
     }
 
     @Override
-    protected void variableTableSync(List<Variable<IRubyObject>> vars) {
+    protected void variableTableSync(List<Variable<Object>> vars) {
         delegate.variableTableSync(vars);
     }
 
-    /**
-     * Method to help ease transition to new variables implementation.
-     * Will likely be deprecated in the near future.
-     */
-    @SuppressWarnings("unchecked")
-    @Override
-    @Deprecated // born deprecated
-    protected Map variableTableGetMap() {
-        return delegate.variableTableGetMap();
-    }
-
-    /**
-     * Method to help ease transition to new variables implementation.
-     * Will likely be deprecated in the near future.
-     */
-    @SuppressWarnings("unchecked")
-    @Override
-    @Deprecated // born deprecated
-    protected Map variableTableGetMap(Map map) {
-        return delegate.variableTableGetMap(map);
-    }
-
     //
     // CONSTANT TABLE METHODS - pass to delegate
     //
