@@ -35,6 +35,7 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import com.kenai.constantine.platform.OpenFlags;
 import org.jcodings.Encoding;
 import org.jruby.util.io.OpenFile;
 import org.jruby.util.io.ChannelDescriptor;
@@ -192,6 +193,8 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     
     public static RubyClass createFileClass(Ruby runtime) {
         RubyClass fileClass = runtime.defineClass("File", runtime.getIO(), FILE_ALLOCATOR);
+        // Create Constants class
+        RubyModule constants = fileClass.defineModuleUnder("Constants");
         runtime.setFile(fileClass);
         RubyString separator = runtime.newString("/");
         ThreadContext context = runtime.getCurrentContext();
@@ -223,9 +226,6 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         // File::Constants below? File::Constants is included in IO.
 
         // TODO: These were missing, so we're not handling them elsewhere?
-        // FIXME: The old value, 32786, didn't match what IOModes expected, so I reference
-        // the constant here. THIS MAY NOT BE THE CORRECT VALUE.
-        fileClass.fastSetConstant("BINARY", runtime.newFixnum(ModeFlags.BINARY));
         fileClass.fastSetConstant("FNM_NOESCAPE", runtime.newFixnum(FNM_NOESCAPE));
         fileClass.fastSetConstant("FNM_CASEFOLD", runtime.newFixnum(FNM_CASEFOLD));
         fileClass.fastSetConstant("FNM_SYSCASE", runtime.newFixnum(FNM_SYSCASE));
@@ -233,15 +233,20 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         fileClass.fastSetConstant("FNM_PATHNAME", runtime.newFixnum(FNM_PATHNAME));
         
         // Create constants for open flags
-        fileClass.fastSetConstant("RDONLY", runtime.newFixnum(ModeFlags.RDONLY));
-        fileClass.fastSetConstant("WRONLY", runtime.newFixnum(ModeFlags.WRONLY));
-        fileClass.fastSetConstant("RDWR", runtime.newFixnum(ModeFlags.RDWR));
-        fileClass.fastSetConstant("CREAT", runtime.newFixnum(ModeFlags.CREAT));
-        fileClass.fastSetConstant("EXCL", runtime.newFixnum(ModeFlags.EXCL));
-        fileClass.fastSetConstant("NOCTTY", runtime.newFixnum(ModeFlags.NOCTTY));
-        fileClass.fastSetConstant("TRUNC", runtime.newFixnum(ModeFlags.TRUNC));
-        fileClass.fastSetConstant("APPEND", runtime.newFixnum(ModeFlags.APPEND));
-        fileClass.fastSetConstant("NONBLOCK", runtime.newFixnum(ModeFlags.NONBLOCK));
+        for (OpenFlags f : OpenFlags.values()) {
+            // Strip off the O_ prefix, so they become File::RDONLY, and so on
+            final String name = f.name();
+            if (name.startsWith("O_")) {
+                final String cname = name.substring(2);
+                // Special case for handling ACCMODE, since constantine will generate
+                // an invalid value if it is not defined by the platform.
+                final RubyFixnum cvalue = f == OpenFlags.O_ACCMODE
+                        ? runtime.newFixnum(ModeFlags.ACCMODE)
+                        : runtime.newFixnum(f.value());
+                fileClass.fastSetConstant(cname, cvalue);
+                constants.fastSetConstant(cname, cvalue);
+            }
+        }
         
         // Create constants for flock
         fileClass.fastSetConstant("LOCK_SH", runtime.newFixnum(RubyFile.LOCK_SH));
@@ -249,29 +254,12 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         fileClass.fastSetConstant("LOCK_NB", runtime.newFixnum(RubyFile.LOCK_NB));
         fileClass.fastSetConstant("LOCK_UN", runtime.newFixnum(RubyFile.LOCK_UN));
         
-        // Create Constants class
-        RubyModule constants = fileClass.defineModuleUnder("Constants");
-        
-        // TODO: These were missing, so we're not handling them elsewhere?
-        constants.fastSetConstant("BINARY", runtime.newFixnum(ModeFlags.BINARY));
-        constants.fastSetConstant("SYNC", runtime.newFixnum(0x1000));
         constants.fastSetConstant("FNM_NOESCAPE", runtime.newFixnum(FNM_NOESCAPE));
         constants.fastSetConstant("FNM_CASEFOLD", runtime.newFixnum(FNM_CASEFOLD));
         constants.fastSetConstant("FNM_SYSCASE", runtime.newFixnum(FNM_SYSCASE));
         constants.fastSetConstant("FNM_DOTMATCH", runtime.newFixnum(FNM_DOTMATCH));
         constants.fastSetConstant("FNM_PATHNAME", runtime.newFixnum(FNM_PATHNAME));
         
-        // Create constants for open flags
-        constants.fastSetConstant("RDONLY", runtime.newFixnum(ModeFlags.RDONLY));
-        constants.fastSetConstant("WRONLY", runtime.newFixnum(ModeFlags.WRONLY));
-        constants.fastSetConstant("RDWR", runtime.newFixnum(ModeFlags.RDWR));
-        constants.fastSetConstant("CREAT", runtime.newFixnum(ModeFlags.CREAT));
-        constants.fastSetConstant("EXCL", runtime.newFixnum(ModeFlags.EXCL));
-        constants.fastSetConstant("NOCTTY", runtime.newFixnum(ModeFlags.NOCTTY));
-        constants.fastSetConstant("TRUNC", runtime.newFixnum(ModeFlags.TRUNC));
-        constants.fastSetConstant("APPEND", runtime.newFixnum(ModeFlags.APPEND));
-        constants.fastSetConstant("NONBLOCK", runtime.newFixnum(ModeFlags.NONBLOCK));
-        
         // Create constants for flock
         constants.fastSetConstant("LOCK_SH", runtime.newFixnum(RubyFile.LOCK_SH));
         constants.fastSetConstant("LOCK_EX", runtime.newFixnum(RubyFile.LOCK_EX));
@@ -403,7 +391,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         }
     }
 
-    @JRubyMethod(required = 1, optional = 2, frame = true, visibility = Visibility.PRIVATE)
+    @JRubyMethod(required = 1, optional = 2, frame = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_8)
     @Override
     public IRubyObject initialize(IRubyObject[] args, Block block) {
         if (openFile == null) {
@@ -420,12 +408,71 @@ public class RubyFile extends RubyIO implements EncodingCapable {
 
         return openFile(args);
     }
+
+    @JRubyMethod(name = "initialize", required = 1, optional = 2, frame = true, visibility = Visibility.PRIVATE, compat = CompatVersion.RUBY1_9)
+    public IRubyObject initialize19(ThreadContext context, IRubyObject[] args, Block block) {
+        if (openFile == null) {
+            throw context.getRuntime().newRuntimeError("reinitializing File");
+        }
+
+        if (args.length > 0 && args.length <= 3) {
+            IRubyObject fd = TypeConverter.convertToTypeWithCheck(args[0], context.getRuntime().getFixnum(), "to_int");
+            if (!fd.isNil()) {
+                args[0] = fd;
+                if (args.length == 1) {
+                    return super.initialize19(context, args[0], block);
+                } else if (args.length == 2) {
+                    return super.initialize19(context, args[0], args[1], block);
+                }
+                return super.initialize19(context, args[0], args[1], args[2], block);
+            }
+        }
+
+        return openFile19(context, args);
+    }
+
+    private IRubyObject openFile19(ThreadContext context, IRubyObject args[]) {
+        RubyString filename = get_path(context, args[0]);
+        context.getRuntime().checkSafeString(filename);
+
+        path = filename.getUnicodeValue();
+
+        String modeString;
+        ModeFlags modes = new ModeFlags();
+        int perm = 0;
+
+        try {
+            if (args.length > 1) {
+                modeString = args[1].convertToString().toString();
+                modes = parseModes19(context, args[1]);
+            } else {
+                modeString = "r";
+                modes = parseModes19(context, RubyString.newString(context.getRuntime(), modeString));
+            }
+            if (args.length > 2 && !args[2].isNil()) {
+                if (args[2] instanceof RubyHash) {
+                    modes = parseOptions(context, args[2], modes);
+                } else {
+                    perm = getFilePermissions(args);
+                }
+            }
+            if (perm > 0) {
+                sysopenInternal(path, modes, perm);
+            } else {
+                openInternal(path, modeString, modes);
+            }
+        } catch (InvalidValueException ex) {
+            throw context.getRuntime().newErrnoEINVALError();
+        } finally {}
+
+        return this;
+    }
     
     private IRubyObject openFile(IRubyObject args[]) {
-        IRubyObject filename = args[0].convertToString();
+        RubyString filename = get_path(getRuntime().getCurrentContext(), args[0]);
         getRuntime().checkSafeString(filename);
         
-        path = filename.convertToString().getUnicodeValue();
+        path = filename.getUnicodeValue();
         
         String modeString;
         ModeFlags modes;
@@ -433,25 +480,14 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         
         try {
             if ((args.length > 1 && args[1] instanceof RubyFixnum) || (args.length > 2 && !args[2].isNil())) {
-                if (args[1] instanceof RubyFixnum) {
-                    modes = new ModeFlags(RubyNumeric.num2int(args[1]));
-                } else {
-                    modeString = args[1].convertToString().toString();
-                    modes = getIOModes(getRuntime(), modeString);
-                }
-                if (args.length > 2 && !args[2].isNil()) {
-                    perm = RubyNumeric.num2int(args[2]);
-                } else {
-                    perm = 438; // 0666
-                }
+                modes = parseModes(args[1]);
+                perm = getFilePermissions(args);
 
                 sysopenInternal(path, modes, perm);
             } else {
                 modeString = "r";
-                if (args.length > 1) {
-                    if (!args[1].isNil()) {
-                        modeString = args[1].convertToString().toString();
-                    }
+                if (args.length > 1 && !args[1].isNil()) {
+                    modeString = args[1].convertToString().toString();
                 }
                 
                 openInternal(path, modeString);
@@ -463,17 +499,33 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         return this;
     }
 
+    private int getFilePermissions(IRubyObject[] args) {
+        return (args.length > 2 && !args[2].isNil()) ? RubyNumeric.num2int(args[2]) : 438;
+    }
+
     protected void sysopenInternal(String path, ModeFlags modes, int perm) throws InvalidValueException {
         openFile = new OpenFile();
         
         openFile.setPath(path);
         openFile.setMode(modes.getOpenFileFlags());
+        int umask = getRuntime().getPosix().umask(0);
+        perm = perm - (perm & umask);
         
         ChannelDescriptor descriptor = sysopen(path, modes, perm);
         openFile.setMainStream(fdopen(descriptor, modes));
         
         registerDescriptor(descriptor);
     }
+
+    protected void openInternal(String path, String modeString, ModeFlags modes) throws InvalidValueException {
+        openFile = new OpenFile();
+
+        openFile.setMode(modes.getOpenFileFlags());
+        openFile.setPath(path);
+        openFile.setMainStream(fopen(path, modeString));
+
+        registerDescriptor(openFile.getMainStream().getDescriptor());
+    }
     
     protected void openInternal(String path, String modeString) throws InvalidValueException {
         openFile = new OpenFile();
@@ -556,6 +608,8 @@ public class RubyFile extends RubyIO implements EncodingCapable {
             throw getRuntime().newErrnoEINVALError();
         } catch (PipeException ex) {
             throw getRuntime().newErrnoEPIPEError();
+        } catch (SecurityException ex) {
+            throw getRuntime().newErrnoEACCESError("Permission denied - " + path);
         }
     }
 
@@ -640,7 +694,32 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         return getLastModified(context.getRuntime(), path);
     }
 
-    @JRubyMethod
+    @JRubyMethod(meta = true, frame = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject path(ThreadContext context, IRubyObject self, IRubyObject str) {
+        return get_path(context, str);
+    }
+
+    /**
+     * similar in spirit to rb_get_path from 1.9 source
+     * @param context
+     * @param obj
+     * @return
+     */
+    public static RubyString get_path(ThreadContext context, IRubyObject obj) {
+        if (context.getRuntime().is1_9()) {
+            IRubyObject str = obj.checkStringType();
+            if (!str.isNil()) {
+                return (RubyString) str;
+            }
+            if (obj.respondsTo("to_path")) {
+                obj = obj.callMethod(context, "to_path");
+            }
+        }
+
+        return obj.convertToString();
+    }
+
+    @JRubyMethod(name = {"path", "to_path"})
     public RubyString path(ThreadContext context) {
         return context.getRuntime().newString(path);
     }
@@ -706,7 +785,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     
     @JRubyMethod(required = 1, optional = 1, meta = true)
     public static IRubyObject basename(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
-        String name = RubyString.stringValue(args[0]).toString();
+        String name = get_path(context,args[0]).getUnicodeValue();
 
         // MRI-compatible basename handling for windows drive letter paths
         if (Platform.IS_WINDOWS) {
@@ -780,13 +859,13 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         int count = 0;
         RubyInteger mode = args[0].convertToInteger();
         for (int i = 1; i < args.length; i++) {
-            IRubyObject filename = args[i];
+            RubyString filename = get_path(context, args[i]);
             
-            if (!RubyFileTest.exist_p(filename, filename.convertToString()).isTrue()) {
+            if (!RubyFileTest.exist_p(filename, filename).isTrue()) {
                 throw runtime.newErrnoENOENTError("No such file or directory - " + filename);
             }
             
-            boolean result = 0 == runtime.getPosix().chmod(filename.toString(), (int)mode.getLongValue());
+            boolean result = 0 == runtime.getPosix().chmod(filename.getUnicodeValue(), (int)mode.getLongValue());
             if (result) {
                 count++;
             }
@@ -810,14 +889,13 @@ public class RubyFile extends RubyIO implements EncodingCapable {
             group = RubyNumeric.num2int(args[1]);
         }
         for (int i = 2; i < args.length; i++) {
-            IRubyObject filename = args[i];
+            RubyString filename = get_path(context, args[i]);
 
-            RubyString fileString = filename.convertToString();
-            if (!RubyFileTest.exist_p(filename, fileString).isTrue()) {
+            if (!RubyFileTest.exist_p(filename, filename).isTrue()) {
                 throw runtime.newErrnoENOENTError("No such file or directory - " + filename);
             }
             
-            boolean result = 0 == runtime.getPosix().chown(fileString.getUnicodeValue(), owner, group);
+            boolean result = 0 == runtime.getPosix().chown(filename.getUnicodeValue(), owner, group);
             if (result) {
                 count++;
             }
@@ -828,7 +906,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     
     @JRubyMethod(required = 1, meta = true)
     public static IRubyObject dirname(ThreadContext context, IRubyObject recv, IRubyObject arg) {
-        RubyString filename = RubyString.stringValue(arg);
+        RubyString filename = get_path(context, arg);
         
         String jfilename = filename.getUnicodeValue();
         
@@ -930,25 +1008,58 @@ public class RubyFile extends RubyIO implements EncodingCapable {
      */
     @JRubyMethod(required = 1, optional = 1, meta = true)
     public static IRubyObject expand_path(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+        return expandPathInternal(context, recv, args, true);
+    }
+
+
+    /**
+     * ---------------------------------------------------- File::absolute_path
+     *      File.absolute_path(file_name [, dir_string] ) -> abs_file_name
+     *
+     *      From Ruby 1.9.1
+     * ------------------------------------------------------------------------
+     *      Converts a pathname to an absolute pathname. Relative paths are
+     *      referenced from the current working directory of the process unless
+     *      _dir_string_ is given, in which case it will be used as the
+     *      starting point. If the given pathname starts with a ``+~+'' it is
+     *      NOT expanded, it is treated as a normal directory name.
+     *
+     *         File.absolute_path("~oracle/bin")       #=> "<relative_path>/~oracle/bin"
+     *
+     * @param context
+     * @param recv
+     * @param args
+     * @return
+     */
+    @JRubyMethod(required = 1, optional = 1, meta = true, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject absolute_path(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
+        return expandPathInternal(context, recv, args, false);
+    }
+
+    private static IRubyObject expandPathInternal(ThreadContext context, IRubyObject recv, IRubyObject[] args, boolean expandUser) {
         Ruby runtime = context.getRuntime();
-        
-        String relativePath = RubyString.stringValue(args[0]).toString();
+
+        String relativePath = get_path(context, args[0]).getUnicodeValue();
 
         boolean isAbsoluteWithFilePrefix = relativePath.startsWith("file:");
 
         String cwd = null;
         
         // Handle ~user paths 
-        relativePath = expandUserPath(context, relativePath);
+        if (expandUser) {
+            relativePath = expandUserPath(context, relativePath);
+        }
         
         // If there's a second argument, it's the path to which the first 
         // argument is relative.
         if (args.length == 2 && !args[1].isNil()) {
             
-            String cwdArg = RubyString.stringValue(args[1]).toString();
-            
+            cwd = get_path(context, args[1]).getUnicodeValue();
+
             // Handle ~user paths.
-            cwd = expandUserPath(context, cwdArg);
+            if (expandUser) {
+                cwd = expandUserPath(context, cwd);
+            }
 
             cwd = adjustRootPathOnWindows(runtime, cwd, null);
 
@@ -1176,7 +1287,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         int flags = args.length == 3 ? RubyNumeric.num2int(args[2]) : 0;
 
         ByteList pattern = args[0].convertToString().getByteList();
-        ByteList path = args[1].convertToString().getByteList();
+        ByteList path = get_path(context, args[1]).getByteList();
 
         if (org.jruby.util.Dir.fnmatch(pattern.bytes, pattern.begin, pattern.begin+pattern.realSize, 
                                        path.bytes, path.begin, path.begin+path.realSize, flags) == 0) {
@@ -1187,7 +1298,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     
     @JRubyMethod(name = "ftype", required = 1, meta = true)
     public static IRubyObject ftype(ThreadContext context, IRubyObject recv, IRubyObject filename) {
-        return context.getRuntime().newFileStat(filename.convertToString().getUnicodeValue(), true).ftype();
+        return context.getRuntime().newFileStat(get_path(context, filename).getUnicodeValue(), true).ftype();
     }
 
     private static String inspectJoin(ThreadContext context, IRubyObject recv, RubyArray parent, RubyArray array) {
@@ -1216,15 +1327,16 @@ public class RubyFile extends RubyIO implements EncodingCapable {
             }
             String element;
             if (args[i] instanceof RubyString) {
-                element = args[i].toString();
+                element = args[i].convertToString().getUnicodeValue();
             } else if (args[i] instanceof RubyArray) {
                 if (runtime.isInspecting(args[i])) {
-                    element = "[...]";
+                    throw runtime.newArgumentError("recursive array");
                 } else {
                     element = inspectJoin(context, recv, ary, ((RubyArray)args[i]));
                 }
             } else {
-                element = args[i].convertToString().toString();
+                RubyString path = get_path(context, args[i]);
+                element = path.getUnicodeValue();
             }
             
             chomp(buffer);
@@ -1259,7 +1371,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     
     @JRubyMethod(name = "lstat", required = 1, meta = true)
     public static IRubyObject lstat(ThreadContext context, IRubyObject recv, IRubyObject filename) {
-        String f = filename.convertToString().getUnicodeValue();
+        String f = get_path(context, filename).getUnicodeValue();
         if(f.startsWith("file:") && f.indexOf('!') != -1) {
             f = f.substring(5, f.indexOf("!"));
         }
@@ -1268,7 +1380,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
 
     @JRubyMethod(name = "stat", required = 1, meta = true)
     public static IRubyObject stat(ThreadContext context, IRubyObject recv, IRubyObject filename) {
-        String f = filename.convertToString().getUnicodeValue();
+        String f = get_path(context, filename).getUnicodeValue();
         if(f.startsWith("file:") && f.indexOf('!') != -1) {
             f = f.substring(5, f.indexOf("!"));
         }
@@ -1277,7 +1389,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
 
     @JRubyMethod(name = "atime", required = 1, meta = true)
     public static IRubyObject atime(ThreadContext context, IRubyObject recv, IRubyObject filename) {
-        String f = filename.convertToString().getUnicodeValue();
+        String f = get_path(context, filename).getUnicodeValue();
         if(f.startsWith("file:") && f.indexOf('!') != -1) {
             f = f.substring(5, f.indexOf("!"));
         }
@@ -1286,7 +1398,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
 
     @JRubyMethod(name = "ctime", required = 1, meta = true)
     public static IRubyObject ctime(ThreadContext context, IRubyObject recv, IRubyObject filename) {
-        String f = filename.convertToString().getUnicodeValue();
+        String f = get_path(context, filename).getUnicodeValue();
         if(f.startsWith("file:") && f.indexOf('!') != -1) {
             f = f.substring(5, f.indexOf("!"));
         }
@@ -1300,13 +1412,13 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         int count = 0;
         RubyInteger mode = args[0].convertToInteger();
         for (int i = 1; i < args.length; i++) {
-            IRubyObject filename = args[i];
+            RubyString filename = get_path(context, args[i]);
             
-            if (!RubyFileTest.exist_p(filename, filename.convertToString()).isTrue()) {
+            if (!RubyFileTest.exist_p(filename, filename).isTrue()) {
                 throw runtime.newErrnoENOENTError("No such file or directory - " + filename);
             }
             
-            boolean result = 0 == runtime.getPosix().lchmod(filename.toString(), (int)mode.getLongValue());
+            boolean result = 0 == runtime.getPosix().lchmod(filename.getUnicodeValue(), (int)mode.getLongValue());
             if (result) {
                 count++;
             }
@@ -1314,27 +1426,24 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         
         return runtime.newFixnum(count);
     }
-    
-    @JRubyMethod(required = 3, rest = true, meta = true)
+
+    @JRubyMethod(required = 2, rest = true, meta = true)
     public static IRubyObject lchown(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
         int owner = !args[0].isNil() ? RubyNumeric.num2int(args[0]) : -1;
         int group = !args[1].isNil() ? RubyNumeric.num2int(args[1]) : -1;
         int count = 0;
-        
+
         for (int i = 2; i < args.length; i++) {
             IRubyObject filename = args[i];
-            
-            if (!RubyFileTest.exist_p(filename, filename.convertToString()).isTrue()) {
-                throw runtime.newErrnoENOENTError("No such file or directory - " + filename);
-            }
-            
-            boolean result = 0 == runtime.getPosix().lchown(filename.toString(), owner, group);
-            if (result) {
+
+            if (0 != runtime.getPosix().lchown(filename.toString(), owner, group)) {
+                throw runtime.newErrnoFromLastPOSIXErrno();
+            } else {
                 count++;
             }
         }
-        
+
         return runtime.newFixnum(count);
     }
 
@@ -1360,7 +1469,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
 
     @JRubyMethod(name = "mtime", required = 1, meta = true)
     public static IRubyObject mtime(ThreadContext context, IRubyObject recv, IRubyObject filename) {
-        return getLastModified(context.getRuntime(), filename.convertToString().getUnicodeValue());
+        return getLastModified(context.getRuntime(), get_path(context, filename).getUnicodeValue());
     }
     
     @JRubyMethod(required = 2, meta = true)
@@ -1404,7 +1513,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     
     @JRubyMethod(required = 1, meta = true)
     public static RubyArray split(ThreadContext context, IRubyObject recv, IRubyObject arg) {
-        RubyString filename = RubyString.stringValue(arg);
+        RubyString filename = get_path(context, arg);
         
         return context.getRuntime().newArray(dirname(context, recv, filename),
                 basename(context, recv, new IRubyObject[] { filename }));
@@ -1413,8 +1522,8 @@ public class RubyFile extends RubyIO implements EncodingCapable {
     @JRubyMethod(required = 2, meta = true)
     public static IRubyObject symlink(ThreadContext context, IRubyObject recv, IRubyObject from, IRubyObject to) {
         Ruby runtime = context.getRuntime();
-        RubyString fromStr = RubyString.stringValue(from);
-        RubyString toStr = RubyString.stringValue(to);
+        RubyString fromStr = get_path(context, from);
+        RubyString toStr = get_path(context, to);
         String tovalue = toStr.getUnicodeValue();
         tovalue = JRubyFile.create(runtime.getCurrentDirectory(), tovalue).getAbsolutePath();
         try {
@@ -1517,7 +1626,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         }
 
         for (int i = 2, j = args.length; i < j; i++) {
-            RubyString filename = RubyString.stringValue(args[i]);
+            RubyString filename = get_path(context, args[i]);
             runtime.checkSafeString(filename);
             
             JRubyFile fileToTouch = JRubyFile.create(runtime.getCurrentDirectory(),filename.getUnicodeValue());
@@ -1537,7 +1646,7 @@ public class RubyFile extends RubyIO implements EncodingCapable {
         Ruby runtime = context.getRuntime();
          
         for (int i = 0; i < args.length; i++) {
-            RubyString filename = RubyString.stringValue(args[i]);
+            RubyString filename = get_path(context, args[i]);
             runtime.checkSafeString(filename);
             JRubyFile lToDelete = JRubyFile.create(runtime.getCurrentDirectory(), filename.getUnicodeValue());
             
