@@ -37,6 +37,7 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import java.math.BigInteger;
 import static org.jruby.util.Numeric.f_expt;
 import static org.jruby.util.Numeric.f_mul;
 import static org.jruby.util.Numeric.frexp;
@@ -46,6 +47,7 @@ import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.util.Locale;
 
+import java.util.regex.Pattern;
 import org.jcodings.specific.ASCIIEncoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
@@ -63,12 +65,22 @@ import org.jruby.util.Sprintf;
  */
 @JRubyClass(name="Float", parent="Numeric", include="Precision")
 public class RubyFloat extends RubyNumeric {
+    public static final int ROUNDS = 1;
+    public static final int RADIX = 2;
+    public static final int MANT_DIG = 53;
+    public static final int DIG = 15;
+    public static final int MIN_EXP = -1021;
+    public static final int MAX_EXP = 1024;
+    public static final int MAX_10_EXP = 308;
+    public static final int MIN_10_EXP = -307;
+    public static final double EPSILON = 2.2204460492503131e-16;
 
     public static RubyClass createFloatClass(Ruby runtime) {
         RubyClass floatc = runtime.defineClass("Float", runtime.getNumeric(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
         runtime.setFloat(floatc);
         floatc.index = ClassIndex.FLOAT;
         floatc.kindOf = new RubyModule.KindOf() {
+            @Override
             public boolean isKindOf(IRubyObject obj, RubyModule type) {
                 return obj instanceof RubyFloat;
             }
@@ -81,19 +93,19 @@ public class RubyFloat extends RubyNumeric {
         }
 
         // Java Doubles are 64 bit long:            
-        floatc.defineConstant("ROUNDS", RubyFixnum.newFixnum(runtime, 1));
-        floatc.defineConstant("RADIX", RubyFixnum.newFixnum(runtime, 2));
-        floatc.defineConstant("MANT_DIG", RubyFixnum.newFixnum(runtime, 53));
-        floatc.defineConstant("DIG", RubyFixnum.newFixnum(runtime, 15));
+        floatc.defineConstant("ROUNDS", RubyFixnum.newFixnum(runtime, ROUNDS));
+        floatc.defineConstant("RADIX", RubyFixnum.newFixnum(runtime, RADIX));
+        floatc.defineConstant("MANT_DIG", RubyFixnum.newFixnum(runtime, MANT_DIG));
+        floatc.defineConstant("DIG", RubyFixnum.newFixnum(runtime, DIG));
         // Double.MAX_EXPONENT since Java 1.6
-        floatc.defineConstant("MIN_EXP", RubyFixnum.newFixnum(runtime, -1021));
+        floatc.defineConstant("MIN_EXP", RubyFixnum.newFixnum(runtime, MIN_EXP));
         // Double.MAX_EXPONENT since Java 1.6            
-        floatc.defineConstant("MAX_EXP", RubyFixnum.newFixnum(runtime, 1024));
-        floatc.defineConstant("MIN_10_EXP", RubyFixnum.newFixnum(runtime, -307));
-        floatc.defineConstant("MAX_10_EXP", RubyFixnum.newFixnum(runtime, 308));
+        floatc.defineConstant("MAX_EXP", RubyFixnum.newFixnum(runtime, MAX_EXP));
+        floatc.defineConstant("MIN_10_EXP", RubyFixnum.newFixnum(runtime, MIN_10_EXP));
+        floatc.defineConstant("MAX_10_EXP", RubyFixnum.newFixnum(runtime, MAX_10_EXP));
         floatc.defineConstant("MIN", RubyFloat.newFloat(runtime, Double.MIN_VALUE));
         floatc.defineConstant("MAX", RubyFloat.newFloat(runtime, Double.MAX_VALUE));
-        floatc.defineConstant("EPSILON", RubyFloat.newFloat(runtime, 2.2204460492503131e-16));
+        floatc.defineConstant("EPSILON", RubyFloat.newFloat(runtime, EPSILON));
         
         floatc.defineAnnotatedMethods(RubyFloat.class);
 
@@ -102,6 +114,7 @@ public class RubyFloat extends RubyNumeric {
 
     private final double value;
     
+    @Override
     public int getNativeTypeIndex() {
         return ClassIndex.FLOAT;
     }
@@ -115,6 +128,7 @@ public class RubyFloat extends RubyNumeric {
         this.value = value;
     }
 
+    @Override
     public Class<?> getJavaClass() {
         // this needs to be thought out more along with the changes in RubyFixnum
         // since "to Object" coercion will generally want to produce the same
@@ -132,14 +146,22 @@ public class RubyFloat extends RubyNumeric {
         return this.value;
     }
 
+    @Override
     public double getDoubleValue() {
         return value;
     }
 
+    @Override
     public long getLongValue() {
         return (long) value;
     }
+
+    @Override
+    public BigInteger getBigIntegerValue() {
+        return BigInteger.valueOf((long)value);
+    }
     
+    @Override
     public RubyFloat convertToFloat() {
     	return this;
     }
@@ -179,6 +201,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "to_s")
+    @Override
     public IRubyObject to_s() {
         Ruby runtime = getRuntime();
         if (Double.isInfinite(value)) return RubyString.newString(runtime, value < 0 ? "-Infinity" : "Infinity");
@@ -208,6 +231,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "coerce", required = 1)
+    @Override
     public IRubyObject coerce(IRubyObject other) {
         return getRuntime().newArray(RubyKernel.new_float(this, other), this);
     }
@@ -317,6 +341,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "divmod", required = 1)
+    @Override
     public IRubyObject divmod(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().index) {
         case ClassIndex.FIXNUM:
@@ -364,6 +389,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "==", required = 1)
+    @Override
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
         if (Double.isNaN(value)) {
             return getRuntime().getFalse();
@@ -400,6 +426,9 @@ public class RubyFloat extends RubyNumeric {
         switch (other.getMetaClass().index) {
         case ClassIndex.FIXNUM:
         case ClassIndex.BIGNUM:
+            if (Double.isInfinite(value)) {
+                return value > 0.0 ? RubyFixnum.one(getRuntime()) : RubyFixnum.minus_one(getRuntime());
+            }
         case ClassIndex.FLOAT:
             double b = ((RubyNumeric) other).getDoubleValue();
             return dbl_cmp(getRuntime(), value, b);
@@ -476,6 +505,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "eql?", required = 1)
+    @Override
     public IRubyObject eql_p(IRubyObject other) {
         if (other instanceof RubyFloat) {
             double b = ((RubyFloat) other).value;
@@ -493,10 +523,12 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "hash")
+    @Override
     public RubyFixnum hash() {
         return getRuntime().newFixnum(hashCode());
     }
 
+    @Override
     public final int hashCode() {
         long l = Double.doubleToLongBits(value);
         return (int)(l ^ l >>> 32);
@@ -514,6 +546,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "abs")
+    @Override
     public IRubyObject abs(ThreadContext context) {
         if (Double.doubleToLongBits(value) < 0) {
             return RubyFloat.newFloat(context.getRuntime(), Math.abs(value));
@@ -525,6 +558,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "magnitude", compat = CompatVersion.RUBY1_9)
+    @Override
     public IRubyObject magnitude(ThreadContext context) {
         return abs(context);
     }
@@ -541,6 +575,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = {"truncate", "to_i", "to_int"})
+    @Override
     public IRubyObject truncate() {
         double f = value;
         if (f > 0.0) f = Math.floor(f);
@@ -553,6 +588,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "numerator", compat = CompatVersion.RUBY1_9)
+    @Override
     public IRubyObject numerator(ThreadContext context) {
         if (Double.isInfinite(value) || Double.isNaN(value)) return this;
         return super.numerator(context);
@@ -562,6 +598,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "denominator", compat = CompatVersion.RUBY1_9)
+    @Override
     public IRubyObject denominator(ThreadContext context) {
         if (Double.isInfinite(value) || Double.isNaN(value)) return RubyFixnum.one(context.getRuntime());
         return super.denominator(context);
@@ -590,6 +627,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "floor")
+    @Override
     public IRubyObject floor() {
         return dbl2num(getRuntime(), Math.floor(value));
     }
@@ -598,6 +636,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "ceil")
+    @Override
     public IRubyObject ceil() {
         return dbl2num(getRuntime(), Math.ceil(value));
     }
@@ -606,6 +645,7 @@ public class RubyFloat extends RubyNumeric {
      * 
      */
     @JRubyMethod(name = "round")
+    @Override
     public IRubyObject round() {
         double f = value;
         if (f > 0.0) {
@@ -647,25 +687,20 @@ public class RubyFloat extends RubyNumeric {
         return getRuntime().getTrue();
     }
 
+    // CRuby uses sprintf(buf, "%.*g", FLOAT_DIG, d);
+    // This pattern adjusts the output of String.pattern("%g") to mimic
+    // the C version.
+    private static final Pattern pattern = Pattern.compile("\\.?0+(e|$)");
+
+    private static String formatDouble(double x) {
+        return pattern.matcher(String.format("%.32g", x)).replaceFirst("$1");
+    }
+
     private String marshalDump() {
         if (Double.isInfinite(value)) return value < 0 ? "-inf" : "inf";
         if (Double.isNaN(value)) return "nan";
 
-        String val = String.valueOf(value);
-
-        if(val.indexOf('E') != -1) {
-            String v2 = FORMAT.format(value);
-            int ix = v2.length()-1;
-            while(v2.charAt(ix) == '0' && v2.charAt(ix-1) != '.') {
-                ix--;
-            }
-            if(ix > 16 || (v2.charAt(0) != '-' && ix > 15) || "0.0".equals(v2.substring(0,ix+1))) {
-                val = val.replaceFirst("E(\\d)", "e+$1").replaceFirst("E-","e-");
-            } else {
-                val = v2.substring(0,ix+1);
-            }
-        }
-        return val;
+        return formatDouble(value);
     }
 
     public static void marshalTo(RubyFloat aFloat, MarshalStream output) throws java.io.IOException {
