@@ -46,6 +46,7 @@ import org.jruby.environment.OSEnvironment;
 import org.jruby.internal.runtime.ValueAccessor;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.javasupport.util.RuntimeHelpers;
+import org.jruby.platform.Platform;
 import org.jruby.runtime.Constants;
 import org.jruby.runtime.GlobalVariable;
 import org.jruby.runtime.IAccessor;
@@ -80,7 +81,8 @@ public class RubyGlobal {
 
         @Override
         public IRubyObject op_aref(ThreadContext context, IRubyObject key) {
-            return super.op_aref(context, key.convertToString());
+            RubyString actualKey = getCorrectKey(key, context);
+            return super.op_aref(context, actualKey);
         }
 
         @Override
@@ -91,13 +93,15 @@ public class RubyGlobal {
             if (!value.respondsTo("to_str") && !value.isNil()) {
                 throw getRuntime().newTypeError("can't convert " + value.getMetaClass() + " into String");
             }
+            
+            RubyString actualKey = getCorrectKey(key, context);
 
             if (value.isNil()) {
-                return super.delete(context, key, org.jruby.runtime.Block.NULL_BLOCK);
+                return super.delete(context, actualKey, org.jruby.runtime.Block.NULL_BLOCK);
             }
             
             //return super.aset(getRuntime().newString("sadfasdF"), getRuntime().newString("sadfasdF"));
-            return super.op_aset(context, RuntimeHelpers.invoke(context, key, "to_str"),
+            return super.op_aset(context, RuntimeHelpers.invoke(context, actualKey, "to_str"),
                     value.isNil() ? getRuntime().getNil() : RuntimeHelpers.invoke(context, value, "to_str"));
         }
         
@@ -106,6 +110,25 @@ public class RubyGlobal {
         public IRubyObject to_s(){
             return getRuntime().newString("ENV");
         }
+
+        private RubyString getCorrectKey(IRubyObject key, ThreadContext context) {
+            RubyString originalKey = key.convertToString();
+            RubyString actualKey = originalKey;
+            Ruby runtime = context.getRuntime();
+            if (Platform.IS_WINDOWS) {
+                // this is a rather ugly hack, but similar to MRI. See hash.c:ruby_setenv and similar in MRI
+                // we search all keys for a case-insensitive match, and use that
+                RubyArray keys = super.keys();
+                for (int i = 0; i < keys.size(); i++) {
+                    RubyString candidateKey = keys.eltInternal(i).convertToString();
+                    if (candidateKey.casecmp(context, originalKey).op_equal(context, RubyFixnum.zero(runtime)).isTrue()) {
+                        actualKey = candidateKey;
+                        break;
+                    }
+                }
+            }
+            return actualKey;
+        }
     }
     
     public static void createGlobals(ThreadContext context, Ruby runtime) {
@@ -139,11 +162,11 @@ public class RubyGlobal {
         switch (runtime.getInstanceConfig().getCompatVersion()) {
         case RUBY1_8:
             version = runtime.newString(Constants.RUBY_VERSION).freeze(context);
-            patchlevel = runtime.newString(Constants.RUBY_PATCHLEVEL).freeze(context);
+            patchlevel = runtime.newFixnum(Constants.RUBY_PATCHLEVEL).freeze(context);
             break;
         case RUBY1_9:
             version = runtime.newString(Constants.RUBY1_9_VERSION).freeze(context);
-            patchlevel = runtime.newString(Constants.RUBY1_9_PATCHLEVEL).freeze(context);
+            patchlevel = runtime.newFixnum(Constants.RUBY1_9_PATCHLEVEL).freeze(context);
             break;
         }
         runtime.defineGlobalConstant("RUBY_VERSION", version);
@@ -152,6 +175,12 @@ public class RubyGlobal {
         runtime.defineGlobalConstant("RUBY_PLATFORM", platform);
         runtime.defineGlobalConstant("RUBY_ENGINE", engine);
 
+        IRubyObject description = runtime.newString(runtime.getInstanceConfig().getVersionString()).freeze(context);
+        runtime.defineGlobalConstant("RUBY_DESCRIPTION", description);
+
+        IRubyObject copyright = runtime.newString(runtime.getInstanceConfig().getCopyrightString()).freeze(context);
+        runtime.defineGlobalConstant("RUBY_COPYRIGHT", copyright);
+
         runtime.defineGlobalConstant("VERSION", version);
         runtime.defineGlobalConstant("RELEASE_DATE", release);
         runtime.defineGlobalConstant("PLATFORM", platform);
@@ -323,7 +352,7 @@ public class RubyGlobal {
         
         @Override
         public IRubyObject get() {
-            return RubyRegexp.last_match(runtime.getCurrentContext().getCurrentFrame().getBackRef());
+            return RubyRegexp.last_match(runtime.getCurrentContext().getCurrentScope().getBackRef(runtime));
         }
     }
 
@@ -334,7 +363,7 @@ public class RubyGlobal {
         
         @Override
         public IRubyObject get() {
-            return RubyRegexp.match_pre(runtime.getCurrentContext().getCurrentFrame().getBackRef());
+            return RubyRegexp.match_pre(runtime.getCurrentContext().getCurrentScope().getBackRef(runtime));
         }
     }
 
@@ -345,7 +374,7 @@ public class RubyGlobal {
         
         @Override
         public IRubyObject get() {
-            return RubyRegexp.match_post(runtime.getCurrentContext().getCurrentFrame().getBackRef());
+            return RubyRegexp.match_post(runtime.getCurrentContext().getCurrentScope().getBackRef(runtime));
         }
     }
 
@@ -356,7 +385,7 @@ public class RubyGlobal {
         
         @Override
         public IRubyObject get() {
-            return RubyRegexp.match_last(runtime.getCurrentContext().getCurrentFrame().getBackRef());
+            return RubyRegexp.match_last(runtime.getCurrentContext().getCurrentScope().getBackRef(runtime));
         }
     }
 
@@ -401,7 +430,7 @@ public class RubyGlobal {
         public IRubyObject set(IRubyObject value) {
             if (!value.isNil() &&
                     !runtime.getException().isInstance(value) &&
-                    !(JavaUtil.isJavaObject(value) && JavaUtil.unwrapJavaObject(value) instanceof Exception)) {
+                    !(JavaUtil.isJavaObject(value) && JavaUtil.unwrapJavaObject(value) instanceof Throwable)) {
                 throw runtime.newTypeError("assigning non-exception to $!");
             }
             
@@ -559,7 +588,7 @@ public class RubyGlobal {
         }
     }
 
-    private static class InputGlobalVariable extends GlobalVariable {
+    public static class InputGlobalVariable extends GlobalVariable {
         public InputGlobalVariable(Ruby runtime, String name, IRubyObject value) {
             super(runtime, name, value);
         }
@@ -574,7 +603,7 @@ public class RubyGlobal {
         }
     }
 
-    private static class OutputGlobalVariable extends GlobalVariable {
+    public static class OutputGlobalVariable extends GlobalVariable {
         public OutputGlobalVariable(Ruby runtime, String name, IRubyObject value) {
             super(runtime, name, value);
         }
