@@ -59,6 +59,8 @@ import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.internal.runtime.JumpTarget;
+import org.jruby.internal.runtime.methods.CallConfiguration;
+import org.jruby.internal.runtime.methods.JavaMethod.JavaMethodNBlock;
 import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Binding;
 import org.jruby.runtime.Block;
@@ -81,6 +83,24 @@ import org.jruby.util.TypeConverter;
 public class RubyKernel {
     public final static Class<?> IRUBY_OBJECT = IRubyObject.class;
 
+    public static abstract class MethodMissingMethod extends JavaMethodNBlock {
+        public MethodMissingMethod(RubyModule implementationClass) {
+            super(implementationClass, Visibility.PRIVATE, CallConfiguration.FrameFullScopeNone);
+        }
+
+        @Override
+        public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+            try {
+                preFrameOnly(context, self, name, block);
+                return methodMissing(context, self, clazz, name, args, block);
+            } finally {
+                postFrameOnly(context);
+            }
+        }
+
+        public abstract IRubyObject methodMissing(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block);
+
+    }
     public static RubyModule createKernelModule(Ruby runtime) {
         RubyModule module = runtime.defineModule("Kernel");
         runtime.setKernel(module);
@@ -92,6 +112,43 @@ public class RubyKernel {
         
         module.setFlag(RubyObject.USER7_F, false); //Kernel is the only Module that doesn't need an implementor
 
+        runtime.setPrivateMethodMissing(new MethodMissingMethod(module) {
+            @Override
+            public IRubyObject methodMissing(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+                return RubyKernel.methodMissing(context, self, name, PRIVATE, CallType.NORMAL, args, block);
+            }
+        });
+
+        runtime.setProtectedMethodMissing(new MethodMissingMethod(module) {
+            @Override
+            public IRubyObject methodMissing(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+                return RubyKernel.methodMissing(context, self, name, PROTECTED, CallType.NORMAL, args, block);
+            }
+        });
+
+        runtime.setVariableMethodMissing(new MethodMissingMethod(module) {
+            @Override
+            public IRubyObject methodMissing(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+                return RubyKernel.methodMissing(context, self, name, PUBLIC, CallType.VARIABLE, args, block);
+            }
+        });
+
+        runtime.setSuperMethodMissing(new MethodMissingMethod(module) {
+            @Override
+            public IRubyObject methodMissing(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+                return RubyKernel.methodMissing(context, self, name, PUBLIC, CallType.SUPER, args, block);
+            }
+        });
+
+        runtime.setNormalMethodMissing(new MethodMissingMethod(module) {
+            @Override
+            public IRubyObject methodMissing(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+                return RubyKernel.methodMissing(context, self, name, PUBLIC, CallType.NORMAL, args, block);
+            }
+        });
+
+        runtime.setDefaultMethodMissing(module.searchMethod("method_missing"));
+
         return module;
     }
 
@@ -159,29 +216,57 @@ public class RubyKernel {
 
     @JRubyMethod(name = "method_missing", rest = true, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject method_missing(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
-        Ruby runtime = context.getRuntime();
-
-        if (args.length == 0 || !(args[0] instanceof RubySymbol)) throw runtime.newArgumentError("no id given");
-
         Visibility lastVis = context.getLastVisibility();
         CallType lastCallType = context.getLastCallType();
 
+        if (args.length == 0 || !(args[0] instanceof RubySymbol)) throw context.getRuntime().newArgumentError("no id given");
+
+        return methodMissingDirect(context, recv, (RubySymbol)args[0], lastVis, lastCallType, args, block);
+    }
+
+    private static IRubyObject methodMissingDirect(ThreadContext context, IRubyObject recv, RubySymbol symbol, Visibility lastVis, CallType lastCallType, IRubyObject[] args, Block block) {
+        Ruby runtime = context.getRuntime();
+        
         // create a lightweight thunk
-        IRubyObject msg = new RubyNameError.RubyNameErrorMessage(runtime, 
+        IRubyObject msg = new RubyNameError.RubyNameErrorMessage(runtime,
                                                                  recv,
-                                                                 args[0],
+                                                                 symbol,
                                                                  lastVis,
                                                                  lastCallType);
         final IRubyObject[]exArgs;
         final RubyClass exc;
         if (lastCallType != CallType.VARIABLE) {
             exc = runtime.getNoMethodError();
-            exArgs = new IRubyObject[]{msg, args[0], RubyArray.newArrayNoCopy(runtime, args, 1)};
+            exArgs = new IRubyObject[]{msg, symbol, RubyArray.newArrayNoCopy(runtime, args, 1)};
         } else {
             exc = runtime.getNameError();
-            exArgs = new IRubyObject[]{msg, args[0]};
+            exArgs = new IRubyObject[]{msg, symbol};
         }
-        
+
+        throw new RaiseException((RubyException)exc.newInstance(context, exArgs, Block.NULL_BLOCK));
+    }
+
+    private static IRubyObject methodMissing(ThreadContext context, IRubyObject recv, String name, Visibility lastVis, CallType lastCallType, IRubyObject[] args, Block block) {
+        Ruby runtime = context.getRuntime();
+        // TODO: pass this in?
+        RubySymbol symbol = runtime.newSymbol(name);
+
+        // create a lightweight thunk
+        IRubyObject msg = new RubyNameError.RubyNameErrorMessage(runtime,
+                                                                 recv,
+                                                                 symbol,
+                                                                 lastVis,
+                                                                 lastCallType);
+        final IRubyObject[]exArgs;
+        final RubyClass exc;
+        if (lastCallType != CallType.VARIABLE) {
+            exc = runtime.getNoMethodError();
+            exArgs = new IRubyObject[]{msg, symbol, RubyArray.newArrayNoCopy(runtime, args)};
+        } else {
+            exc = runtime.getNameError();
+            exArgs = new IRubyObject[]{msg, symbol};
+        }
+
         throw new RaiseException((RubyException)exc.newInstance(context, exArgs, Block.NULL_BLOCK));
     }
 
@@ -339,25 +424,22 @@ public class RubyKernel {
     @JRubyMethod(name = "putc", required = 1, module = true, visibility = PRIVATE)
     public static IRubyObject putc(ThreadContext context, IRubyObject recv, IRubyObject ch) {
         IRubyObject defout = context.getRuntime().getGlobalVariables().get("$>");
-        return defout.callMethod(context, "putc", ch);
+        
+        return RubyIO.putc(context, defout, ch);
     }
 
     @JRubyMethod(name = "puts", rest = true, module = true, visibility = PRIVATE)
     public static IRubyObject puts(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         IRubyObject defout = context.getRuntime().getGlobalVariables().get("$>");
-        
-        defout.callMethod(context, "puts", args);
 
-        return context.getRuntime().getNil();
+        return RubyIO.puts(context, defout, args);
     }
 
     @JRubyMethod(name = "print", rest = true, module = true, visibility = PRIVATE)
     public static IRubyObject print(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         IRubyObject defout = context.getRuntime().getGlobalVariables().get("$>");
 
-        defout.callMethod(context, "print", args);
-
-        return context.getRuntime().getNil();
+        return RubyIO.print(context, defout, args);
     }
 
     @JRubyMethod(name = "printf", rest = true, module = true, visibility = PRIVATE)
@@ -396,7 +478,7 @@ public class RubyKernel {
      * @return value of $_ as String.
      */
     private static RubyString getLastlineString(ThreadContext context, Ruby runtime) {
-        IRubyObject line = context.getPreviousFrame().getLastLine();
+        IRubyObject line = context.getCurrentScope().getLastLine(runtime);
 
         if (line.isNil()) {
             throw runtime.newTypeError("$_ value need to be String (nil given).");
@@ -433,7 +515,7 @@ public class RubyKernel {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
         if (!str.sub_bang(context, args, block).isNil()) {
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -444,7 +526,7 @@ public class RubyKernel {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
         if (!str.sub_bang(context, arg0, block).isNil()) {
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -455,7 +537,7 @@ public class RubyKernel {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
         if (!str.sub_bang(context, arg0, arg1, block).isNil()) {
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -487,7 +569,7 @@ public class RubyKernel {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
         if (!str.gsub_bang(context, args, block).isNil()) {
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -498,7 +580,7 @@ public class RubyKernel {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
         if (!str.gsub_bang(context, arg0, block).isNil()) {
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -509,7 +591,7 @@ public class RubyKernel {
         RubyString str = (RubyString) getLastlineString(context, context.getRuntime()).dup();
 
         if (!str.gsub_bang(context, arg0, arg1, block).isNil()) {
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -527,7 +609,7 @@ public class RubyKernel {
         if (str.getByteList().realSize > 0) {
             str = (RubyString) str.dup();
             str.chop_bang(context);
-            context.getPreviousFrame().setLastLine(str);
+            context.getCurrentScope().setLastLine(str);
         }
 
         return str;
@@ -563,7 +645,7 @@ public class RubyKernel {
             return str;
         } 
 
-        context.getPreviousFrame().setLastLine(dup);
+        context.getCurrentScope().setLastLine(dup);
         return dup;
     }
 
@@ -576,7 +658,7 @@ public class RubyKernel {
             return str;
         } 
 
-        context.getPreviousFrame().setLastLine(dup);
+        context.getCurrentScope().setLastLine(dup);
         return dup;
     }
 
@@ -589,7 +671,7 @@ public class RubyKernel {
             return str;
         } 
 
-        context.getPreviousFrame().setLastLine(dup);
+        context.getCurrentScope().setLastLine(dup);
         return dup;
     }
 
@@ -816,13 +898,12 @@ public class RubyKernel {
     private static void printExceptionSummary(ThreadContext context, Ruby runtime, RubyException rEx) {
         Frame currentFrame = context.getCurrentFrame();
 
-        String msg = String.format("Exception `%s' at %s:%s - %s\n",
-                rEx.getMetaClass(),
+        String msg = String.format("Exception `%s':%s at %s:%s - %s\n",
+                rEx.getMetaClass(), rEx.convertToString().toString(),
                 currentFrame.getFile(), currentFrame.getLine() + 1,
                 rEx.to_s());
 
-        IRubyObject errorStream = runtime.getGlobalVariables().get("$stderr");
-        errorStream.callMethod(context, "write", runtime.newString(msg));
+        runtime.getErr().print(msg);
     }
 
     /**
@@ -836,7 +917,7 @@ public class RubyKernel {
     public static IRubyObject require(IRubyObject recv, IRubyObject name, Block block) {
         Ruby runtime = recv.getRuntime();
         
-        if (runtime.getLoadService().require(name.convertToString().toString())) {
+        if (runtime.getLoadService().lockAndRequire(name.convertToString().toString())) {
             return runtime.getTrue();
         }
         return runtime.getFalse();
@@ -904,7 +985,6 @@ public class RubyKernel {
         return continuation.enter(context, block);
     }
 
-    @JRubyMethod(name = "caller", optional = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static IRubyObject caller(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         int level = args.length > 0 ? RubyNumeric.fix2int(args[0]) : 1;
 
@@ -915,7 +995,7 @@ public class RubyKernel {
         return context.createCallerBacktrace(context.getRuntime(), level);
     }
 
-    @JRubyMethod(name = "caller", optional = 1, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = "caller", optional = 1, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject caller1_9(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
         int level = args.length > 0 ? RubyNumeric.fix2int(args[0]) : 1;
 
@@ -992,6 +1072,11 @@ public class RubyKernel {
 
     @JRubyMethod(name = "set_trace_func", required = 1, frame = true, module = true, visibility = PRIVATE)
     public static IRubyObject set_trace_func(ThreadContext context, IRubyObject recv, IRubyObject trace_func, Block block) {
+        if (!RubyInstanceConfig.FULL_TRACE_ENABLED) {
+            // without full tracing, many events will not fire
+            context.getRuntime().getWarnings().warn("set_trace_func will not capture all events without --debug flag");
+        }
+        
         if (trace_func.isNil()) {
             context.getRuntime().setTraceFunction(null);
         } else if (!(trace_func instanceof RubyProc)) {
@@ -1056,7 +1141,18 @@ public class RubyKernel {
     public static IRubyObject singleton_method_undefined(ThreadContext context, IRubyObject recv, IRubyObject symbolId, Block block) {
         return context.getRuntime().getNil();
     }
-    
+
+    @JRubyMethod(name = "define_singleton_method", required = 1, optional = 1, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_9)
+    public static IRubyObject define_singleton_method(ThreadContext context, IRubyObject recv, IRubyObject[] args, Block block) {
+        if (args.length == 0) throw context.getRuntime().newArgumentError(0, 1);
+
+        RubyClass singleton_class = recv.getSingletonClass();
+        IRubyObject obj = args.length > 1 ?
+            singleton_class.define_method(context, args[0], args[1], block) :
+            singleton_class.define_method(context, args[0], block);
+        return obj;
+    }
+
     @JRubyMethod(name = {"proc", "lambda"}, frame = true, module = true, visibility = PRIVATE, compat = CompatVersion.RUBY1_8)
     public static RubyProc proc(ThreadContext context, IRubyObject recv, Block block) {
         return context.getRuntime().newProc(Block.Type.LAMBDA, block);
@@ -1077,16 +1173,24 @@ public class RubyKernel {
         return context.getRuntime().newProc(Block.Type.PROC, block);
     }
 
-    @JRubyMethod(name = "loop", frame = true, module = true, visibility = PRIVATE)
-    public static IRubyObject loop(ThreadContext context, IRubyObject recv, Block block) {
+    @JRubyMethod(name = {"loop"}, frame = true, module = true, visibility = PRIVATE)
+    public static IRubyObject loop_1_9(ThreadContext context, IRubyObject recv, Block block) {
         IRubyObject nil = context.getRuntime().getNil();
-        while (true) {
-            block.yield(context, nil);
+        RubyClass stopIteration = context.getRuntime().getStopIteration();
+        try {
+            while (true) {
+                block.yield(context, nil);
 
-            context.pollThreadEvents();
+                context.pollThreadEvents();
+            }
+        } catch (RaiseException ex) {
+            if (!stopIteration.op_eqq(context, ex.getException()).isTrue()) {
+                throw ex;
+            }
         }
+        return nil;
     }
-    
+
     @JRubyMethod(name = "test", required = 2, optional = 1, module = true, visibility = PRIVATE)
     public static IRubyObject test(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         if (args.length == 0) throw context.getRuntime().newArgumentError("wrong number of arguments");
@@ -1345,7 +1449,7 @@ public class RubyKernel {
         }
     }
 
-    @JRubyMethod(frame = true, module = true, compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(frame = true, module = true)
     public static IRubyObject tap(ThreadContext context, IRubyObject recv, Block block) {
         block.yield(context, recv);
         return recv;
@@ -1365,7 +1469,7 @@ public class RubyKernel {
         }
     }
 
-    @JRubyMethod(name = { "__method__", "__callee__" }, module = true, compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = { "__method__", "__callee__" }, module = true, visibility = PRIVATE)
     public static IRubyObject __method__(ThreadContext context, IRubyObject recv) {
         Frame f = context.getCurrentFrame();
         String name = f != null ? f.getName() : null;
