@@ -55,6 +55,8 @@ import org.jruby.runtime.CallBlock;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ObjectMarshal;
 import org.jruby.runtime.ThreadContext;
+import org.jruby.runtime.Visibility;
+
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.runtime.builtin.Variable;
 import org.jruby.runtime.component.VariableEntry;
@@ -84,10 +86,6 @@ public class RubyRange extends RubyObject {
         result.setMarshal(RANGE_MARSHAL);
         result.includeModule(runtime.getEnumerable());
 
-        // We override Enumerable#member? since ranges in 1.8.1 are continuous.
-        //        result.defineMethod("member?", callbackFactory.getMethod("include_p", RubyKernel.IRUBY_OBJECT));
-        //        result.defineMethod("===", callbackFactory.getMethod("include_p", RubyKernel.IRUBY_OBJECT));
-
         result.defineAnnotatedMethods(RubyRange.class);
         return result;
     }
@@ -200,8 +198,11 @@ public class RubyRange extends RubyObject {
         this.isExclusive = isExclusive;
     }
     
-    @JRubyMethod(name = "initialize", required = 2, optional = 1, frame = true)
+    @JRubyMethod(name = "initialize", required = 2, optional = 1, frame = true, visibility = Visibility.PRIVATE)
     public IRubyObject initialize(ThreadContext context, IRubyObject[] args, Block unusedBlock) {
+        if (!begin.isNil() || !end.isNil()) {
+            throw getRuntime().newNameError("`initialize' called twice", "initialize");
+        }
         init(context, args[0], args[1], args.length > 2 && args[2].isTrue());
         return getRuntime().getNil();
     }
@@ -363,24 +364,26 @@ public class RubyRange extends RubyObject {
             if (size > Integer.MAX_VALUE) {
                 throw runtime.newRangeError("Range size too large for to_a");
             }
+            if (size < 0) return RubyArray.newEmptyArray(runtime);
             IRubyObject[] array = new IRubyObject[(int)size];
             for (int i = 0; i < size; i++) {
                 array[i] = RubyFixnum.newFixnum(runtime, base + i);
             }
             return RubyArray.newArrayNoCopy(runtime, array);
         } else {
-            return RubyEnumerable.to_a(context, this);
+            return RubyEnumerable.to_a19(context, this);
         }
     }
 
     @JRubyMethod(name = "each", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject each(ThreadContext context, final Block block) {
         final Ruby runtime = context.getRuntime();
+        if (!block.isGiven()) return enumeratorize(runtime, this, "each");
 
         if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
             fixnumEach(context, runtime, block);
         } else if (begin instanceof RubyString) {
-            ((RubyString) begin).uptoCommon(context, end, isExclusive, block);
+            ((RubyString) begin).uptoCommon18(context, end, isExclusive, block);
         } else {
             if (!begin.respondsTo("succ")) throw getRuntime().newTypeError(
                     "can't iterate from " + begin.getMetaClass().getName());
@@ -415,14 +418,13 @@ public class RubyRange extends RubyObject {
         Ruby runtime = context.getRuntime();
         if (!block.isGiven()) return enumeratorize(runtime, this, "each");
 
-        if (!begin.respondsTo("succ")) throw getRuntime().newTypeError(
-                "can't iterate from " + begin.getMetaClass().getName());
-
         if (begin instanceof RubyFixnum && end instanceof RubyFixnum) {
             fixnumEach(context, runtime, block);
         } else if (begin instanceof RubyString) {
             ((RubyString) begin).uptoCommon19(context, end, isExclusive, block);
         } else {
+            if (!begin.respondsTo("succ")) throw getRuntime().newTypeError(
+                    "can't iterate from " + begin.getMetaClass().getName());
             rangeEach(context, new RangeCallBack() {
                 @Override
                 void call(ThreadContext context, IRubyObject arg) {
@@ -435,12 +437,12 @@ public class RubyRange extends RubyObject {
 
     @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject step(ThreadContext context, IRubyObject step, Block block) {
-        return stepCommon(context, step, block);
+        return block.isGiven() ? stepCommon(context, step, block) : enumeratorize(context.getRuntime(), this, "step", step);
     }
 
     @JRubyMethod(name = "step", frame = true, compat = CompatVersion.RUBY1_8)
     public IRubyObject step(ThreadContext context, Block block) {
-        return stepCommon(context, RubyFixnum.one(context.getRuntime()), block);
+        return block.isGiven() ? stepCommon(context, RubyFixnum.one(context.getRuntime()), block)  : enumeratorize(context.getRuntime(), this, "step");
     }
 
     private IRubyObject stepCommon(ThreadContext context, IRubyObject step, Block block) {
@@ -458,7 +460,7 @@ public class RubyRange extends RubyObject {
                 // rb_iterate((VALUE(*)_((VALUE)))str_step, (VALUE)args, step_i, (VALUE)iter);
                 StepBlockCallBack callback = new StepBlockCallBack(block, RubyFixnum.one(runtime), step);
                 Block blockCallback = CallBlock.newCallClosure(this, runtime.getRange(), Arity.singleArgument(), callback, context);
-                ((RubyString)tmp).uptoCommon(context, end, isExclusive, blockCallback);
+                ((RubyString)tmp).uptoCommon18(context, end, isExclusive, blockCallback);
             } else if (begin instanceof RubyNumeric) {
                 if (equalInternal(context, step, RubyFixnum.zero(runtime))) throw runtime.newArgumentError("step can't be 0");
                 numericStep(context, runtime, step, block);
@@ -666,11 +668,11 @@ public class RubyRange extends RubyObject {
             RubyRange range = (RubyRange)obj;
 
             marshalStream.registerLinkTarget(range);
-            List<Variable<IRubyObject>> attrs = range.getVariableList();
+            List<Variable<Object>> attrs = range.getVariableList();
 
-            attrs.add(new VariableEntry<IRubyObject>("begin", range.begin));
-            attrs.add(new VariableEntry<IRubyObject>("end", range.end));
-            attrs.add(new VariableEntry<IRubyObject>("excl", range.isExclusive ? runtime.getTrue() : runtime.getFalse()));
+            attrs.add(new VariableEntry<Object>("begin", range.begin));
+            attrs.add(new VariableEntry<Object>("end", range.end));
+            attrs.add(new VariableEntry<Object>("excl", range.isExclusive ? runtime.getTrue() : runtime.getFalse()));
 
             marshalStream.dumpVariables(attrs);
         }
