@@ -57,11 +57,12 @@ import org.joni.exception.JOniException;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.common.IRubyWarnings.ID;
+import org.jruby.exceptions.RaiseException;
 import org.jruby.parser.ReOptions;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
-import org.jruby.runtime.Frame;
+import org.jruby.runtime.DynamicScope;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
@@ -260,7 +261,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
 
     private RubyRegexp(Ruby runtime, ByteList str, int options) {
         this(runtime);
-        setKCode(runtime, options);
+        setKCode(runtime, options & 0x7f); // mask off "once" flag
         this.str = str;
         this.pattern = getRegexpFromCache(runtime, str, kcode.getEncoding(), options & 0xf);
     }
@@ -272,7 +273,20 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
 
     // used only by the compiler/interpreter (will set the literal flag)
     public static RubyRegexp newRegexp(Ruby runtime, ByteList pattern, int options) {
-        return new RubyRegexp(runtime, pattern, options);
+        try {
+            return new RubyRegexp(runtime, pattern, options);
+        } catch (RaiseException re) {
+            throw runtime.newSyntaxError(re.getMessage());
+        }
+    }
+
+    // used only by the compiler/interpreter (will set the literal flag)
+    public static RubyRegexp newDRegexp(Ruby runtime, RubyString pattern, int options) {
+        try {
+            return new RubyRegexp(runtime, pattern.getByteList(), options);
+        } catch (RaiseException re) {
+            throw runtime.newRegexpError(re.getMessage());
+        }
     }
 
     public static RubyRegexp newRegexp(Ruby runtime, ByteList pattern) {
@@ -883,7 +897,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     */
     @JRubyMethod(name = "last_match", meta = true, reads = BACKREF)
     public static IRubyObject last_match_s(ThreadContext context, IRubyObject recv) {
-        IRubyObject match = context.getCurrentFrame().getBackRef();
+        IRubyObject match = context.getCurrentScope().getBackRef(context.getRuntime());
         if (match instanceof RubyMatchData) ((RubyMatchData)match).use();
         return match;
     }
@@ -893,7 +907,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     */
     @JRubyMethod(name = "last_match", meta = true, reads = BACKREF)
     public static IRubyObject last_match_s(ThreadContext context, IRubyObject recv, IRubyObject nth) {
-        IRubyObject match = context.getCurrentFrame().getBackRef();
+        IRubyObject match = context.getCurrentScope().getBackRef(context.getRuntime());
         if (match.isNil()) return match;
         return nth_match(((RubyMatchData)match).backrefNumber(nth), match);
     }
@@ -904,6 +918,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     @JRubyMethod(name = "union", rest = true, meta = true)
     public static IRubyObject union(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = context.getRuntime();
+        IRubyObject[] realArgs = args;
         if (args.length == 0) {
             return newRegexp(runtime, ByteList.create("(?!)"), 0);
         } else if (args.length == 1) {
@@ -911,51 +926,61 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
             if (!v.isNil()) {
                 return v;
             } else {
-                // newInstance here
-                return newRegexp(runtime, quote(context, recv, args).getByteList(), 0);
-            }
-        } else {
-            KCode kcode = null;
-            IRubyObject kcode_re = runtime.getNil();
-            RubyString source = runtime.newString();
-            IRubyObject[] _args = new IRubyObject[3];
-
-            for (int i = 0; i < args.length; i++) {
-                if (0 < i) source.cat((byte)'|');
-                IRubyObject v = TypeConverter.convertToTypeWithCheck(args[i], runtime.getRegexp(), "to_regexp");
-                if (!v.isNil()) {
-                    if (!((RubyRegexp)v).isKCodeDefault()) {
-                        if (kcode == null) {
-                            kcode_re = v;
-                            kcode = ((RubyRegexp)v).kcode;
-                        } else if (((RubyRegexp)v).kcode != kcode) {
-                            IRubyObject str1 = kcode_re.inspect();
-                            IRubyObject str2 = v.inspect();
-                            throw runtime.newArgumentError("mixed kcode " + str1 + " and " + str2);
-                        }
+                IRubyObject a = TypeConverter.convertToTypeWithCheck(args[0], runtime.getArray(), "to_ary");
+                if (!a.isNil()) {
+                    RubyArray aa = (RubyArray)a;
+                    int len = aa.getLength();
+                    realArgs = new IRubyObject[len];
+                    for(int i = 0; i<len; i++) {
+                        realArgs[i] = aa.entry(i);
                     }
-                    v = ((RubyRegexp)v).to_s();
                 } else {
-                    v = quote(context, recv, new IRubyObject[]{args[i]});
+                    // newInstance here
+                    return newRegexp(runtime, quote(context, recv, args).getByteList(), 0);
                 }
-                source.append(v);
             }
-
-            _args[0] = source;
-            _args[1] = runtime.getNil();
-            if (kcode == null) {
-                _args[2] = runtime.getNil();
-            } else if (kcode == KCode.NONE) {
-                _args[2] = runtime.newString("n");
-            } else if (kcode == KCode.EUC) {
-                _args[2] = runtime.newString("e");
-            } else if (kcode == KCode.SJIS) {
-                _args[2] = runtime.newString("s");
-            } else if (kcode == KCode.UTF8) {
-                _args[2] = runtime.newString("u");
-            }
-            return recv.callMethod(context, "new", _args);
         }
+
+        KCode kcode = null;
+        IRubyObject kcode_re = runtime.getNil();
+        RubyString source = runtime.newString();
+        IRubyObject[] _args = new IRubyObject[3];
+
+        for (int i = 0; i < realArgs.length; i++) {
+            if (0 < i) source.cat((byte)'|');
+            IRubyObject v = TypeConverter.convertToTypeWithCheck(realArgs[i], runtime.getRegexp(), "to_regexp");
+            if (!v.isNil()) {
+                if (!((RubyRegexp)v).isKCodeDefault()) {
+                    if (kcode == null) {
+                        kcode_re = v;
+                        kcode = ((RubyRegexp)v).kcode;
+                    } else if (((RubyRegexp)v).kcode != kcode) {
+                        IRubyObject str1 = kcode_re.inspect();
+                        IRubyObject str2 = v.inspect();
+                        throw runtime.newArgumentError("mixed kcode " + str1 + " and " + str2);
+                    }
+                }
+                v = ((RubyRegexp)v).to_s();
+            } else {
+                v = quote(context, recv, new IRubyObject[]{realArgs[i]});
+            }
+            source.append(v);
+        }
+
+        _args[0] = source;
+        _args[1] = runtime.getNil();
+        if (kcode == null) {
+            _args[2] = runtime.getNil();
+        } else if (kcode == KCode.NONE) {
+            _args[2] = runtime.newString("n");
+        } else if (kcode == KCode.EUC) {
+            _args[2] = runtime.newString("e");
+        } else if (kcode == KCode.SJIS) {
+            _args[2] = runtime.newString("s");
+        } else if (kcode == KCode.UTF8) {
+            _args[2] = runtime.newString("u");
+        }
+        return recv.callMethod(context, "new", _args);
     }
 
     // rb_reg_raise
@@ -1152,6 +1177,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     // rb_reg_initialize
     private RubyRegexp initializeCommon19(ByteList bytes, Encoding enc, int options) {
         Ruby runtime = getRuntime();        
+        setKCode(runtime, options);
         if (!isTaint() && runtime.getSafeLevel() >= 4) throw runtime.newSecurityError("Insecure: can't modify regexp");
         checkFrozen();
         if (isLiteral()) throw runtime.newSecurityError("can't modify literal regexp");
@@ -1217,13 +1243,13 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     @JRubyMethod(name = "~", reads = {LASTLINE, BACKREF}, writes = BACKREF)
     public IRubyObject op_match2(ThreadContext context) {
         Ruby runtime = context.getRuntime();
-        IRubyObject line = context.getCurrentFrame().getLastLine();
+        IRubyObject line = context.getCurrentScope().getLastLine(runtime);
         if (line instanceof RubyString) {
             int start = search(context, (RubyString)line, 0, false);
             if (start < 0) return runtime.getNil();
             return runtime.newFixnum(start);
         }
-        context.getCurrentFrame().setBackRef(runtime.getNil());
+        context.getCurrentScope().setBackRef(runtime.getNil());
         return runtime.getNil();
     }
 
@@ -1239,7 +1265,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
         } else {
             IRubyObject tmp = arg.checkStringType();
             if (tmp.isNil()) {
-                context.getCurrentFrame().setBackRef(tmp);
+                context.getCurrentScope().setBackRef(tmp);
                 return runtime.getFalse();
             }
             str = (RubyString)tmp;
@@ -1254,7 +1280,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
         Ruby runtime = context.getRuntime();
         arg = operandNoCheck(arg);
         if (arg.isNil()) {
-            context.getCurrentFrame().setBackRef(arg);
+            context.getCurrentScope().setBackRef(arg);
             return runtime.getFalse();
         }
         int start = search19(context, (RubyString)arg, 0, false);
@@ -1264,12 +1290,12 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     /** rb_reg_match
      * 
      */
-    @JRubyMethod(name = "=~", required = 1, reads = BACKREF, writes = BACKREF, compat = CompatVersion.RUBY1_8)
+    @JRubyMethod(name = "=~", required = 1, writes = BACKREF, compat = CompatVersion.RUBY1_8)
     @Override
     public IRubyObject op_match(ThreadContext context, IRubyObject str) {
         Ruby runtime = context.getRuntime();
         if (str.isNil()) {
-            context.getCurrentFrame().setBackRef(str);
+            context.getCurrentScope().setBackRef(str);
             return str;
         }
         int start = search(context, str.convertToString(), 0, false);
@@ -1277,11 +1303,11 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
         return RubyFixnum.newFixnum(runtime, start);
     }
     
-    @JRubyMethod(name = "=~", required = 1, reads = BACKREF, writes = BACKREF, compat = CompatVersion.RUBY1_9)
+    @JRubyMethod(name = "=~", required = 1, writes = BACKREF, compat = CompatVersion.RUBY1_9)
     public IRubyObject op_match19(ThreadContext context, IRubyObject arg) {
         Ruby runtime = context.getRuntime();
         if (arg.isNil()) {
-            context.getCurrentFrame().setBackRef(arg);
+            context.getCurrentScope().setBackRef(arg);
             return arg;
         }
         RubyString str = operandCheck(runtime, arg);
@@ -1297,7 +1323,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     public IRubyObject match_m(ThreadContext context, IRubyObject str) {
         IRubyObject result = op_match(context, str);
         if (result.isNil()) return result;
-        result = context.getCurrentFrame().getBackRef();
+        result = context.getCurrentScope().getBackRef(context.getRuntime());
         ((RubyMatchData)result).use();
         return result;
     }
@@ -1313,20 +1339,20 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     }
 
     private IRubyObject match19Common(ThreadContext context, IRubyObject arg, int pos, Block block) {
-        Frame frame = context.getCurrentFrame();
+        DynamicScope scope = context.getCurrentScope();
         if (arg.isNil()) {
-            frame.setBackRef(arg);
+            scope.setBackRef(arg);
             return arg;
         }
         Ruby runtime = context.getRuntime();
         RubyString str = operandCheck(runtime, arg);
 
         if (matchPos(context, str, pos) < 0) {
-            frame.setBackRef(runtime.getNil());
+            scope.setBackRef(runtime.getNil());
             return runtime.getNil();
         }
 
-        IRubyObject backref = frame.getBackRef();
+        IRubyObject backref = scope.getBackRef(runtime);
         ((RubyMatchData)backref).use();
         if (block.isGiven()) return block.yield(context, backref);
         return backref;
@@ -1347,7 +1373,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
      */
     public final int search(ThreadContext context, RubyString str, int pos, boolean reverse) {
         check();
-        Frame frame = context.getCurrentRubyFrame();
+        DynamicScope scope = context.getCurrentScope();
         ByteList value = str.getByteList();
 
         if (pos <= value.realSize && pos >= 0) {
@@ -1357,34 +1383,38 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
 
             int result = matcher.search(begin + pos, begin + (reverse ? 0 : realSize), Option.NONE);
             if (result >= 0) {
-                updateBackRef(context, str, frame, matcher);
+                updateBackRef(context, str, scope, matcher);
                 return result;
             }
         }
 
-        frame.setBackRef(context.getRuntime().getNil());
+        scope.setBackRef(context.getRuntime().getNil());
         return -1;
     }
 
-    private RubyMatchData updateBackRef(ThreadContext context, RubyString str, Frame frame, Matcher matcher) {
-        RubyMatchData match = updateBackRef(context, str, frame, matcher, pattern);
+    private RubyMatchData updateBackRef(ThreadContext context, RubyString str, DynamicScope scope, Matcher matcher) {
+        RubyMatchData match = updateBackRef(context, str, scope, matcher, pattern);
         match.regexp = this;
         match.infectBy(this);
         return match;
     }
 
-    static final RubyMatchData updateBackRef(ThreadContext context, RubyString str, Frame frame, Matcher matcher, Regex pattern) {
+    static final RubyMatchData updateBackRef(ThreadContext context, RubyString str, DynamicScope scope, Matcher matcher, Regex pattern) {
         Ruby runtime = context.getRuntime();
-        IRubyObject backref = frame.getBackRef();
+        IRubyObject backref = scope.getBackRef(runtime);
         final RubyMatchData match;
+        boolean setBackRef = false;
         if (backref.isNil() || ((RubyMatchData)backref).used()) {
             match = new RubyMatchData(runtime);
-            frame.setBackRef(match);
+            setBackRef = true;
         } else {
             match = (RubyMatchData)backref;
             match.setTaint(runtime.getSafeLevel() >= 3);
         }
 
+        // FIXME: This is pretty gross; we should have a cleaner initialization
+        // that doesn't depend on package-visible fields and ideally is atomic,
+        // probably using an immutable structure we replace all at once.
         match.regs = matcher.getRegion(); // lazy, null when no groups defined
         match.begin = matcher.getBegin();
         match.end = matcher.getEnd();
@@ -1392,12 +1422,16 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
         match.str = (RubyString)str.strDup(runtime).freeze(context);
 
         match.infectBy(str);
+
+        // JRUBY-3625: delay setting backref until the MatchData is completely initialized
+        if (setBackRef) scope.setBackRef(match);
+
         return match;
     }
 
     public final int search19(ThreadContext context, RubyString str, int pos, boolean reverse) {
         check();
-        Frame frame = context.getCurrentRubyFrame();
+        DynamicScope scope = context.getCurrentScope();
         ByteList value = str.getByteList();
 
         if (pos <= value.realSize && pos >= 0) {
@@ -1407,17 +1441,17 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
 
             int result = matcher.search(begin + pos, begin + (reverse ? 0 : realSize), Option.NONE);
             if (result >= 0) {
-                updateBackRef(context, str, frame, matcher).charOffsetUpdated = false;;
+                updateBackRef(context, str, scope, matcher).charOffsetUpdated = false;;
                 return result;
             }
         }
 
-        frame.setBackRef(context.getRuntime().getNil());
+        scope.setBackRef(context.getRuntime().getNil());
         return -1;
     }
 
-    static final RubyMatchData updateBackRef19(ThreadContext context, RubyString str, Frame frame, Matcher matcher, Regex pattern) {
-        RubyMatchData match = updateBackRef(context, str, frame, matcher, pattern);
+    static final RubyMatchData updateBackRef19(ThreadContext context, RubyString str, DynamicScope scope, Matcher matcher, Regex pattern) {
+        RubyMatchData match = updateBackRef(context, str, scope, matcher, pattern);
         match.charOffsetUpdated = false;
         return match;
     }
@@ -1641,7 +1675,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
                     p += n;
                     continue;
                 } else if (c == '/') {
-                    to.append(c);
+                    to.append((byte) '\\');
                     to.append(bytes, p, cl);
                 } else if (!Encoding.isAscii(c)) {
                     int l = StringSupport.length(enc, bytes, p, end);
@@ -1718,24 +1752,22 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     public static IRubyObject nth_match(int nth, IRubyObject match) {
         if (match.isNil()) return match;
         RubyMatchData m = (RubyMatchData)match;
+        Ruby runtime = m.getRuntime();
 
-        int start, end;
-        
+        final int start, end;
         if (m.regs == null) {
-            if (nth >= 1) return match.getRuntime().getNil();
-            if (nth < 0 && ++nth <= 0) return match.getRuntime().getNil();
+            if (nth >= 1 || (nth < 0 && ++nth <= 0)) return runtime.getNil();
             start = m.begin;
             end = m.end;
         } else {
-            if (nth >= m.regs.numRegs) return match.getRuntime().getNil();
-            if (nth < 0 && (nth+=m.regs.numRegs) <= 0) return match.getRuntime().getNil();
+            if (nth >= m.regs.numRegs || (nth < 0 && (nth+=m.regs.numRegs) <= 0)) return runtime.getNil();
             start = m.regs.beg[nth];
             end = m.regs.end[nth];
         }
-        
-        if (start == -1) return match.getRuntime().getNil();
 
-        RubyString str = m.str.makeShared(match.getRuntime(), start, end - start);
+        if (start == -1) return runtime.getNil();
+
+        RubyString str = m.str.makeShared(runtime, start, end - start);
         str.infectBy(m);
         return str;
     }
@@ -1753,8 +1785,9 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     public static IRubyObject match_pre(IRubyObject match) {
         if (match.isNil()) return match;
         RubyMatchData m = (RubyMatchData)match;
-        if (m.begin == -1) match.getRuntime().getNil(); 
-        return m.str.makeShared(match.getRuntime(), 0,  m.begin).infectBy(m);
+        Ruby runtime = m.getRuntime();
+        if (m.begin == -1) runtime.getNil(); 
+        return m.str.makeShared(runtime, 0,  m.begin).infectBy(m);
     }
 
     /** rb_reg_match_post
@@ -1763,8 +1796,9 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     public static IRubyObject match_post(IRubyObject match) {
         if (match.isNil()) return match;
         RubyMatchData m = (RubyMatchData)match;
-        if (m.begin == -1) return match.getRuntime().getNil();
-        return m.str.makeShared(match.getRuntime(), m.end, m.str.getByteList().realSize - m.end).infectBy(m);
+        Ruby runtime = m.getRuntime();
+        if (m.begin == -1) return runtime.getNil();
+        return m.str.makeShared(runtime, m.end, m.str.getByteList().realSize - m.end).infectBy(m);
     }
 
     /** rb_reg_match_last
@@ -2054,7 +2088,7 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     }
 
     public static RubyRegexp unmarshalFrom(UnmarshalStream input) throws java.io.IOException {
-        RubyRegexp result = newRegexp(input.getRuntime(), input.unmarshalString(), input.unmarshalInt());
+        RubyRegexp result = newRegexp(input.getRuntime(), input.unmarshalString(), input.readSignedByte());
         input.registerLinkTarget(result);
         return result;
     }
@@ -2062,6 +2096,6 @@ public class RubyRegexp extends RubyObject implements ReOptions, EncodingCapable
     public static void marshalTo(RubyRegexp regexp, MarshalStream output) throws java.io.IOException {
         output.registerLinkTarget(regexp);
         output.writeString(new String(regexp.str.bytes,regexp.str.begin,regexp.str.realSize));
-        output.writeInt(regexp.pattern.getOptions() & EMBEDDABLE);
+        output.writeByte(regexp.pattern.getOptions() & EMBEDDABLE);
     }
 }
