@@ -31,11 +31,14 @@
 package org.jruby;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.List;
 
 import org.jruby.anno.FrameField;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
+import org.jruby.java.addons.IOJavaAddons;
+import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
@@ -47,6 +50,8 @@ import org.jruby.util.io.InvalidValueException;
 import org.jruby.util.io.ModeFlags;
 import org.jruby.util.io.Stream;
 
+import static org.jruby.RubyEnumerator.enumeratorize;
+
 @JRubyClass(name="StringIO")
 public class RubyStringIO extends RubyObject {
     private static ObjectAllocator STRINGIO_ALLOCATOR = new ObjectAllocator() {
@@ -62,6 +67,10 @@ public class RubyStringIO extends RubyObject {
         stringIOClass.defineAnnotatedMethods(RubyStringIO.class);
         stringIOClass.includeModule(runtime.getEnumerable());
 
+        if (runtime.getObject().isConstantDefined("Java")) {
+            stringIOClass.defineAnnotatedMethods(IOJavaAddons.AnyIO.class);
+        }
+
         return stringIOClass;
     }
 
@@ -238,19 +247,32 @@ public class RubyStringIO extends RubyObject {
         return getRuntime().newBoolean(closedWrite);
     }
 
-    @JRubyMethod(name = "each", optional = 1, frame = true, writes = FrameField.LASTLINE)
     public IRubyObject each(ThreadContext context, IRubyObject[] args, Block block) {
-        IRubyObject line = gets(context, args);
+        IRubyObject line = getsOnly(context, args);
        
         while (!line.isNil()) {
             block.yield(context, line);
-            line = gets(context, args);
+            line = getsOnly(context, args);
         }
        
         return this;
     }
 
-    @JRubyMethod(name = "each_byte", frame = true)
+    @JRubyMethod(name = "each", optional = 1, frame = true, writes = FrameField.LASTLINE)
+    public IRubyObject each19(ThreadContext context, IRubyObject[] args, Block block) {
+        return block.isGiven() ? each(context, args, block) : enumeratorize(context.getRuntime(), this, "each", args);
+    }
+
+    @JRubyMethod(name = "each_line", optional = 1, frame = true)
+    public IRubyObject each_line(ThreadContext context, IRubyObject[] args, Block block) {
+        return block.isGiven() ? each(context, args, block) : enumeratorize(context.getRuntime(), this, "each_line", args);
+    }
+
+    @JRubyMethod(name = "lines", optional = 1, frame = true)
+    public IRubyObject lines(ThreadContext context, IRubyObject[] args, Block block) {
+        return block.isGiven() ? each(context, args, block) : enumeratorize(context.getRuntime(), this, "lines", args);
+    }
+
     public IRubyObject each_byte(ThreadContext context, Block block) {
         checkReadable();
         Ruby runtime = context.getRuntime();
@@ -261,12 +283,47 @@ public class RubyStringIO extends RubyObject {
         while (pos < bytes.length()) {
             block.yield(context, runtime.newFixnum(bytes.get((int) pos++) & 0xFF));
         }
-        return runtime.getNil();
+        return this;
     }
 
-    @JRubyMethod(name = "each_line", optional = 1, frame = true)
-    public IRubyObject each_line(ThreadContext context, IRubyObject[] args, Block block) {
-        return each(context, args, block);
+    @JRubyMethod(name = "each_byte", frame = true)
+    public IRubyObject each_byte19(ThreadContext context, Block block) {
+        return block.isGiven() ? each_byte(context, block) : enumeratorize(context.getRuntime(), this, "each_byte");
+    }
+
+    @JRubyMethod(name = "bytes", frame = true)
+    public IRubyObject bytes(ThreadContext context, Block block) {
+        return block.isGiven() ? each_byte(context, block) : enumeratorize(context.getRuntime(), this, "bytes");
+    }
+
+    public IRubyObject each_char(final ThreadContext context, final Block block) {
+        checkReadable();
+
+        Ruby runtime = context.getRuntime();
+        ByteList bytes = internal.getByteList();
+        int len = bytes.realSize;
+        while (pos < len) {
+            int pos = (int)this.pos;
+            byte c = bytes.bytes[bytes.begin + pos];
+            int n = runtime.getKCode().getEncoding().length(c);
+            if(len < pos + n) {
+                n = len - pos;
+            }
+            this.pos += n;
+            block.yield(context, internal.substr19(runtime, pos, n));
+        }
+
+        return this;
+    }
+
+    @JRubyMethod(name = "each_char", frame = true)
+    public IRubyObject each_char19(final ThreadContext context, final Block block) {
+        return block.isGiven() ? each_char(context, block) : enumeratorize(context.getRuntime(), this, "each_char");
+    }
+
+    @JRubyMethod(name = "chars", frame = true)
+    public IRubyObject chars19(final ThreadContext context, final Block block) {
+        return block.isGiven() ? each_char(context, block) : enumeratorize(context.getRuntime(), this, "chars");
     }
 
     @JRubyMethod(name = {"eof", "eof?"})
@@ -298,7 +355,7 @@ public class RubyStringIO extends RubyObject {
         return RubyFixnum.zero(getRuntime());
     }
 
-    @JRubyMethod(name = "getc")
+    @JRubyMethod(name = {"getc", "getbyte"})
     public IRubyObject getc() {
         checkReadable();
         if (pos >= internal.getByteList().length()) {
@@ -372,13 +429,17 @@ public class RubyStringIO extends RubyObject {
 
     @JRubyMethod(name = "gets", optional = 1, writes = FrameField.LASTLINE)
     public IRubyObject gets(ThreadContext context, IRubyObject[] args) {
-        checkReadable();
-
-        IRubyObject result = internalGets(context, args);
-        context.getCurrentFrame().setLastLine(result);
+        IRubyObject result = getsOnly(context, args);
+        context.getCurrentScope().setLastLine(result);
 
         return result;
     }
+    
+    public IRubyObject getsOnly(ThreadContext context, IRubyObject[] args) {
+        checkReadable();
+
+        return internalGets(context, args);
+    }
 
     @JRubyMethod(name = {"tty?", "isatty"})
     public IRubyObject isatty() {
@@ -490,28 +551,32 @@ public class RubyStringIO extends RubyObject {
         }
 
         for (int i = 0; i < args.length; i++) {
-            String line;
+            RubyString line;
             
             if (args[i].isNil()) {
-                line = "nil";
+                line = getRuntime().newString("nil");
             } else {
                 IRubyObject tmp = args[i].checkArrayType();
                 if (!tmp.isNil()) {
                     RubyArray arr = (RubyArray) tmp;
                     if (getRuntime().isInspecting(arr)) {
-                        line = "[...]";
+                        line = getRuntime().newString("[...]");
                     } else {
                         inspectPuts(context, arr);
                         continue;
                     }
                 } else {
-                    line = args[i].toString();
+                    if (args[i] instanceof RubyString) {
+                        line = (RubyString)args[i];
+                    } else {
+                        line = args[i].asString();
+                    }
                 }
             }
 
-            callMethod(context, "write", getRuntime().newString(line));
+            callMethod(context, "write", line);
 
-            if (!line.endsWith("\n")) {
+            if (!line.getByteList().endsWith(NEWLINE)) {
                 callMethod(context, "write", RubyString.newStringShared(getRuntime(), NEWLINE));
             }
         }
@@ -600,9 +665,19 @@ public class RubyStringIO extends RubyObject {
             
             if (length > rest) length = rest;
 
-            // Yow...this is ugly
-            buf.realSize = length;
-            buf.replace(0, length, internal.getByteList().bytes, (int) pos, length);
+            // Yow...this is still ugly
+            byte[] target = buf.bytes;
+            if (target.length > length) {
+                System.arraycopy(internal.getByteList().bytes, (int) pos, target, 0, length);
+                buf.begin = 0;
+                buf.realSize = length;
+            } else {
+                target = new byte[length];
+                System.arraycopy(internal.getByteList().bytes, (int) pos, target, 0, length);
+                buf.begin = 0;
+                buf.realSize = length;
+                buf.bytes = target;
+            }
         }
         
         if (buf == null) {
@@ -618,7 +693,7 @@ public class RubyStringIO extends RubyObject {
         return originalString != null ? originalString : getRuntime().newString(buf);
     }
 
-    @JRubyMethod(name = "readchar")
+    @JRubyMethod(name = {"readchar", "readbyte"})
     public IRubyObject readchar() {
         IRubyObject c = getc();
         
@@ -679,8 +754,7 @@ public class RubyStringIO extends RubyObject {
 
     @JRubyMethod(name = "seek", required = 1, optional = 1, frame=true)
     public IRubyObject seek(IRubyObject[] args) {
-        // MRI 1.8.7 behavior:
-        // checkOpen();
+        checkOpen();
         checkFinalized();
         long amount = RubyNumeric.num2long(args[0]);
         int whence = Stream.SEEK_SET;
@@ -733,7 +807,7 @@ public class RubyStringIO extends RubyObject {
         IRubyObject obj = read(args);
 
         if (isEOF()) {
-            if (obj.isNil() || ((RubyString) obj).getByteList().length() == 0) {
+            if (obj.isNil()) {
                 throw getRuntime().newEOFError();
             }
         }
@@ -751,7 +825,11 @@ public class RubyStringIO extends RubyObject {
         }
 
         internal.modify();
-        internal.getByteList().length(len);
+        ByteList buf = internal.getByteList();
+        if (len < buf.length()) {
+            Arrays.fill(buf.unsafeBytes(), len, buf.length(), (byte) 0);
+        }
+        buf.length(len);
         return arg;
     }
 
