@@ -76,6 +76,13 @@ public class RubySymbol extends RubyObject {
 
         //        assert internedSymbol == internedSymbol.intern() : internedSymbol + " is not interned";
 
+        int length = symbolBytes.begin + symbolBytes.realSize;
+        for (int i = symbolBytes.begin; i < length; i++) {
+            if (symbolBytes.bytes[i] == 0) {
+                throw runtime.newSyntaxError("symbol cannot contain '\\0'");
+            }
+        }
+
         this.symbol = internedSymbol;
         this.symbolBytes = symbolBytes;
         this.id = runtime.allocSymbolId();
@@ -116,6 +123,12 @@ public class RubySymbol extends RubyObject {
         return symbol;
     }
 
+    @Override
+    public RubyString convertToString() {
+        Ruby runtime = getRuntime();
+        return runtime.is1_9() ? newShared(runtime) : super.convertToString(); 
+    }
+
     @Override
     public String toString() {
         return symbol;
@@ -570,6 +583,13 @@ public class RubySymbol extends RubyObject {
         return result;
     }
 
+    public Object toJava(Class target) {
+        if (target == String.class || target == CharSequence.class) {
+            return symbol;
+        }
+        return super.toJava(target);
+    }
+
     public static final class SymbolTable {
         static final int DEFAULT_INITIAL_CAPACITY = 2048; // *must* be power of 2!
         static final int MAXIMUM_CAPACITY = 1 << 30;
