@@ -167,7 +167,7 @@ public class RubyTime extends RubyObject {
     private static ObjectAllocator TIME_ALLOCATOR = new ObjectAllocator() {
         public IRubyObject allocate(Ruby runtime, RubyClass klass) {
             DateTimeZone dtz = getLocalTimeZone(runtime);
-            DateTime dt = new DateTime(System.currentTimeMillis(), dtz);
+            DateTime dt = new DateTime(dtz);
             RubyTime rt =  new RubyTime(runtime, klass, dt);
             rt.setUSec(0);
 
@@ -272,12 +272,23 @@ public class RubyTime extends RubyObject {
 
     @JRubyMethod(name = "strftime", required = 1)
     public RubyString strftime(IRubyObject format) {
-        final RubyDateFormat rubyDateFormat = new RubyDateFormat("-", Locale.US);
-        rubyDateFormat.applyPattern(format.toString());
+        final RubyDateFormat rubyDateFormat = new RubyDateFormat("-", Locale.US, getRuntime().is1_9());
+        rubyDateFormat.applyPattern(format.convertToString().getUnicodeValue());
         rubyDateFormat.setDateTime(dt);
         String result = rubyDateFormat.format(null);
         return getRuntime().newString(result);
     }
+
+    @JRubyMethod(name = "==", required = 1, compat= CompatVersion.RUBY1_9)
+    public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
+        if (other.isNil()) {
+            return RubyBoolean.newBoolean(getRuntime(), false);
+        } else if (other instanceof RubyTime) {
+            return getRuntime().newBoolean(cmp((RubyTime) other) == 0);
+        }
+
+        return RubyComparable.op_equal(context, this, other);
+    }
     
     @JRubyMethod(name = ">=", required = 1)
     public IRubyObject op_ge(ThreadContext context, IRubyObject other) {
@@ -615,6 +626,12 @@ public class RubyTime extends RubyObject {
         return newInstance(context, recv);
     }
 
+    @JRubyMethod(name = "times", meta = true, compat = CompatVersion.RUBY1_8)
+    public static IRubyObject times(ThreadContext context, IRubyObject recv) {
+        context.getRuntime().getWarnings().warn("obsolete method Time::times; use Process::times");
+        return RubyProcess.times(context, recv, Block.NULL_BLOCK);
+    }
+
     @JRubyMethod(name = "now", backtrace = true, meta = true)
     public static IRubyObject newInstance(ThreadContext context, IRubyObject recv) {
         IRubyObject obj = ((RubyClass) recv).allocate();
@@ -645,7 +662,9 @@ public class RubyTime extends RubyObject {
             // the decimal point is honored.
             if (arg instanceof RubyFloat) {
                 double dbl = ((RubyFloat) arg).getDoubleValue();
-                long micro = (long) ((dbl - seconds) * 1000000);
+                long micro = Math.round((dbl - seconds) * 1000000);
+                if(dbl < 0)
+                    micro += 1000000;
                 millisecs = micro / 1000;
                 microsecs = micro % 1000;
             }
@@ -696,6 +715,10 @@ public class RubyTime extends RubyObject {
         return s_mload(recv, (RubyTime)(((RubyClass)recv).allocate()), from);
     }
 
+    public Object toJava(Class target) {
+        return getJavaDate();
+    }
+    
     protected static RubyTime s_mload(IRubyObject recv, RubyTime time, IRubyObject from) {
         Ruby runtime = recv.getRuntime();
 
