@@ -105,25 +105,15 @@ public class YieldNode extends Node {
     
     @Override
     public IRubyObject interpret(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
-        IRubyObject result = null;
-        
-        if (argsNode != null) result = argsNode.interpret(runtime, context, self, aBlock);
-
         if (expandedArguments) {
-            return context.getCurrentFrame().getBlock().yield(context, result, null, null, true);
-        } else {
-            return context.getCurrentFrame().getBlock().yield(context, result);
-        }
+            return context.getCurrentFrame().getBlock().yield(context, argsNode.interpret(runtime, context, self, aBlock), null, null, true);
+        } 
+
+        return context.getCurrentFrame().getBlock().yield(context, argsNode.interpret(runtime, context, self, aBlock));
     }
     
     @Override
     public String definition(Ruby runtime, ThreadContext context, IRubyObject self, Block aBlock) {
         return aBlock.isGiven() ? "yield" : null;
     }
-
-    @Override
-    public String toString() {
-        return "YieldNode[" + (argsNode != null ? argsNode : "") + "]";
-
-    }
 }
