@@ -122,60 +122,36 @@ public abstract class AbstractVariableCompiler implements VariableCompiler {
     }
 
     public void checkMethodArity(int requiredArgs, int optArgs, int restArg) {
-        // check arity
-        Label arityError = new Label();
-        Label noArityError = new Label();
-
         if (specificArity) {
             // do nothing; arity check is done before call
-            // FIXME: arity check is not yet done before call :)
         } else {
             boolean needsError = false;
             if (restArg != -1) {
                 if (requiredArgs > 0) {
                     needsError = true;
                     // just confirm minimum args provided
+                    methodCompiler.loadRuntime();
                     method.aload(argsIndex);
-                    method.arraylength();
                     method.pushInt(requiredArgs);
-                    method.if_icmplt(arityError);
+                    method.pushInt(-1);
                 }
             } else if (optArgs > 0) {
                 needsError = true;
-                if (requiredArgs > 0) {
-                    // confirm minimum args provided
-                    method.aload(argsIndex);
-                    method.arraylength();
-                    method.pushInt(requiredArgs);
-                    method.if_icmplt(arityError);
-                }
-
-                // confirm maximum not greater than optional
-                method.aload(argsIndex);
-                method.arraylength();
-                method.pushInt(requiredArgs + optArgs);
-                method.if_icmpgt(arityError);
-            } else {
-                needsError = true;
-                // just confirm args length == required
-                method.aload(argsIndex);
-                method.arraylength();
-                method.pushInt(requiredArgs);
-                method.if_icmpne(arityError);
-            }
-
-            if (needsError) {
-                method.go_to(noArityError);
-
-                method.label(arityError);
                 methodCompiler.loadRuntime();
                 method.aload(argsIndex);
                 method.pushInt(requiredArgs);
                 method.pushInt(requiredArgs + optArgs);
-                method.invokestatic(p(Arity.class), "checkArgumentCount", sig(int.class, Ruby.class, IRubyObject[].class, int.class, int.class));
-                method.pop();
+            } else {
+                needsError = true;
+                // just confirm args length == required
+                methodCompiler.loadRuntime();
+                method.aload(argsIndex);
+                method.pushInt(requiredArgs);
+                method.pushInt(requiredArgs);
+            }
 
-                method.label(noArityError);
+            if (needsError) {
+                method.invokestatic(p(Arity.class), "raiseArgumentError", sig(void.class, Ruby.class, IRubyObject[].class, int.class, int.class));
             }
         }
     }
@@ -195,89 +171,58 @@ public abstract class AbstractVariableCompiler implements VariableCompiler {
             for (; currentArgElement < scope.getRequiredArgs(); currentArgElement++) {
                 method.aload(argsIndex + currentArgElement);
                 requiredAssignment.nextValue(methodCompiler, requiredArgs, currentArgElement);
-
-                // normal assignment leaves the value; pop it.
-                method.pop();
             }
         } else {
             if (requiredArgsCount > 0 || optArgsCount > 0 || restAssignment != null) {
-                // load arguments array
-                method.aload(argsIndex);
-
-                // NOTE: This assumes arity has already been confirmed, and does not re-check
-
                 // first, iterate over all required args
                 int currentArgElement = 0;
                 for (; currentArgElement < requiredArgsCount; currentArgElement++) {
                     // extract item from array
-                    method.dup(); // dup the original array
+                    method.aload(argsIndex);
                     method.pushInt(currentArgElement); // index for the item
                     method.arrayload();
                     requiredAssignment.nextValue(methodCompiler, requiredArgs, currentArgElement);
-
-                    // normal assignment leaves the value; pop it.
-                    method.pop();
                 }
 
-                // next, iterate over all optional args, until no more arguments
-                for (int optArgElement = 0; optArgElement < optArgsCount; currentArgElement++, optArgElement++) {
-                    Label noMoreArrayElements = new Label();
-                    Label doneWithElement = new Label();
+                if (optArgsCount > 0) {
+                    // prepare labels for opt logic
+                    Label doneWithOpt = new Label();
+                    Label[] optLabels = new Label[optArgsCount];
+                    for (int i = 0; i < optLabels.length; i ++) optLabels[i] = new Label();
 
-                    // confirm we're not past the end of the array
-                    method.dup(); // dup the original array
-                    method.arraylength();
-                    method.pushInt(currentArgElement);
-                    method.if_icmple(noMoreArrayElements); // if length <= start, end loop
+                    // next, iterate over all optional args, until no more arguments
+                    for (int optArgElement = 0; optArgElement < optArgsCount; currentArgElement++, optArgElement++) {
+                        method.aload(argsIndex);
+                        method.pushInt(currentArgElement); // index for the item
+                        methodCompiler.invokeUtilityMethod("elementOrNull", sig(IRubyObject.class, IRubyObject[].class, int.class));
+                        method.dup();
+                        method.ifnull(optLabels[optArgElement]);
 
-                    // extract item from array
-                    method.dup(); // dup the original array
-                    method.pushInt(currentArgElement); // index for the item
-                    method.arrayload();
-                    optGivenAssignment.nextValue(methodCompiler, optArgs, optArgElement);
-                    method.go_to(doneWithElement);
+                        optGivenAssignment.nextValue(methodCompiler, optArgs, optArgElement);
+                    }
+                    method.go_to(doneWithOpt);
 
-                    // otherwise no items left available, use the code for default
-                    method.label(noMoreArrayElements);
-                    optNotGivenAssignment.nextValue(methodCompiler, optArgs, optArgElement);
+                    // now logic for each optional value
+                    for (int optArgElement = 0; optArgElement < optArgsCount; optArgElement++) {
+                        // otherwise no items left available, use the code for default
+                        method.label(optLabels[optArgElement]);
+                        optNotGivenAssignment.nextValue(methodCompiler, optArgs, optArgElement);
+                    }
 
-                    // end of this element
-                    method.label(doneWithElement);
-                    // normal assignment leaves the value; pop it.
+                    // pop extra failed value from first cycle and we're done
                     method.pop();
+                    method.label(doneWithOpt);
                 }
 
                 // if there's args left and we want them, assign to rest arg
                 if (restAssignment != null) {
-                    Label emptyArray = new Label();
-                    Label readyForArgs = new Label();
-
-                    // confirm we're not past the end of the array
-                    method.dup(); // dup the original array
-                    method.arraylength();
-                    method.pushInt(currentArgElement);
-                    method.if_icmple(emptyArray); // if length <= start, end loop
-
-                    // assign remaining elements as an array for rest args
-                    method.dup(); // dup the original array object
+                    // assign remaining elements as an array for rest args (or empty array)
+                    method.aload(argsIndex);
                     methodCompiler.loadRuntime();
                     method.pushInt(currentArgElement);
                     methodCompiler.invokeUtilityMethod("createSubarray", sig(RubyArray.class, IRubyObject[].class, Ruby.class, int.class));
-                    method.go_to(readyForArgs);
-
-                    // create empty array
-                    method.label(emptyArray);
-                    methodCompiler.createEmptyArray();
-
-                    // assign rest args
-                    method.label(readyForArgs);
                     restAssignment.call(methodCompiler);
-                    //consume leftover assigned value
-                    method.pop();
                 }
-
-                // done with arguments array
-                method.pop();
             }
         }
         
@@ -288,7 +233,105 @@ public abstract class AbstractVariableCompiler implements VariableCompiler {
 
             methodCompiler.invokeUtilityMethod("processBlockArgument", sig(IRubyObject.class, params(Ruby.class, Block.class)));
             blockAssignment.call(methodCompiler);
-            method.pop();
+        }
+    }
+
+    public void assignMethodArguments19(
+            Object preArgs,
+            int preArgsCount,
+            Object postArgs,
+            int postArgsCount,
+            int postArgsIndex,
+            Object optArgs,
+            int optArgsCount,
+            ArrayCallback requiredAssignment,
+            ArrayCallback optGivenAssignment,
+            ArrayCallback optNotGivenAssignment,
+            CompilerCallback restAssignment,
+            CompilerCallback blockAssignment) {
+        
+        if (specificArity) {
+            int currentArgElement = 0;
+            for (; currentArgElement < scope.getRequiredArgs(); currentArgElement++) {
+                method.aload(argsIndex + currentArgElement);
+                requiredAssignment.nextValue(methodCompiler, preArgs, currentArgElement);
+            }
+        } else {
+            if (preArgsCount > 0 || postArgsCount > 0 || optArgsCount > 0 || restAssignment != null) {
+                // first, iterate over all pre args
+                int currentArgElement = 0;
+                for (; currentArgElement < preArgsCount; currentArgElement++) {
+                    // extract item from array
+                    method.aload(argsIndex);
+                    method.pushInt(currentArgElement); // index for the item
+                    // this could probably be more efficient, bailing out on assigning args past the end?
+                    methodCompiler.loadNil();
+                    methodCompiler.invokeUtilityMethod("elementOrNil", sig(IRubyObject.class, IRubyObject[].class, int.class, IRubyObject.class));
+                    requiredAssignment.nextValue(methodCompiler, preArgs, currentArgElement);
+                }
+
+                // then optional args
+                if (optArgsCount > 0) {
+                    // prepare labels for opt logic
+                    Label doneWithOpt = new Label();
+                    Label[] optLabels = new Label[optArgsCount];
+                    for (int i = 0; i < optLabels.length; i ++) optLabels[i] = new Label();
+
+                    // next, iterate over all optional args, until no more arguments
+                    for (int optArgElement = 0; optArgElement < optArgsCount; currentArgElement++, optArgElement++) {
+                        method.aload(argsIndex);
+                        method.pushInt(currentArgElement); // index for the item
+                        methodCompiler.invokeUtilityMethod("elementOrNull", sig(IRubyObject.class, IRubyObject[].class, int.class));
+                        method.dup();
+                        method.ifnull(optLabels[optArgElement]);
+
+                        optGivenAssignment.nextValue(methodCompiler, optArgs, optArgElement);
+                    }
+                    method.go_to(doneWithOpt);
+
+                    // now logic for each optional value
+                    for (int optArgElement = 0; optArgElement < optArgsCount; optArgElement++) {
+                        // otherwise no items left available, use the code for default
+                        method.label(optLabels[optArgElement]);
+                        optNotGivenAssignment.nextValue(methodCompiler, optArgs, optArgElement);
+                    }
+
+                    // pop extra failed value from first cycle and we're done
+                    method.pop();
+                    method.label(doneWithOpt);
+                }
+
+                // if rest args, excluding post args
+                if (restAssignment != null) {
+                    // assign remaining elements as an array for rest args (or empty array)
+                    method.aload(argsIndex);
+                    methodCompiler.loadRuntime();
+                    method.pushInt(currentArgElement);
+                    method.pushInt(postArgsCount);
+                    methodCompiler.invokeUtilityMethod("createSubarray", sig(RubyArray.class, IRubyObject[].class, Ruby.class, int.class, int.class));
+                    restAssignment.call(methodCompiler);
+                }
+
+                // finally, post args
+                for (; currentArgElement < postArgsCount; currentArgElement++) {
+                    // extract item from array
+                    method.aload(argsIndex);
+                    method.pushInt(currentArgElement); // index for the item
+                    // this could probably be more efficient, bailing out on assigning args past the end?
+                    methodCompiler.loadNil();
+                    methodCompiler.invokeUtilityMethod("elementOrNil", sig(IRubyObject.class, IRubyObject[].class, int.class, IRubyObject.class));
+                    requiredAssignment.nextValue(methodCompiler, postArgs, currentArgElement);
+                }
+            }
+        }
+
+        // block argument assignment, if there's a block arg
+        if (blockAssignment != null) {
+            methodCompiler.loadRuntime();
+            method.aload(methodCompiler.getClosureIndex());
+
+            methodCompiler.invokeUtilityMethod("processBlockArgument", sig(IRubyObject.class, params(Ruby.class, Block.class)));
+            blockAssignment.call(methodCompiler);
         }
     }
         
@@ -308,7 +351,7 @@ public abstract class AbstractVariableCompiler implements VariableCompiler {
         tempVariableIndex--;
     }
 
-    protected void assignHeapLocal(CompilerCallback value, int depth, int index) {
+    protected void assignHeapLocal(CompilerCallback value, int depth, int index, boolean expr) {
         switch (index) {
         case 0:
             unwrapParentScopes(depth);
@@ -337,9 +380,14 @@ public abstract class AbstractVariableCompiler implements VariableCompiler {
             method.pushInt(depth);
             method.invokevirtual(p(DynamicScope.class), "setValue", sig(IRubyObject.class, params(Integer.TYPE, IRubyObject.class, Integer.TYPE)));
         }
+
+        if (!expr) {
+            // not an expression, don't want result; pop it
+            method.pop();
+        }
     }
 
-    protected void assignHeapLocal(int depth, int index) {
+    protected void assignHeapLocal(int depth, int index, boolean expr) {
         
         switch (index) {
         case 0:
@@ -369,6 +417,11 @@ public abstract class AbstractVariableCompiler implements VariableCompiler {
             method.pushInt(depth);
             method.invokevirtual(p(DynamicScope.class), "setValue", sig(IRubyObject.class, params(IRubyObject.class, Integer.TYPE, Integer.TYPE)));
         }
+
+        if (!expr) {
+            // not an expression, don't want result; pop it
+            method.pop();
+        }
     }
 
     protected void retrieveHeapLocal(int depth, int index) {
