@@ -7,12 +7,10 @@ package org.jruby.compiler.impl;
 import java.io.PrintStream;
 import java.lang.reflect.InvocationTargetException;
 import java.math.BigInteger;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -60,9 +58,9 @@ import org.jruby.parser.ReOptions;
 import org.jruby.parser.StaticScope;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.BlockBody;
+import org.jruby.runtime.CallType;
 import org.jruby.runtime.CompiledBlockCallback;
 import org.jruby.runtime.DynamicScope;
-import org.jruby.runtime.Frame;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -105,7 +103,7 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
             argParamCount = scope.getRequiredArgs(); // specific arity
         }
 
-        method = new SkinnyMethodAdapter(script.getClassVisitor().visitMethod(ACC_PUBLIC, methodName, getSignature(), null, null));
+        method = new SkinnyMethodAdapter(script.getClassVisitor().visitMethod(ACC_PUBLIC | ACC_STATIC, methodName, getSignature(), null, null));
 
         createVariableCompiler();
         if (StandardASMCompiler.invDynInvCompilerConstructor != null) {
@@ -250,14 +248,6 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         method.aconst_null();
     }
 
-    public void loadSymbol(String symbol) {
-        loadRuntime();
-
-        method.ldc(symbol);
-
-        invokeRuby("newSymbol", sig(RubySymbol.class, params(String.class)));
-    }
-
     public void loadObject() {
         loadRuntime();
 
@@ -366,8 +356,7 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
 
     public void assignConstantInObject(String name) {
         // load Object under value
-        loadRuntime();
-        invokeRuby("getObject", sig(RubyClass.class, params()));
+        loadObject();
         method.swap();
 
         assignConstantInModule(name);
@@ -505,7 +494,7 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
     public void createEmptyArray() {
         loadRuntime();
 
-        invokeRuby("newArray", sig(RubyArray.class, params()));
+        invokeRuby("newArray", sig(RubyArray.class));
     }
 
     public void createObjectArray(Object[] sourceArray, ArrayCallback callback) {
@@ -587,6 +576,15 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
     }
 
     public void createNewHash(Object elements, ArrayCallback callback, int keyCount) {
+        createNewHashCommon(elements, callback, keyCount, "constructHash", "fastASetCheckString");
+    }
+    
+    public void createNewHash19(Object elements, ArrayCallback callback, int keyCount) {
+        createNewHashCommon(elements, callback, keyCount, "constructHash19", "fastASetCheckString19");
+    }
+    
+    private void createNewHashCommon(Object elements, ArrayCallback callback, int keyCount,
+            String constructorName, String methodName) {
         loadRuntime();
 
         // use specific-arity for as much as possible
@@ -595,12 +593,13 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
             callback.nextValue(this, elements, i);
         }
 
-        invokeUtilityMethod("constructHash", sig(RubyHash.class, params(Ruby.class, IRubyObject.class, i * 2)));
+        invokeUtilityMethod(constructorName, sig(RubyHash.class, params(Ruby.class, IRubyObject.class, i * 2)));
 
         for (; i < keyCount; i++) {
             method.dup();
+            loadRuntime();
             callback.nextValue(this, elements, i);
-            method.invokevirtual(p(RubyHash.class), "fastASet", sig(void.class, params(IRubyObject.class, IRubyObject.class)));
+            method.invokevirtual(p(RubyHash.class), methodName, sig(void.class, params(Ruby.class, IRubyObject.class, IRubyObject.class)));
         }
     }
 
@@ -996,23 +995,22 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
     }
 
     public void retrieveInstanceVariable(String name) {
-        loadSelf();
-        loadRuntime();
-        method.ldc(name);
-        invokeUtilityMethod("getInstanceVariable", sig(IRubyObject.class, IRubyObject.class, Ruby.class, String.class));
+        script.getCacheCompiler().cachedGetVariable(this, name);
     }
 
     public void assignInstanceVariable(String name) {
-        loadSelf();
-        method.ldc(name);
-        invokeUtilityMethod("setInstanceVariable", sig(IRubyObject.class, IRubyObject.class, IRubyObject.class, String.class));
+        final int tmp = getVariableCompiler().grabTempLocal();
+        getVariableCompiler().setTempLocal(tmp);
+        CompilerCallback callback = new CompilerCallback() {
+            public void call(BodyCompiler context) {
+                context.getVariableCompiler().getTempLocal(tmp);
+            }
+        };
+        script.getCacheCompiler().cachedSetVariable(this, name, callback);
     }
 
     public void assignInstanceVariable(String name, CompilerCallback value) {
-        value.call(this);
-        loadSelf();
-        method.ldc(name);
-        invokeUtilityMethod("setInstanceVariable", sig(IRubyObject.class, IRubyObject.class, IRubyObject.class, String.class));
+        script.getCacheCompiler().cachedSetVariable(this, name, value);
     }
 
     public void retrieveGlobalVariable(String name) {
@@ -1183,9 +1181,8 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         } else {
             loadRuntime();
             createStringCallback.call(this);
-            method.invokevirtual(p(RubyString.class), "getByteList", sig(ByteList.class));
             method.pushInt(options);
-            method.invokestatic(p(RubyRegexp.class), "newRegexp", sig(RubyRegexp.class, params(Ruby.class, ByteList.class, Integer.TYPE))); //[reg]
+            method.invokestatic(p(RubyRegexp.class), "newDRegexp", sig(RubyRegexp.class, params(Ruby.class, RubyString.class, int.class))); //[reg]
         }
     }
 
@@ -1253,9 +1250,9 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
     }
 
     public void backref() {
+        loadRuntime();
         loadThreadContext();
-        invokeThreadContext("getCurrentFrame", sig(Frame.class));
-        method.invokevirtual(p(Frame.class), "getBackRef", sig(IRubyObject.class));
+        invokeUtilityMethod("getBackref", sig(IRubyObject.class, Ruby.class, ThreadContext.class));
     }
 
     public void backrefMethod(String methodName) {
@@ -1285,7 +1282,13 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
 
     public void protect(BranchCallback regularCode, BranchCallback protectedCode, Class ret) {
         String mname = getNewEnsureName();
-        SkinnyMethodAdapter mv = new SkinnyMethodAdapter(script.getClassVisitor().visitMethod(ACC_PUBLIC | ACC_SYNTHETIC, mname, sig(ret, new Class[]{ThreadContext.class, IRubyObject.class, Block.class}), null, null));
+        SkinnyMethodAdapter mv = new SkinnyMethodAdapter(
+                script.getClassVisitor().visitMethod(
+                ACC_PUBLIC | ACC_SYNTHETIC | ACC_STATIC,
+                mname,
+                sig(ret, "L" + script.getClassname() + ";", ThreadContext.class, IRubyObject.class, Block.class),
+                null,
+                null));
         SkinnyMethodAdapter old_method = null;
         SkinnyMethodAdapter var_old_method = null;
         SkinnyMethodAdapter inv_old_method = null;
@@ -1362,7 +1365,10 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         } else {
             loadBlock();
         }
-        method.invokevirtual(script.getClassname(), mname, sig(ret, new Class[]{ThreadContext.class, IRubyObject.class, Block.class}));
+        method.invokestatic(
+                script.getClassname(),
+                mname,
+                sig(ret, "L" + script.getClassname() + ";", ThreadContext.class, IRubyObject.class, Block.class));
     }
 
     public void performEnsure(BranchCallback regularCode, BranchCallback protectedCode) {
@@ -1408,7 +1414,7 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
     public void storeExceptionInErrorInfo() {
         loadException();
         loadThreadContext();
-        invokeUtilityMethod("storeExceptionInErrorInfo", sig(void.class, Exception.class, ThreadContext.class));
+        invokeUtilityMethod("storeExceptionInErrorInfo", sig(void.class, Throwable.class, ThreadContext.class));
     }
 
     public void clearErrorInfo() {
@@ -1418,7 +1424,13 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
 
     public void rescue(BranchCallback regularCode, Class exception, BranchCallback catchCode, Class ret) {
         String mname = getNewRescueName();
-        SkinnyMethodAdapter mv = new SkinnyMethodAdapter(script.getClassVisitor().visitMethod(ACC_PUBLIC | ACC_SYNTHETIC, mname, sig(ret, new Class[]{ThreadContext.class, IRubyObject.class, Block.class}), null, null));
+        SkinnyMethodAdapter mv = new SkinnyMethodAdapter(
+                script.getClassVisitor().visitMethod(
+                    ACC_PUBLIC | ACC_SYNTHETIC | ACC_STATIC,
+                    mname,
+                    sig(ret, "L" + script.getClassname() + ";", ThreadContext.class, IRubyObject.class, Block.class),
+                    null,
+                    null));
         SkinnyMethodAdapter old_method = null;
         SkinnyMethodAdapter var_old_method = null;
         SkinnyMethodAdapter inv_old_method = null;
@@ -1534,7 +1546,10 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         } else {
             loadBlock();
         }
-        method.invokevirtual(script.getClassname(), mname, sig(ret, new Class[]{ThreadContext.class, IRubyObject.class, Block.class}));
+        method.invokestatic(
+                script.getClassname(),
+                mname,
+                sig(ret, "L" + script.getClassname() + ";", ThreadContext.class, IRubyObject.class, Block.class));
     }
 
     public void performRescue(BranchCallback regularCode, BranchCallback rubyCatchCode, boolean needsRetry) {
@@ -1560,7 +1575,7 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         Label rubyCatchBlock = new Label();
         Label flowCatchBlock = new Label();
         method.visitTryCatchBlock(beforeBody, afterBody, flowCatchBlock, p(JumpException.FlowControlException.class));
-        method.visitTryCatchBlock(beforeBody, afterBody, rubyCatchBlock, p(Exception.class));
+        method.visitTryCatchBlock(beforeBody, afterBody, rubyCatchBlock, p(Throwable.class));
 
         method.visitLabel(beforeBody);
         {
@@ -1958,6 +1973,13 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         method.invokevirtual(p(GlobalVariables.class), "alias", sig(Void.TYPE, params(String.class, String.class)));
         loadNil();
     }
+    
+    public void raiseTypeError(String msg) {
+        loadRuntime();        
+        method.ldc(msg);
+        invokeRuby("newTypeError", sig(RaiseException.class, params(String.class)));
+        method.athrow();                                    
+    }    
 
     public void undefMethod(String name) {
         loadThreadContext();
@@ -2001,9 +2023,11 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         CompilerCallback bodyPrep = new CompilerCallback() {
             public void call(BodyCompiler context) {
                 if (receiverCallback == null) {
+                    // no receiver for singleton class
                     if (superCallback != null) {
+                        // but there's a superclass passed in, use it
                         classBody.loadRuntime();
-                        superCallback.call(classBody);
+                        classBody.method.aload(StandardASMCompiler.SELF_INDEX);
 
                         classBody.invokeUtilityMethod("prepareSuperClass", sig(RubyClass.class, params(Ruby.class, IRubyObject.class)));
                     } else {
@@ -2087,16 +2111,20 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         method.aload(StandardASMCompiler.THIS);
         loadThreadContext();
         if (receiverCallback == null) {
-            // if there's no receiver, there could potentially be a superclass like class Foo << self
-            // so we pass in self here
-            method.aload(StandardASMCompiler.SELF_INDEX);
+            // if there's no receiver, evaluate and pass in the superclass, or
+            // pass self if it no superclass
+            if (superCallback != null) {
+                superCallback.call(this);
+            } else {
+                method.aload(StandardASMCompiler.SELF_INDEX);
+            }
         } else {
             // otherwise, there's a receiver, so we pass that in directly for the sclass logic
             receiverCallback.call(this);
         }
         method.getstatic(p(Block.class), "NULL_BLOCK", ci(Block.class));
 
-        method.invokevirtual(script.getClassname(), classMethodName, StandardASMCompiler.METHOD_SIGNATURES[0]);
+        method.invokestatic(script.getClassname(), classMethodName, StandardASMCompiler.getStaticMethodSignature(script.getClassname(), 0));
     }
 
     public void defineModule(final String name, final StaticScope staticScope, final CompilerCallback pathCallback, final CompilerCallback bodyCallback, final ASTInspector inspector) {
@@ -2170,7 +2198,7 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
         method.getstatic(p(IRubyObject.class), "NULL_ARRAY", ci(IRubyObject[].class));
         method.getstatic(p(Block.class), "NULL_BLOCK", ci(Block.class));
 
-        method.invokevirtual(script.getClassname(), moduleMethodName, StandardASMCompiler.METHOD_SIGNATURES[4]);
+        method.invokestatic(script.getClassname(), moduleMethodName, StandardASMCompiler.getStaticMethodSignature(script.getClassname(), 4));
     }
 
     public void unwrapPassedBlock() {
@@ -2202,16 +2230,17 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
     }
 
     public void callZSuper(CompilerCallback closure) {
-        loadRuntime();
-        loadThreadContext();
-        if (closure != null) {
-            closure.call(this);
-        } else {
-            method.getstatic(p(Block.class), "NULL_BLOCK", ci(Block.class));
-        }
-        loadSelf();
+        ArgumentsCallback argsCallback = new ArgumentsCallback() {
+            public int getArity() {
+                return -1;
+            }
 
-        invokeUtilityMethod("callZSuper", sig(IRubyObject.class, params(Ruby.class, ThreadContext.class, Block.class, IRubyObject.class)));
+            public void call(BodyCompiler context) {
+                loadThreadContext();
+                invokeUtilityMethod("getArgValues", sig(IRubyObject[].class, ThreadContext.class));
+            }
+        };
+        getInvocationCompiler().invokeDynamic(null, null, argsCallback, CallType.SUPER, closure, false);
     }
 
     public void checkIsExceptionHandled(ArgumentsCallback rescueArgs) {
@@ -2221,16 +2250,16 @@ public abstract class BaseBodyCompiler implements BodyCompiler {
 
         switch (rescueArgs.getArity()) {
         case 1:
-            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Exception.class, IRubyObject.class, ThreadContext.class));
+            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Throwable.class, IRubyObject.class, ThreadContext.class));
             break;
         case 2:
-            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Exception.class, IRubyObject.class, IRubyObject.class, ThreadContext.class));
+            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Throwable.class, IRubyObject.class, IRubyObject.class, ThreadContext.class));
             break;
         case 3:
-            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Exception.class, IRubyObject.class, IRubyObject.class, IRubyObject.class, ThreadContext.class));
+            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Throwable.class, IRubyObject.class, IRubyObject.class, IRubyObject.class, ThreadContext.class));
             break;
         default:
-            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Exception.class, IRubyObject[].class, ThreadContext.class));
+            invokeUtilityMethod("isJavaExceptionHandled", sig(IRubyObject.class, Throwable.class, IRubyObject[].class, ThreadContext.class));
         }
     }
 
