@@ -21,6 +21,7 @@ import org.jruby.ast.executable.AbstractScript;
 import org.jruby.compiler.ASTInspector;
 import org.jruby.compiler.CacheCompiler;
 import org.jruby.compiler.CompilerCallback;
+import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.parser.StaticScope;
 import org.jruby.runtime.BlockBody;
 import org.jruby.runtime.CallSite;
@@ -30,7 +31,6 @@ import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 import org.jruby.util.ByteList;
 import org.objectweb.asm.Label;
-import org.objectweb.asm.Opcodes;
 import static org.jruby.util.CodegenUtils.*;
 
 /**
@@ -51,10 +51,15 @@ public class InheritedCacheCompiler implements CacheCompiler {
     int inheritedStringCount = 0;
     int inheritedRegexpCount = 0;
     int inheritedBigIntegerCount = 0;
+    int inheritedVariableReaderCount = 0;
+    int inheritedVariableWriterCount = 0;
     int inheritedFixnumCount = 0;
     int inheritedConstantCount = 0;
     int inheritedBlockBodyCount = 0;
     int inheritedBlockCallbackCount = 0;
+    int inheritedMethodCount = 0;
+
+    boolean runtimeCacheInited = false;
     
     public InheritedCacheCompiler(StandardASMCompiler scriptCompiler) {
         this.scriptCompiler = scriptCompiler;
@@ -140,23 +145,22 @@ public class InheritedCacheCompiler implements CacheCompiler {
         Label alreadyCompiled = new Label();
 
         method.loadThis();
+        method.method.getfield(scriptCompiler.getClassname(), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
         method.method.pushInt(index);
-        method.method.invokevirtual(scriptCompiler.getClassname(), "getRegexp", sig(RubyRegexp.class, int.class));
+        method.method.invokevirtual(p(AbstractScript.RuntimeCache.class), "getRegexp", sig(RubyRegexp.class, int.class));
+        method.method.dup();
 
         method.ifNotNull(alreadyCompiled);
 
+        method.method.pop();
         method.loadThis();
-        method.loadRuntime();
+        method.method.getfield(scriptCompiler.getClassname(), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
         method.method.pushInt(index);
         createStringCallback.call(method);
-        method.method.invokevirtual(p(RubyString.class), "getByteList", sig(ByteList.class));
         method.method.ldc(options);
-        method.method.invokevirtual(scriptCompiler.getClassname(), "cacheRegexp", sig(void.class, Ruby.class, int.class, ByteList.class, int.class));
+        method.method.invokevirtual(p(AbstractScript.RuntimeCache.class), "cacheRegexp", sig(RubyRegexp.class, int.class, RubyString.class, int.class));
 
         method.method.label(alreadyCompiled);
-        method.loadThis();
-        method.method.pushInt(index);
-        method.method.invokevirtual(scriptCompiler.getClassname(), "getRegexp", sig(RubyRegexp.class, int.class));
     }
     
     public void cacheFixnum(BaseBodyCompiler method, long value) {
@@ -275,6 +279,40 @@ public class InheritedCacheCompiler implements CacheCompiler {
         }
     }
 
+    public void cachedGetVariable(BaseBodyCompiler method, String name) {
+        method.loadThis();
+        method.loadRuntime();
+        int index = inheritedVariableReaderCount++;
+        if (index < AbstractScript.NUMBERED_VARIABLEREADER_COUNT) {
+            method.method.ldc(name);
+            method.loadSelf();
+            method.method.invokevirtual(scriptCompiler.getClassname(), "getVariable" + index, sig(IRubyObject.class, Ruby.class, String.class, IRubyObject.class));
+        } else {
+            method.method.pushInt(index);
+            method.method.ldc(name);
+            method.loadSelf();
+            method.method.invokevirtual(scriptCompiler.getClassname(), "getVariable", sig(IRubyObject.class, Ruby.class, int.class, String.class, IRubyObject.class));
+        }
+    }
+
+    public void cachedSetVariable(BaseBodyCompiler method, String name, CompilerCallback valueCallback) {
+        method.loadThis();
+        method.loadRuntime();
+        int index = inheritedVariableWriterCount++;
+        if (index < AbstractScript.NUMBERED_VARIABLEWRITER_COUNT) {
+            method.method.ldc(name);
+            method.loadSelf();
+            valueCallback.call(method);
+            method.method.invokevirtual(scriptCompiler.getClassname(), "setVariable" + index, sig(IRubyObject.class, Ruby.class, String.class, IRubyObject.class, IRubyObject.class));
+        } else {
+            method.method.pushInt(index);
+            method.method.ldc(name);
+            method.loadSelf();
+            valueCallback.call(method);
+            method.method.invokevirtual(scriptCompiler.getClassname(), "setVariable", sig(IRubyObject.class, Ruby.class, int.class, String.class, IRubyObject.class, IRubyObject.class));
+        }
+    }
+
     public void cacheClosure(BaseBodyCompiler method, String closureMethod, int arity, StaticScope scope, boolean hasMultipleArgsHead, NodeType argsNodeId, ASTInspector inspector) {
         // build scope names string
         StringBuffer scopeNames = new StringBuffer();
@@ -355,102 +393,195 @@ public class InheritedCacheCompiler implements CacheCompiler {
         inheritedBlockCallbackCount++;
     }
 
+    public void cacheMethod(BaseBodyCompiler method, String methodName) {
+        method.loadThis();
+        method.loadThreadContext();
+        method.loadSelf();
+
+        if (inheritedMethodCount < AbstractScript.NUMBERED_METHOD_COUNT) {
+            method.method.ldc(methodName);
+            method.method.invokevirtual(scriptCompiler.getClassname(), "getMethod" + inheritedMethodCount, sig(DynamicMethod.class, ThreadContext.class, IRubyObject.class, String.class));
+        } else {
+            method.method.pushInt(inheritedMethodCount);
+            method.method.ldc(methodName);
+            method.method.invokevirtual(scriptCompiler.getClassname(), "getMethod", sig(DynamicMethod.class, ThreadContext.class, IRubyObject.class, int.class, String.class));
+        }
+
+        inheritedMethodCount++;
+    }
+
+    public void cacheMethod(BaseBodyCompiler method, String methodName, int receiverLocal) {
+        method.loadThis();
+        method.loadThreadContext();
+        method.method.aload(receiverLocal);
+
+        if (inheritedMethodCount < AbstractScript.NUMBERED_METHOD_COUNT) {
+            method.method.ldc(methodName);
+            method.method.invokevirtual(scriptCompiler.getClassname(), "getMethod" + inheritedMethodCount, sig(DynamicMethod.class, ThreadContext.class, IRubyObject.class, String.class));
+        } else {
+            method.method.pushInt(inheritedMethodCount);
+            method.method.ldc(methodName);
+            method.method.invokevirtual(scriptCompiler.getClassname(), "getMethod", sig(DynamicMethod.class, ThreadContext.class, IRubyObject.class, int.class, String.class));
+        }
+
+        inheritedMethodCount++;
+    }
+
     public void finish() {
         SkinnyMethodAdapter initMethod = scriptCompiler.getInitMethod();
 
         // generate call sites initialization code
         int size = callSiteList.size();
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+            
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+            initMethod.dup();
             initMethod.pushInt(size);
-            initMethod.anewarray(p(CallSite.class));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initCallSites", sig(void.class, int.class));
+            initMethod.getfield(p(AbstractScript.RuntimeCache.class), "callSites", ci(CallSite[].class));
             
             for (int i = size - 1; i >= 0; i--) {
                 String name = callSiteList.get(i);
                 CallType callType = callTypeList.get(i);
 
                 initMethod.pushInt(i);
-                initMethod.ldc(name);
                 if (callType.equals(CallType.NORMAL)) {
+                    initMethod.ldc(name);
                     initMethod.invokestatic(scriptCompiler.getClassname(), "setCallSite", sig(CallSite[].class, params(CallSite[].class, int.class, String.class)));
                 } else if (callType.equals(CallType.FUNCTIONAL)) {
+                    initMethod.ldc(name);
                     initMethod.invokestatic(scriptCompiler.getClassname(), "setFunctionalCallSite", sig(CallSite[].class, params(CallSite[].class, int.class, String.class)));
                 } else if (callType.equals(CallType.VARIABLE)) {
+                    initMethod.ldc(name);
                     initMethod.invokestatic(scriptCompiler.getClassname(), "setVariableCallSite", sig(CallSite[].class, params(CallSite[].class, int.class, String.class)));
+                } else if (callType.equals(CallType.SUPER)) {
+                    initMethod.invokestatic(scriptCompiler.getClassname(), "setSuperCallSite", sig(CallSite[].class, params(CallSite[].class, int.class)));
                 }
             }
-            initMethod.putfield(scriptCompiler.getClassname(), "callSites", ci(CallSite[].class));
+
+            initMethod.pop();
         }
 
         size = scopeCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initScopes", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initScopes", sig(void.class, params(int.class)));
         }
 
         // generate symbols initialization code
         size = inheritedSymbolCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initSymbols", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initSymbols", sig(void.class, params(int.class)));
         }
 
         // generate fixnums initialization code
         size = inheritedFixnumCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initFixnums", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initFixnums", sig(void.class, params(int.class)));
         }
 
         // generate constants initialization code
         size = inheritedConstantCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initConstants", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initConstants", sig(void.class, params(int.class)));
         }
 
         // generate regexps initialization code
         size = inheritedRegexpCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initRegexps", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initRegexps", sig(void.class, params(int.class)));
         }
 
         // generate regexps initialization code
         size = inheritedBigIntegerCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initBigIntegers", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initBigIntegers", sig(void.class, params(int.class)));
+        }
+
+        // generate variable readers initialization code
+        size = inheritedVariableReaderCount;
+        if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
+            initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+            initMethod.pushInt(size);
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initVariableReaders", sig(void.class, params(int.class)));
+        }
+
+        // generate variable writers initialization code
+        size = inheritedVariableWriterCount;
+        if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
+            initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+            initMethod.pushInt(size);
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initVariableWriters", sig(void.class, params(int.class)));
         }
 
         // generate block bodies initialization code
         size = inheritedBlockBodyCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initBlockBodies", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initBlockBodies", sig(void.class, params(int.class)));
         }
 
         // generate block bodies initialization code
         size = inheritedBlockCallbackCount;
         if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initBlockCallbacks", sig(void.class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initBlockCallbacks", sig(void.class, params(int.class)));
         }
 
         // generate bytelists initialization code
         size = inheritedStringCount;
         if (inheritedStringCount > 0) {
+            ensureRuntimeCacheInited(initMethod);
+
             initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
             initMethod.pushInt(size);
-            initMethod.invokevirtual(scriptCompiler.getClassname(), "initStrings", sig(ByteList[].class, params(int.class)));
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initStrings", sig(ByteList[].class, params(int.class)));
 
             for (Map.Entry<String, Integer> entry : stringIndices.entrySet()) {
                 initMethod.ldc(entry.getValue());
@@ -461,5 +592,26 @@ public class InheritedCacheCompiler implements CacheCompiler {
             initMethod.pop();
         }
 
+        // generate method cache initialization code
+        size = inheritedMethodCount;
+        if (size != 0) {
+            ensureRuntimeCacheInited(initMethod);
+            
+            initMethod.aload(0);
+            initMethod.getfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+            initMethod.pushInt(size);
+            initMethod.invokevirtual(p(AbstractScript.RuntimeCache.class), "initMethodCache", sig(void.class, params(int.class)));
+        }
+    }
+
+    private void ensureRuntimeCacheInited(SkinnyMethodAdapter initMethod) {
+        if (!runtimeCacheInited) {
+            initMethod.aload(0);
+            initMethod.newobj(p(AbstractScript.RuntimeCache.class));
+            initMethod.dup();
+            initMethod.invokespecial(p(AbstractScript.RuntimeCache.class), "<init>", sig(void.class));
+            initMethod.putfield(p(AbstractScript.class), "runtimeCache", ci(AbstractScript.RuntimeCache.class));
+            runtimeCacheInited = true;
+        }
     }
 }
