@@ -6,22 +6,28 @@ import com.kenai.jffi.Function;
 import com.kenai.jffi.HeapInvocationBuffer;
 import com.kenai.jffi.InvocationBuffer;
 import com.kenai.jffi.Invoker;
-import com.kenai.jffi.ObjectBuffer;
+import com.kenai.jffi.ArrayFlags;
 import org.jruby.Ruby;
+import org.jruby.RubyBoolean;
+import org.jruby.RubyHash;
 import org.jruby.RubyModule;
 import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
+import org.jruby.RubySymbol;
+import org.jruby.ext.ffi.AbstractMemory;
 import org.jruby.ext.ffi.ArrayMemoryIO;
-import org.jruby.ext.ffi.BasePointer;
 import org.jruby.ext.ffi.Buffer;
+import org.jruby.ext.ffi.CallbackInfo;
 import org.jruby.ext.ffi.DirectMemoryIO;
+import org.jruby.ext.ffi.MemoryIO;
 import org.jruby.ext.ffi.MemoryPointer;
-import org.jruby.ext.ffi.NativeParam;
 import org.jruby.ext.ffi.NativeType;
-import org.jruby.ext.ffi.NullMemoryIO;
 import org.jruby.ext.ffi.Platform;
 import org.jruby.ext.ffi.Pointer;
 import org.jruby.ext.ffi.Struct;
+import org.jruby.ext.ffi.StructByValue;
+import org.jruby.ext.ffi.StructLayout;
+import org.jruby.ext.ffi.Type;
 import org.jruby.ext.ffi.Util;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.runtime.ThreadContext;
@@ -37,13 +43,20 @@ public final class DefaultMethodFactory {
     public static final DefaultMethodFactory getFactory() {
         return SingletonHolder.INSTANCE;
     }
+    
     DynamicMethod createMethod(RubyModule module, Function function, 
-            NativeType returnType, NativeParam[] parameterTypes, CallingConvention convention) {
+            Type returnType, Type[] parameterTypes, CallingConvention convention, IRubyObject enums) {
+
         FunctionInvoker functionInvoker = getFunctionInvoker(returnType);
+
         ParameterMarshaller[] marshallers = new ParameterMarshaller[parameterTypes.length];
         for (int i = 0; i < parameterTypes.length; ++i)  {
-            marshallers[i] = getMarshaller(parameterTypes[i], convention);
+            marshallers[i] = getMarshaller(parameterTypes[i], convention, enums);
+            if (marshallers[i] == null) {
+                throw module.getRuntime().newTypeError("Could not create marshaller for " + parameterTypes[i]);
+            }
         }
+
         /*
          * If there is exactly _one_ callback argument to the function,
          * then a block can be given and automatically subsituted for the callback
@@ -61,7 +74,32 @@ public final class DefaultMethodFactory {
                 return new CallbackMethodWithBlock(module, function, functionInvoker, marshallers, cbindex);
             }
         }
-        switch (parameterTypes.length) {
+
+        //
+        // Determine if the parameter might be passed as a 32bit int parameter.
+        // This just applies to buffer/pointer types.
+        //
+        FastIntMethodFactory fastIntFactory = FastIntMethodFactory.getFactory();
+        boolean canBeFastInt = enums.isNil() && parameterTypes.length <= 3 && fastIntFactory.isFastIntResult(returnType);
+        for (int i = 0; canBeFastInt && i < parameterTypes.length; ++i) {
+            if (!(parameterTypes[i] instanceof Type.Builtin) || marshallers[i].needsInvocationSession()) {
+                canBeFastInt = false;
+            } else {
+                switch (parameterTypes[i].getNativeType()) {
+                    case POINTER:
+                    case BUFFER_IN:
+                    case BUFFER_OUT:
+                    case BUFFER_INOUT:
+                        canBeFastInt = Platform.getPlatform().addressSize() == 32;
+                        break;
+                    default:
+                        canBeFastInt = fastIntFactory.isFastIntParam(parameterTypes[i]);
+                        break;
+                }
+            }
+        }
+
+        if (!canBeFastInt) switch (parameterTypes.length) {
             case 0:
                 return new DefaultMethodZeroArg(module, function, functionInvoker);
             case 1:
@@ -73,28 +111,67 @@ public final class DefaultMethodFactory {
             default:
                 return new DefaultMethod(module, function, functionInvoker, marshallers);
         }
+        //
+        // Set up for potentially fast-int operations
+        //
+        
+        IntResultConverter resultConverter = fastIntFactory.getIntResultConverter(returnType);
+        IntParameterConverter[] intParameterConverters = new IntParameterConverter[parameterTypes.length];
+        for (int i = 0; i < parameterTypes.length; ++i) {
+            intParameterConverters[i] = fastIntFactory.getIntParameterConverter(parameterTypes[i]);
+        }
+        switch (parameterTypes.length) {
+            case 0:
+                return new FastIntMethodZeroArg(module, function, resultConverter, intParameterConverters);
+            case 1:
+                return new FastIntPointerMethodOneArg(module, function, resultConverter, 
+                        intParameterConverters, marshallers);
+            case 2:
+                return new FastIntPointerMethodTwoArg(module, function, resultConverter,
+                        intParameterConverters, marshallers);
+            case 3:
+                return new FastIntPointerMethodThreeArg(module, function, resultConverter,
+                        intParameterConverters, marshallers);
+        }
+        throw new IllegalArgumentException("Parameter types not supported");
     }
+
+    static FunctionInvoker getFunctionInvoker(Type returnType) {
+        if (returnType instanceof Type.Builtin) {
+            return getFunctionInvoker(returnType.getNativeType());
+        } else if (returnType instanceof CallbackInfo) {
+            return new CallbackInvoker((CallbackInfo) returnType);
+        } else if (returnType instanceof org.jruby.ext.ffi.Enum) {
+            return new EnumInvoker((org.jruby.ext.ffi.Enum) returnType);
+        } else if (returnType instanceof StructByValue) {
+            return new StructByValueInvoker((StructByValue) returnType);
+        }
+        throw returnType.getRuntime().newArgumentError("Cannot get FunctionInvoker for " + returnType);
+    }
+
     static FunctionInvoker getFunctionInvoker(NativeType returnType) {
         switch (returnType) {
             case VOID:
                 return VoidInvoker.INSTANCE;
+            case BOOL:
+                return BooleanInvoker.INSTANCE;
             case POINTER:
                 return PointerInvoker.INSTANCE;
-            case INT8:
+            case CHAR:
                 return Signed8Invoker.INSTANCE;
-            case INT16:
+            case SHORT:
                 return Signed16Invoker.INSTANCE;
-            case INT32:
+            case INT:
                 return Signed32Invoker.INSTANCE;
-            case UINT8:
+            case UCHAR:
                 return Unsigned8Invoker.INSTANCE;
-            case UINT16:
+            case USHORT:
                 return Unsigned16Invoker.INSTANCE;
-            case UINT32:
+            case UINT:
                 return Unsigned32Invoker.INSTANCE;
-            case INT64:
+            case LONG_LONG:
                 return Signed64Invoker.INSTANCE;
-            case UINT64:
+            case ULONG_LONG:
                 return Unsigned64Invoker.INSTANCE;
             case LONG:
                 return Platform.getPlatform().longSize() == 32
@@ -104,9 +181,9 @@ public final class DefaultMethodFactory {
                 return Platform.getPlatform().longSize() == 32
                         ? Unsigned32Invoker.INSTANCE
                         : Unsigned64Invoker.INSTANCE;
-            case FLOAT32:
+            case FLOAT:
                 return Float32Invoker.INSTANCE;
-            case FLOAT64:
+            case DOUBLE:
                 return Float64Invoker.INSTANCE;
             case STRING:
                 return StringInvoker.INSTANCE;
@@ -120,15 +197,47 @@ public final class DefaultMethodFactory {
      * @param type The native type to convert to.
      * @return A new <tt>Marshaller</tt>
      */
-    static final ParameterMarshaller getMarshaller(NativeParam type, CallingConvention convention) {
-        if (type instanceof NativeType) {
-            return getMarshaller((NativeType) type);
+    static final ParameterMarshaller getMarshaller(Type type, CallingConvention convention, IRubyObject enums) {
+        if (type instanceof Type.Builtin) {
+            return enums != null && !enums.isNil() ? getEnumMarshaller(type, enums) : getMarshaller(type.getNativeType());
         } else if (type instanceof org.jruby.ext.ffi.CallbackInfo) {
             return new CallbackMarshaller((org.jruby.ext.ffi.CallbackInfo) type, convention);
+        } else if (type instanceof org.jruby.ext.ffi.Enum) {
+            return getEnumMarshaller(type, type.callMethod(type.getRuntime().getCurrentContext(), "to_hash"));
+        } else if (type instanceof org.jruby.ext.ffi.StructByValue) {
+            return new StructByValueMarshaller((org.jruby.ext.ffi.StructByValue) type);
         } else {
             return null;
         }
     }
+
+    /**
+     * Gets a marshaller to convert from a ruby type to a native type.
+     *
+     * @param type The native type to convert to.
+     * @param enums The enum map
+     * @return A new <tt>ParameterMarshaller</tt>
+     */
+    static final ParameterMarshaller getEnumMarshaller(Type type, IRubyObject enums) {
+        switch (type.getNativeType()) {
+            case CHAR:
+            case UCHAR:
+            case SHORT:
+            case USHORT:
+            case INT:
+            case UINT:
+            case LONG_LONG:
+            case ULONG_LONG:
+                if (!(enums instanceof RubyHash)) {
+                    throw type.getRuntime().newArgumentError("wrong argument type "
+                            + enums.getMetaClass().getName() + " (expected Hash)");
+                }
+                return new EnumMarshaller(getMarshaller(type.getNativeType()), (RubyHash) enums);
+            default:
+                return getMarshaller(type.getNativeType());
+        }
+    }
+
     /**
      * Gets a marshaller to convert from a ruby type to a native type.
      *
@@ -137,21 +246,23 @@ public final class DefaultMethodFactory {
      */
     static final ParameterMarshaller getMarshaller(NativeType type) {
         switch (type) {
-            case INT8:
+            case BOOL:
+                return BooleanMarshaller.INSTANCE;
+            case CHAR:
                 return Signed8Marshaller.INSTANCE;
-            case UINT8:
+            case UCHAR:
                 return Unsigned8Marshaller.INSTANCE;
-            case INT16:
+            case SHORT:
                 return Signed16Marshaller.INSTANCE;
-            case UINT16:
+            case USHORT:
                 return Unsigned16Marshaller.INSTANCE;
-            case INT32:
+            case INT:
                 return Signed32Marshaller.INSTANCE;
-            case UINT32:
+            case UINT:
                 return Unsigned32Marshaller.INSTANCE;
-            case INT64:
+            case LONG_LONG:
                 return Signed64Marshaller.INSTANCE;
-            case UINT64:
+            case ULONG_LONG:
                 return Unsigned64Marshaller.INSTANCE;
             case LONG:
                 return Platform.getPlatform().longSize() == 32
@@ -161,9 +272,9 @@ public final class DefaultMethodFactory {
                 return Platform.getPlatform().longSize() == 32
                         ? Signed32Marshaller.INSTANCE
                         : Unsigned64Marshaller.INSTANCE;
-            case FLOAT32:
+            case FLOAT:
                 return Float32Marshaller.INSTANCE;
-            case FLOAT64:
+            case DOUBLE:
                 return Float64Marshaller.INSTANCE;
             case STRING:
                 return StringMarshaller.INSTANCE;
@@ -193,6 +304,34 @@ public final class DefaultMethodFactory {
         }
         public static final FunctionInvoker INSTANCE = new VoidInvoker();
     }
+
+    /**
+     * Invokes the native function with a boolean return value.
+     * Returns a Boolean to ruby.
+     */
+    private static final class BooleanInvoker extends BaseInvoker {
+        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+            return runtime.newBoolean(invoker.invokeInt(function, args) != 0);
+        }
+        public static final FunctionInvoker INSTANCE = new BooleanInvoker();
+    }
+
+    /**
+     * Invokes the native function with a native int type, and converts to a symbol
+     */
+    private static final class EnumInvoker extends BaseInvoker {
+        private final org.jruby.ext.ffi.Enum returnType;
+
+        public EnumInvoker(org.jruby.ext.ffi.Enum returnType) {
+            this.returnType = returnType;
+        }
+
+        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+            return returnType.callMethod(runtime.getCurrentContext(), "find",
+                    runtime.newFixnum(invoker.invokeInt(function, args)));
+        }
+    }
+
     /**
      * Invokes the native function with n signed 8 bit integer return value.
      * Returns a Fixnum to ruby.
@@ -309,7 +448,7 @@ public final class DefaultMethodFactory {
     private static final class PointerInvoker extends BaseInvoker {
         public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
             final long address = invoker.invokeAddress(function, args);
-            return new BasePointer(runtime, address != 0 ? new NativeMemoryIO(address) : new NullMemoryIO(runtime));
+            return new Pointer(runtime, NativeMemoryIO.wrap(runtime, address));
         }
         public static final FunctionInvoker INSTANCE = new PointerInvoker();
     }
@@ -321,23 +460,51 @@ public final class DefaultMethodFactory {
     private static final class StringInvoker extends BaseInvoker {
         private static final com.kenai.jffi.MemoryIO IO = com.kenai.jffi.MemoryIO.getInstance();
 
+        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+            return FFIUtil.getString(runtime, invoker.invokeAddress(function, args));
+        }
+        public static final FunctionInvoker INSTANCE = new StringInvoker();
+    }
+
+    /**
+     * Invokes the native function with a native struct return value.
+     * Returns a FFI::Struct instance to ruby.
+     */
+    private static final class StructByValueInvoker extends BaseInvoker {
+        private static final com.kenai.jffi.MemoryIO IO = com.kenai.jffi.MemoryIO.getInstance();
+        private final StructByValue info;
+
+        public StructByValueInvoker(StructByValue info) {
+            this.info = info;
+        }
+
+        public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
+            return info.newStruct(runtime, invoker.invokeStruct(function, args), 0);
+        }
+    }
+
+    /**
+     * Invokes the native function with a callback/function pointer return value.
+     * Returns a {@link Invoker} to ruby.
+     */
+    private static final class CallbackInvoker extends BaseInvoker {
+        private static final com.kenai.jffi.MemoryIO IO = com.kenai.jffi.MemoryIO.getInstance();
+        private final Type returnType;
+        private final Type[] parameterTypes;
+
+        public CallbackInvoker(CallbackInfo cbInfo) {
+            this.returnType = cbInfo.getReturnType();
+            this.parameterTypes = cbInfo.getParameterTypes();
+        }
+        
+
         public final IRubyObject invoke(Ruby runtime, Function function, HeapInvocationBuffer args) {
             long address = invoker.invokeAddress(function, args);
             if (address == 0) {
                 return runtime.getNil();
             }
-            int len = (int) IO.getStringLength(address);
-            if (len == 0) {
-                return RubyString.newEmptyString(runtime);
-            }
-            byte[] bytes = new byte[len];
-            IO.getByteArray(address, bytes, 0, len);
-            
-            RubyString s =  RubyString.newStringShared(runtime, bytes);
-            s.setTaint(true);
-            return s;
+            return new JFFIInvoker(runtime, address, returnType, parameterTypes);
         }
-        public static final FunctionInvoker INSTANCE = new StringInvoker();
     }
 
     /*------------------------------------------------------------------------*/
@@ -346,6 +513,48 @@ public final class DefaultMethodFactory {
             return false;
         }
     }
+    /**
+     * Converts a ruby Enum into an native integer.
+     */
+    static final class EnumMarshaller extends BaseMarshaller {
+        private final ParameterMarshaller marshaller;
+        private final RubyHash enums;
+
+        public EnumMarshaller(ParameterMarshaller marshaller, RubyHash enums) {
+            this.marshaller = marshaller;
+            this.enums = enums;
+        }
+
+        public final void marshal(ThreadContext context, InvocationBuffer buffer, IRubyObject parameter) {
+            if (parameter instanceof RubySymbol) {
+                parameter = enums.fastARef(parameter);
+                if (parameter.isNil()) {
+                    throw context.getRuntime().newArgumentError("wrong argument.  Could not locate enum value for " + parameter);
+                }
+            }
+            marshaller.marshal(context, buffer, parameter);
+        }
+        public void marshal(Invocation invocation, InvocationBuffer buffer, IRubyObject parameter) {
+            marshal(invocation.getThreadContext(), buffer, parameter);
+        }
+    }
+
+    /**
+     * Converts a ruby Boolean into an 32 bit native integer.
+     */
+    static final class BooleanMarshaller extends BaseMarshaller {
+        public final void marshal(ThreadContext context, InvocationBuffer buffer, IRubyObject parameter) {
+            buffer.putInt(parameter.isTrue() ? 1 : 0);
+        }
+        public void marshal(Invocation invocation, InvocationBuffer buffer, IRubyObject parameter) {
+            if (!(parameter instanceof RubyBoolean)) {
+                throw invocation.getThreadContext().getRuntime().newTypeError("wrong argument type.  Expected true or false");
+            }
+            buffer.putInt(parameter.isTrue() ? 1 : 0);
+        }
+        public static final ParameterMarshaller INSTANCE = new BooleanMarshaller();
+    }
+
     /**
      * Converts a ruby Fixnum into an 8 bit native integer.
      */
@@ -480,17 +689,17 @@ public final class DefaultMethodFactory {
      * Converts a ruby Buffer into a native address.
      */
     static final class BufferMarshaller extends BaseMarshaller {
-        static final ParameterMarshaller IN = new BufferMarshaller(ObjectBuffer.IN);
-        static final ParameterMarshaller OUT = new BufferMarshaller(ObjectBuffer.OUT);
-        static final ParameterMarshaller INOUT = new BufferMarshaller(ObjectBuffer.IN | ObjectBuffer.OUT);
+        static final ParameterMarshaller IN = new BufferMarshaller(ArrayFlags.IN);
+        static final ParameterMarshaller OUT = new BufferMarshaller(ArrayFlags.OUT);
+        static final ParameterMarshaller INOUT = new BufferMarshaller(ArrayFlags.IN | ArrayFlags.OUT);
         private final int flags;
         public BufferMarshaller(int flags) {
             this.flags = flags;
         }
         private static final int bufferFlags(Buffer buffer) {
             int f = buffer.getInOutFlags();
-            return ((f & Buffer.IN) != 0 ? ObjectBuffer.IN: 0)
-                    | ((f & Buffer.OUT) != 0 ? ObjectBuffer.OUT : 0);
+            return ((f & Buffer.IN) != 0 ? ArrayFlags.IN: 0)
+                    | ((f & Buffer.OUT) != 0 ? ArrayFlags.OUT : 0);
         }
         @Override
         public boolean needsInvocationSession() {
@@ -524,17 +733,25 @@ public final class DefaultMethodFactory {
                 buffer.putAddress(0L);
             } else if (parameter instanceof RubyString) {
                 ByteList bl = ((RubyString) parameter).getByteList();
-                buffer.putArray(bl.unsafeBytes(), bl.begin(), bl.length(), flags | ObjectBuffer.ZERO_TERMINATE);
+                buffer.putArray(bl.unsafeBytes(), bl.begin(), bl.length(), flags | ArrayFlags.NULTERMINATE);
+
             } else if (parameter.respondsTo("to_ptr")) {
-                IRubyObject ptr = parameter.callMethod(context, "to_ptr");
-                if (ptr instanceof Pointer) {
-                    buffer.putAddress(getAddress((Pointer) ptr));
-                } else if (ptr instanceof Buffer) {
-                    addBufferParameter(buffer, ptr, flags);
-                } else if (ptr.isNil()) {
-                    buffer.putAddress(0L);
-                } else {
-                    throw context.getRuntime().newArgumentError("to_ptr returned an invalid pointer");
+                final int MAXRECURSE = 4;
+                for (int depth = 0; depth < MAXRECURSE; ++depth) {
+                    IRubyObject ptr = parameter.callMethod(context, "to_ptr");
+                    if (ptr instanceof Pointer) {
+                        buffer.putAddress(getAddress((Pointer) ptr));
+                    } else if (ptr instanceof Buffer) {
+                        addBufferParameter(buffer, ptr, flags);
+                    } else if (ptr.isNil()) {
+                        buffer.putAddress(0L);
+                    } else if (depth < MAXRECURSE && ptr.respondsTo("to_ptr")) {
+                        parameter = ptr;
+                        continue;
+                    } else {
+                        throw context.getRuntime().newArgumentError("to_ptr returned an invalid pointer");
+                    }
+                    break;
                 }
             } else {
                 throw context.getRuntime().newArgumentError("Invalid buffer/pointer parameter");
@@ -555,7 +772,7 @@ public final class DefaultMethodFactory {
                 Util.checkStringSafety(context.getRuntime(), parameter);
                 ByteList bl = ((RubyString) parameter).getByteList();
                 buffer.putArray(bl.unsafeBytes(), bl.begin(), bl.length(),
-                        ObjectBuffer.IN | ObjectBuffer.ZERO_TERMINATE);
+                        ArrayFlags.IN | ArrayFlags.NULTERMINATE);
             } else if (parameter.isNil()) {
                 buffer.putAddress(0);
             } else {
@@ -568,4 +785,49 @@ public final class DefaultMethodFactory {
         }
         public static final ParameterMarshaller INSTANCE = new StringMarshaller();
     }
+
+    /**
+     * Converts a ruby String into a native pointer.
+     */
+    static final class StructByValueMarshaller extends BaseMarshaller {
+        private final StructLayout layout;
+        public StructByValueMarshaller(org.jruby.ext.ffi.StructByValue sbv) {
+            layout = sbv.getStructLayout();
+        }
+
+
+        public final void marshal(ThreadContext context, InvocationBuffer buffer, IRubyObject parameter) {
+            if (!(parameter instanceof Struct)) {
+                throw context.getRuntime().newTypeError("wrong argument type "
+                        + parameter.getMetaClass().getName() + " (expected instance of FFI::Struct)");
+            }
+
+            IRubyObject memory = ((Struct) parameter).getMemory();
+            if (!(memory instanceof AbstractMemory)) {
+                throw context.getRuntime().newTypeError("wrong struct memory type "
+                        + memory.getMetaClass().getName());
+            }
+
+            if (((AbstractMemory) memory).getSize() < layout.getSize()) {
+                throw context.getRuntime().newArgumentError("struct memory too small for parameter");
+            }
+
+            MemoryIO io = ((AbstractMemory) memory).getMemoryIO();
+            if (io instanceof DirectMemoryIO) {
+                if (io.isNull()) {
+                    throw context.getRuntime().newRuntimeError("Cannot use a NULL pointer as a struct by value argument");
+                }
+                buffer.putStruct(((DirectMemoryIO) io).getAddress());
+            } else if (io instanceof ArrayMemoryIO) {
+                ArrayMemoryIO aio = (ArrayMemoryIO) io;
+                buffer.putStruct(aio.array(), aio.arrayOffset());
+            } else {
+                throw context.getRuntime().newRuntimeError("Invalid struct memory");
+            }
+        }
+
+        public final void marshal(Invocation invocation, InvocationBuffer buffer, IRubyObject parameter) {
+            marshal(invocation.getThreadContext(), buffer, parameter);
+        }
+    }
 }
