@@ -43,6 +43,7 @@ import java.net.ServerSocket;
 import java.net.DatagramSocket;
 import java.net.SocketAddress;
 import java.net.InetSocketAddress;
+import org.jruby.CompatVersion;
 import org.jruby.util.io.OpenFile;
 import org.jruby.Ruby;
 import org.jruby.RubyBoolean;
@@ -77,6 +78,9 @@ public class RubyBasicSocket extends RubyIO {
         rb_cBasicSocket.defineAnnotatedMethods(RubyBasicSocket.class);
     }
 
+    // By default we always reverse lookup unless do_not_reverse_lookup set.
+    private boolean doNotReverseLookup = false;
+
     public RubyBasicSocket(Ruby runtime, RubyClass type) {
         super(runtime, type);
     }
@@ -601,6 +605,21 @@ public class RubyBasicSocket extends RubyIO {
         return close();
     }
 
+    protected boolean doNotReverseLookup(ThreadContext context) {
+        return context.getRuntime().isDoNotReverseLookupEnabled() || doNotReverseLookup;
+    }
+
+    @JRubyMethod(compat = CompatVersion.RUBY1_9)
+    public IRubyObject do_not_reverse_lookup19(ThreadContext context) {
+        return context.getRuntime().newBoolean(doNotReverseLookup);
+    }
+
+    @JRubyMethod(name = "do_not_reverse_lookup=", compat = CompatVersion.RUBY1_9)
+    public IRubyObject set_do_not_reverse_lookup19(ThreadContext context, IRubyObject flag) {
+        doNotReverseLookup = flag.isTrue();
+        return do_not_reverse_lookup19(context);
+    }
+
     @JRubyMethod(meta = true)
     public static IRubyObject do_not_reverse_lookup(IRubyObject recv) {
         return recv.getRuntime().isDoNotReverseLookupEnabled() ? recv.getRuntime().getTrue() : recv.getRuntime().getFalse();
