@@ -34,6 +34,8 @@ import com.kenai.constantine.platform.Sock;
 import static com.kenai.constantine.platform.Sock.*;
 import static com.kenai.constantine.platform.NameInfo.*;
 import static com.kenai.constantine.platform.IPProto.*;
+import java.io.ByteArrayOutputStream;
+import java.io.DataOutputStream;
 import java.io.FileDescriptor;
 import java.io.IOException;
 import java.net.InetAddress;
@@ -54,6 +56,7 @@ import org.jruby.RubyIO;
 import org.jruby.RubyModule;
 import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
+import org.jruby.platform.Platform;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyModule;
@@ -338,46 +341,48 @@ public class RubySocket extends RubyBasicSocket {
     }
     @JRubyMethod(name = {"pack_sockaddr_in", "sockaddr_in"}, meta = true)
     public static IRubyObject pack_sockaddr_in(ThreadContext context, IRubyObject recv, IRubyObject port, IRubyObject host) {
-        StringBuilder sb = new StringBuilder();
-        sb.append((char)16);
-        sb.append((char)2);
-
-        int iport = RubyNumeric.fix2int(port);
-
-        sb.append((char)((iport >> 8) & 0xFF));
-        sb.append((char)((iport & 0xFF)));
-
-        try {
-            String str = host.isNil() ? null : host.convertToString().toString();
-            if(str != null && "".equals(str)) {
-                sb.append((char)0);
-                sb.append((char)0);
-                sb.append((char)0);
-                sb.append((char)0);
-            } else {
-                InetAddress[] addrs = InetAddress.getAllByName(str);
-                byte[] addr = addrs[0].getAddress();
-                sb.append((char)addr[0]);
-                sb.append((char)addr[1]);
-                sb.append((char)addr[2]);
-                sb.append((char)addr[3]);
-            }
-        } catch(UnknownHostException e) {
-            throw sockerr(context.getRuntime(), "getaddrinfo: No address associated with nodename");
-        }
-
-        sb.append((char)0);
-        sb.append((char)0);
-        sb.append((char)0);
-        sb.append((char)0);
-        sb.append((char)0);
-        sb.append((char)0);
-        sb.append((char)0);
-        sb.append((char)0);
-
-        return context.getRuntime().newString(sb.toString());
+	return pack_sockaddr_in(context, recv,
+				RubyNumeric.fix2int(port),
+				host.isNil()
+				? null
+				: host.convertToString().toString());
     }
+    public static IRubyObject pack_sockaddr_in(ThreadContext context, IRubyObject recv, int iport, String host) {
+	ByteArrayOutputStream bufS = new ByteArrayOutputStream();
+	try {
+	    DataOutputStream ds = new DataOutputStream(bufS);
+	    if(Platform.IS_BSD) {
+		ds.write(16);
+		ds.write(2);
+	    } else {
+		ds.write(2);
+		ds.write(0);
+	    }
 
+	    ds.write(iport >> 8);
+	    ds.write(iport);
+
+	    try {
+		if(host != null && "".equals(host)) {
+		    ds.writeInt(0);
+		} else {
+		    InetAddress[] addrs = InetAddress.getAllByName(host);
+		    byte[] addr = addrs[0].getAddress();
+		    ds.write(addr, 0, addr.length);
+		}
+	    } catch (UnknownHostException e) {
+		throw sockerr(context.getRuntime(), "getaddrinfo: No address associated with nodename");
+	    }
+
+	    ds.writeInt(0);
+	    ds.writeInt(0);
+	} catch (IOException e) {
+	    throw sockerr(context.getRuntime(), "pack_sockaddr_in: internal error");
+	}
+
+        return context.getRuntime().newString(new ByteList(bufS.toByteArray(),
+							   false));
+    }
     @Deprecated
     public static IRubyObject unpack_sockaddr_in(IRubyObject recv, IRubyObject addr) {
         return unpack_sockaddr_in(recv.getRuntime().getCurrentContext(), recv, addr);
@@ -385,7 +390,7 @@ public class RubySocket extends RubyBasicSocket {
     @JRubyMethod(meta = true)
     public static IRubyObject unpack_sockaddr_in(ThreadContext context, IRubyObject recv, IRubyObject addr) {
         String val = addr.convertToString().toString();
-        if(val.charAt(0) != 16 || val.charAt(1) != 2) {
+        if((Platform.IS_BSD && val.charAt(0) != 16 && val.charAt(1) != 2) || (!Platform.IS_BSD && val.charAt(0) != 2)) {
             throw context.getRuntime().newArgumentError("can't resolve socket address of wrong type");
         }
         
@@ -453,6 +458,7 @@ public class RubySocket extends RubyBasicSocket {
             Ruby r = context.getRuntime();
             IRubyObject host = args[0];
             IRubyObject port = args[1];
+            boolean emptyHost = host.isNil() || host.convertToString().isEmpty();
 
             if(port instanceof RubyString) {
                 port = getservbyname(context, recv, new IRubyObject[]{port});
@@ -461,20 +467,35 @@ public class RubySocket extends RubyBasicSocket {
             //IRubyObject family = args[2];
             IRubyObject socktype = args[3];
             //IRubyObject protocol = args[4];
-            //IRubyObject flags = args[5];
+            IRubyObject flags = args[5];
+
             boolean sock_stream = true;
             boolean sock_dgram = true;
             if(!socktype.isNil()) {
                 int val = RubyNumeric.fix2int(socktype);
-                if(val == 1) {
+                if(val == SOCK_STREAM.value()) {
                     sock_dgram = false;
-                } else if(val == 2) {
+                } else if(val == SOCK_DGRAM.value()) {
                     sock_stream = false;
                 }
             }
-            InetAddress[] addrs = InetAddress.getAllByName(host.isNil() ? null : host.convertToString().toString());
+
+            // When Socket::AI_PASSIVE and host is nil, return 'any' address. 
+            InetAddress[] addrs = null; 
+            if(!flags.isNil()) {
+                // The value of 1 is for Socket::AI_PASSIVE.
+                int flag = RubyNumeric.fix2int(flags);
+                if ((flag == 1) && emptyHost ) {
+                    addrs = InetAddress.getAllByName("0.0.0.0");
+                }
+
+            }
+
+            if (addrs == null)
+                addrs = InetAddress.getAllByName(emptyHost ? null : host.convertToString().toString());
+
             List<IRubyObject> l = new ArrayList<IRubyObject>();
-            for(int i=0;i<addrs.length;i++) {
+            for(int i = 0; i < addrs.length; i++) {
                 IRubyObject[] c;
                 if(sock_dgram) {
                     c = new IRubyObject[7];
@@ -537,9 +558,20 @@ public class RubySocket extends RubyBasicSocket {
             String arg = ((RubyString)arg0).toString();
             Matcher m = STRING_ADDRESS_PATTERN.matcher(arg);
             if (!m.matches()) {
-                throw runtime.newArgumentError("invalid address string");
-            }
-            if ((host = m.group(HOST_GROUP)) == null || host.length() == 0 ||
+                IRubyObject obj = unpack_sockaddr_in(context, recv, arg0);
+                if (obj instanceof RubyArray) {
+                    List list = ((RubyArray)obj).getList();
+                    int len = list.size();
+                    if (len != 2) {
+                        throw runtime.newArgumentError("invalid address representation");
+                    }
+                    host = list.get(1).toString();
+                    port = list.get(0).toString();
+                }
+                else {
+                    throw runtime.newArgumentError("invalid address string");
+                }
+            } else if ((host = m.group(HOST_GROUP)) == null || host.length() == 0 ||
                     (port = m.group(PORT_GROUP)) == null || port.length() == 0) {
                 throw runtime.newArgumentError("invalid address string");
             }
@@ -558,6 +590,12 @@ public class RubySocket extends RubyBasicSocket {
         } else {
             host = addr.getHostAddress();
         }
+        if ((flags & NI_NUMERICSERV.value()) == 0) {
+            String serv = IANA.portToService.get(Integer.parseInt(port));
+            if (serv != null) {
+                port = serv.substring(0, serv.indexOf('/') );
+            }
+        }
         return runtime.newArray(runtime.newString(host), runtime.newString(port));
 
     }
