@@ -2,7 +2,7 @@ package org.jruby.java.invokers;
 
 import java.lang.reflect.Field;
 import org.jruby.RubyModule;
-import org.jruby.javasupport.JavaObject;
+import org.jruby.java.proxies.JavaProxy;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -16,8 +16,9 @@ public class InstanceFieldSetter extends FieldMethodOne {
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg) {
         try {
-            Object newValue = JavaUtil.convertArgumentToType(context, arg, field.getType());
-            field.set(((JavaObject) self.dataGetStruct()).getValue(), newValue);
+            JavaProxy proxy = InstanceMethodInvoker.castJavaProxy(self);
+            Object newValue = arg.toJava(field.getType());
+            field.set(proxy.getObject(), newValue);
         } catch (IllegalAccessException iae) {
             throw context.getRuntime().newTypeError("illegal access setting variable: " + iae.getMessage());
         }
