@@ -2,7 +2,6 @@ package org.jruby.java.invokers;
 
 import java.lang.reflect.Field;
 import org.jruby.RubyModule;
-import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
@@ -15,7 +14,7 @@ public class StaticFieldSetter extends FieldMethodOne {
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject arg) {
         try {
-            field.set(null, JavaUtil.convertArgumentToType(context, arg, field.getType()));
+            field.set(null, arg.toJava(field.getType()));
         } catch (IllegalAccessException iae) {
             throw context.getRuntime().newTypeError("illegal access setting variable: " + iae.getMessage());
         }
