@@ -1,13 +1,10 @@
 package org.jruby.javasupport;
 
 import org.jruby.Ruby;
-import org.jruby.RubyClass;
 import org.jruby.RubyModule;
 import org.jruby.RubyObject;
 import org.jruby.anno.JRubyMethod;
-import org.jruby.javasupport.util.RuntimeHelpers;
 import org.jruby.runtime.Block;
-import org.jruby.runtime.CallType;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
 
@@ -16,8 +13,6 @@ public class JavaProxyMethods {
         Ruby runtime = context.getRuntime();
         RubyModule javaProxyMethods = runtime.defineModule("JavaProxyMethods");
         
-        javaProxyMethods.addReadWriteAttribute(context, "java_object");
-        
         javaProxyMethods.defineAnnotatedMethods(JavaProxyMethods.class);
         
         return javaProxyMethods;
@@ -28,6 +23,18 @@ public class JavaProxyMethods {
         return recv.getMetaClass().getRealClass().fastGetInstanceVariable("@java_class");
     }
 
+    @JRubyMethod
+    public static IRubyObject java_object(ThreadContext context, IRubyObject recv) {
+        return (IRubyObject)recv.dataGetStruct();
+    }
+
+    @JRubyMethod(name = "java_object=")
+    public static IRubyObject java_object_set(ThreadContext context, IRubyObject recv, IRubyObject obj) {
+        // XXX: Check if it's appropriate type?
+        recv.dataWrapStruct(obj);
+        return obj;
+    }
+
     @JRubyMethod(name = {"=="})
     public static IRubyObject op_equal(IRubyObject recv, IRubyObject rhs) {
         return ((JavaObject)recv.dataGetStruct()).op_equal(rhs);
