@@ -184,7 +184,7 @@ public class JRubyEngine extends BSFEngineImpl {
         }
 
         public IRubyObject setValue(IRubyObject value) {
-            bean.bean = JavaUtil.convertArgument(runtime, Java.ruby_to_java(runtime.getObject(), value, Block.NULL_BLOCK), bean.type);
+            bean.bean = value.toJava(Object.class);
             return value;
         }
     }
