@@ -382,7 +382,7 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             Class[] parameterTypes = getParameterTypes();
             for (int i = 0; i < arguments.length; i++) {
                 arguments[i] = 
-                    JavaUtil.convertRubyToJava((IRubyObject) arguments[i], parameterTypes[i]);
+                    ((IRubyObject) arguments[i]).toJava(parameterTypes[i]);
             }
 
             try {
@@ -395,7 +395,7 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
                         iae.getMessage());
             } catch (InvocationTargetException ite) {
                 if (getRuntime().getDebug().isTrue()) ite.getTargetException().printStackTrace();
-                getRuntime().getJavaSupport().handleNativeException(ite.getTargetException());
+                getRuntime().getJavaSupport().handleNativeException(ite.getTargetException(), sm);
                 // This point is only reached if there was an exception handler
                 // installed.
                 return getRuntime().getNil();
