@@ -38,7 +38,6 @@ import org.jruby.runtime.Block;
 import org.jruby.runtime.BlockBody;
 import org.jruby.runtime.CallSite;
 import org.jruby.runtime.CallType;
-import org.jruby.runtime.CallbackFactory;
 import org.jruby.runtime.CompiledBlock;
 import org.jruby.runtime.CompiledBlock19;
 import org.jruby.runtime.CompiledBlockCallback;
@@ -66,26 +65,6 @@ public class RuntimeHelpers {
         if (receiver == self) return variableSite;
         return normalSite;
     }
-    public static IRubyObject doAttrAsgn(IRubyObject value, IRubyObject receiver, ThreadContext context, IRubyObject caller, CallSite callSite) {
-        callSite.call(context, caller, receiver, value);
-        return value;
-    }
-    public static IRubyObject doAttrAsgn(IRubyObject value, IRubyObject receiver, IRubyObject arg0, ThreadContext context, IRubyObject caller, CallSite callSite) {
-        callSite.call(context, caller, receiver, arg0, value);
-        return value;
-    }
-    public static IRubyObject doAttrAsgn(IRubyObject value, IRubyObject receiver, IRubyObject arg0, IRubyObject arg1, ThreadContext context, IRubyObject caller, CallSite callSite) {
-        callSite.call(context, caller, receiver, arg0, arg1, value);
-        return value;
-    }
-    public static IRubyObject doAttrAsgn(IRubyObject value, IRubyObject receiver, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, ThreadContext context, IRubyObject caller, CallSite callSite) {
-        callSite.call(context, caller, receiver, arg0, arg1, arg2, value);
-        return value;
-    }
-    public static IRubyObject doAttrAsgn(IRubyObject value, IRubyObject receiver, IRubyObject[] args, ThreadContext context, IRubyObject caller, CallSite callSite) {
-        callSite.call(context, caller, receiver, appendToObjectArray(args, value));
-        return value;
-    }
     public static IRubyObject doAttrAsgn(IRubyObject receiver, CallSite callSite, IRubyObject value, ThreadContext context, IRubyObject caller) {
         callSite.call(context, caller, receiver, value);
         return value;
@@ -98,10 +77,21 @@ public class RuntimeHelpers {
         callSite.call(context, caller, receiver, arg0, arg1, value);
         return value;
     }
+    public static IRubyObject doAttrAsgn(IRubyObject receiver, CallSite callSite, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, IRubyObject value, ThreadContext context, IRubyObject caller) {
+        callSite.call(context, caller, receiver, arg0, arg1, arg2, value);
+        return value;
+    }
     public static IRubyObject doAttrAsgn(IRubyObject receiver, CallSite callSite, IRubyObject[] args, ThreadContext context, IRubyObject caller) {
         callSite.call(context, caller, receiver, args);
         return args[args.length - 1];
     }
+    public static IRubyObject doAttrAsgn(IRubyObject receiver, CallSite callSite, IRubyObject[] args, IRubyObject value, ThreadContext context, IRubyObject caller) {
+        IRubyObject[] newArgs = new IRubyObject[args.length + 1];
+        System.arraycopy(args, 0, newArgs, 0, args.length);
+        newArgs[args.length] = value;
+        callSite.call(context, caller, receiver, newArgs);
+        return value;
+    }
 
     public static boolean invokeEqqForCaseWhen(CallSite callSite, ThreadContext context, IRubyObject caller, IRubyObject arg, IRubyObject[] receivers) {
         for (int i = 0; i < receivers.length; i++) {
@@ -151,7 +141,7 @@ public class RuntimeHelpers {
     public static CompiledBlockCallback createBlockCallback(Ruby runtime, Object scriptObject, String closureMethod) {
         Class scriptClass = scriptObject.getClass();
         ClassLoader scriptClassLoader = scriptClass.getClassLoader();
-        CallbackFactory factory = CallbackFactory.createFactory(runtime, scriptClass, scriptClassLoader);
+        MethodFactory factory = MethodFactory.createFactory(scriptClassLoader);
         
         return factory.getBlockCallback(closureMethod, scriptObject);
     }
@@ -159,7 +149,7 @@ public class RuntimeHelpers {
     public static CompiledBlockCallback19 createBlockCallback19(Ruby runtime, Object scriptObject, String closureMethod) {
         Class scriptClass = scriptObject.getClass();
         ClassLoader scriptClassLoader = scriptClass.getClassLoader();
-        CallbackFactory factory = CallbackFactory.createFactory(runtime, scriptClass, scriptClassLoader);
+        MethodFactory factory = MethodFactory.createFactory(scriptClassLoader);
 
         return factory.getBlockCallback19(closureMethod, scriptObject);
     }
@@ -308,57 +298,84 @@ public class RuntimeHelpers {
         return invoke(context, receiver, "method_missing", newArgs, Block.NULL_BLOCK);
     }
 
-    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, DynamicMethod method, String name, 
-                                                IRubyObject[] args, CallType callType, Block block) {
-        context.setLastCallStatusAndVisibility(callType, method.getVisibility());
-        return callMethodMissingInternal(context, receiver, name, args, block);
+    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, Visibility visibility, String name, CallType callType, IRubyObject[] args, Block block) {
+        return selectMethodMissing(context, receiver, visibility, name, callType).call(context, receiver, receiver.getMetaClass(), name, args, block);
     }
-    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, DynamicMethod method, String name, 
-                                                IRubyObject arg, CallType callType, Block block) {
-        context.setLastCallStatusAndVisibility(callType, method.getVisibility());
-        return callMethodMissingInternal(context, receiver, name, arg, block);
+
+    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, Visibility visibility, String name, CallType callType, IRubyObject arg0, Block block) {
+        return selectMethodMissing(context, receiver, visibility, name, callType).call(context, receiver, receiver.getMetaClass(), name, arg0, block);
     }
-    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, DynamicMethod method, String name, 
-                                                IRubyObject arg0, IRubyObject arg1, CallType callType, Block block) {
-        context.setLastCallStatusAndVisibility(callType, method.getVisibility());
-        return callMethodMissingInternal(context, receiver, name, arg0, arg1, block);
+
+    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, Visibility visibility, String name, CallType callType, IRubyObject arg0, IRubyObject arg1, Block block) {
+        return selectMethodMissing(context, receiver, visibility, name, callType).call(context, receiver, receiver.getMetaClass(), name, arg0, arg1, block);
     }
-    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, DynamicMethod method, String name, 
-                                                IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, CallType callType, Block block) {
-        context.setLastCallStatusAndVisibility(callType, method.getVisibility());
-        return callMethodMissingInternal(context, receiver, name, arg0, arg1, arg2, block);
+
+    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, Visibility visibility, String name, CallType callType, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
+        return selectMethodMissing(context, receiver, visibility, name, callType).call(context, receiver, receiver.getMetaClass(), name, arg0, arg1, arg2, block);
     }
-    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, DynamicMethod method, String name, 
-                                                CallType callType, Block block) {
-        context.setLastCallStatusAndVisibility(callType, method.getVisibility());
-        return callMethodMissingInternal(context, receiver, name, block);
+
+    public static IRubyObject callMethodMissing(ThreadContext context, IRubyObject receiver, Visibility visibility, String name, CallType callType, Block block) {
+        return selectMethodMissing(context, receiver, visibility, name, callType).call(context, receiver, receiver.getMetaClass(), name, block);
     }
-    
-    private static IRubyObject callMethodMissingInternal(ThreadContext context, IRubyObject receiver, String name, 
-                                                IRubyObject[] args, Block block) {
-        if (name.equals("method_missing")) return RubyKernel.method_missing(context, receiver, args, block);
-        IRubyObject[] newArgs = prepareMethodMissingArgs(args, context, name);
-        return invoke(context, receiver, "method_missing", newArgs, block);
+
+    public static DynamicMethod selectMethodMissing(ThreadContext context, IRubyObject receiver, Visibility visibility, String name, CallType callType) {
+        Ruby runtime = context.getRuntime();
+
+        if (name.equals("method_missing")) {
+            return selectInternalMM(runtime, visibility, callType);
+        }
+
+        DynamicMethod methodMissing = receiver.getMetaClass().searchMethod("method_missing");
+        if (methodMissing.isUndefined() || methodMissing == runtime.getDefaultMethodMissing()) {
+            return selectInternalMM(runtime, visibility, callType);
+        }
+        return new MethodMissingMethod(methodMissing);
     }
-    private static IRubyObject callMethodMissingInternal(ThreadContext context, IRubyObject receiver, String name, 
-                                                Block block) {
-        if (name.equals("method_missing")) return RubyKernel.method_missing(context, receiver, IRubyObject.NULL_ARRAY, block);
-        return invoke(context, receiver, "method_missing", context.getRuntime().newSymbol(name), block);
+
+    public static DynamicMethod selectMethodMissing(ThreadContext context, RubyClass selfClass, Visibility visibility, String name, CallType callType) {
+        Ruby runtime = context.getRuntime();
+
+        if (name.equals("method_missing")) {
+            return selectInternalMM(runtime, visibility, callType);
+        }
+
+        DynamicMethod methodMissing = selfClass.searchMethod("method_missing");
+        if (methodMissing.isUndefined() || methodMissing == runtime.getDefaultMethodMissing()) {
+            return selectInternalMM(runtime, visibility, callType);
+        }
+        return new MethodMissingMethod(methodMissing);
     }
-    private static IRubyObject callMethodMissingInternal(ThreadContext context, IRubyObject receiver, String name, 
-                                                IRubyObject arg0, Block block) {
-        if (name.equals("method_missing")) return RubyKernel.method_missing(context, receiver, constructObjectArray(arg0), block);
-        return invoke(context, receiver, "method_missing", context.getRuntime().newSymbol(name), arg0, block);
+
+    private static class MethodMissingMethod extends DynamicMethod {
+        private DynamicMethod delegate;
+
+        public MethodMissingMethod(DynamicMethod delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public IRubyObject call(ThreadContext context, IRubyObject self, RubyModule clazz, String name, IRubyObject[] args, Block block) {
+            return this.delegate.call(context, self, clazz, "method_missing", prepareMethodMissingArgs(args, context, name), block);
+        }
+
+        @Override
+        public DynamicMethod dup() {
+            return this;
+        }
     }
-    private static IRubyObject callMethodMissingInternal(ThreadContext context, IRubyObject receiver, String name, 
-                                                IRubyObject arg0, IRubyObject arg1, Block block) {
-        if (name.equals("method_missing")) return RubyKernel.method_missing(context, receiver, constructObjectArray(arg0,arg1), block);
-        return invoke(context, receiver, "method_missing", context.getRuntime().newSymbol(name), arg0, arg1, block);
-    }
-    private static IRubyObject callMethodMissingInternal(ThreadContext context, IRubyObject receiver, String name, 
-                                                IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
-        if (name.equals("method_missing")) return RubyKernel.method_missing(context, receiver, constructObjectArray(arg0,arg1,arg2), block);
-        return invoke(context, receiver, "method_missing", constructObjectArray(context.getRuntime().newSymbol(name), arg0, arg1, arg2), block);
+
+    private static DynamicMethod selectInternalMM(Ruby runtime, Visibility visibility, CallType callType) {
+        if (visibility == Visibility.PRIVATE) {
+            return runtime.getPrivateMethodMissing();
+        } else if (visibility == Visibility.PROTECTED) {
+            return runtime.getProtectedMethodMissing();
+        } else if (callType == CallType.VARIABLE) {
+            return runtime.getVariableMethodMissing();
+        } else if (callType == CallType.SUPER) {
+            return runtime.getSuperMethodMissing();
+        } else {
+            return runtime.getNormalMethodMissing();
+        }
     }
 
     private static IRubyObject[] prepareMethodMissingArgs(IRubyObject[] args, ThreadContext context, String name) {
@@ -449,7 +466,7 @@ public class RuntimeHelpers {
         DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
         
         if (method.isUndefined()) {
-            return callMethodMissing(context, self, method, name, args, CallType.SUPER, block);
+            return callMethodMissing(context, self, method.getVisibility(), name, CallType.SUPER, args, block);
         }
         return method.call(context, self, superClass, name, args, block);
     }
@@ -463,7 +480,7 @@ public class RuntimeHelpers {
         DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
 
         if (method.isUndefined()) {
-            return callMethodMissing(context, self, method, name, CallType.SUPER, block);
+            return callMethodMissing(context, self, method.getVisibility(), name, CallType.SUPER, block);
         }
         return method.call(context, self, superClass, name, block);
     }
@@ -477,7 +494,7 @@ public class RuntimeHelpers {
         DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
 
         if (method.isUndefined()) {
-            return callMethodMissing(context, self, method, name, arg0, CallType.SUPER, block);
+            return callMethodMissing(context, self, method.getVisibility(), name, CallType.SUPER, arg0, block);
         }
         return method.call(context, self, superClass, name, arg0, block);
     }
@@ -491,7 +508,7 @@ public class RuntimeHelpers {
         DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
 
         if (method.isUndefined()) {
-            return callMethodMissing(context, self, method, name, arg0, arg1, CallType.SUPER, block);
+            return callMethodMissing(context, self, method.getVisibility(), name, CallType.SUPER, arg0, arg1, block);
         }
         return method.call(context, self, superClass, name, arg0, arg1, block);
     }
@@ -505,7 +522,7 @@ public class RuntimeHelpers {
         DynamicMethod method = superClass != null ? superClass.searchMethod(name) : UndefinedMethod.INSTANCE;
 
         if (method.isUndefined()) {
-            return callMethodMissing(context, self, method, name, arg0, arg1, arg2, CallType.SUPER, block);
+            return callMethodMissing(context, self, method.getVisibility(), name, CallType.SUPER, arg0, arg1, arg2, block);
         }
         return method.call(context, self, superClass, name, arg0, arg1, arg2, block);
     }
@@ -559,7 +576,7 @@ public class RuntimeHelpers {
     }
     
     public static RubyModule prepareClassNamespace(ThreadContext context, IRubyObject rubyModule) {
-        if (rubyModule == null || rubyModule.isNil()) { // the isNil check should go away since class nil::Foo;end is not supposed be correct
+        if (rubyModule == null || rubyModule.isNil()) {
             rubyModule = context.getCurrentScope().getStaticScope().getModule();
 
             if (rubyModule == null) {
@@ -747,7 +764,7 @@ public class RuntimeHelpers {
     }
     
     public static IRubyObject backref(ThreadContext context) {
-        IRubyObject backref = context.getCurrentFrame().getBackRef();
+        IRubyObject backref = context.getCurrentScope().getBackRef(context.getRuntime());
         
         if(backref instanceof RubyMatchData) {
             ((RubyMatchData)backref).use();
@@ -756,28 +773,32 @@ public class RuntimeHelpers {
     }
     
     public static IRubyObject backrefLastMatch(ThreadContext context) {
-        IRubyObject backref = context.getCurrentFrame().getBackRef();
+        IRubyObject backref = context.getCurrentScope().getBackRef(context.getRuntime());
         
         return RubyRegexp.last_match(backref);
     }
     
     public static IRubyObject backrefMatchPre(ThreadContext context) {
-        IRubyObject backref = context.getCurrentFrame().getBackRef();
+        IRubyObject backref = context.getCurrentScope().getBackRef(context.getRuntime());
         
         return RubyRegexp.match_pre(backref);
     }
     
     public static IRubyObject backrefMatchPost(ThreadContext context) {
-        IRubyObject backref = context.getCurrentFrame().getBackRef();
+        IRubyObject backref = context.getCurrentScope().getBackRef(context.getRuntime());
         
         return RubyRegexp.match_post(backref);
     }
     
     public static IRubyObject backrefMatchLast(ThreadContext context) {
-        IRubyObject backref = context.getCurrentFrame().getBackRef();
+        IRubyObject backref = context.getCurrentScope().getBackRef(context.getRuntime());
         
         return RubyRegexp.match_last(backref);
     }
+
+    public static IRubyObject[] getArgValues(ThreadContext context) {
+        return context.getCurrentScope().getArgValues();
+    }
     
     public static IRubyObject callZSuper(Ruby runtime, ThreadContext context, Block block, IRubyObject self) {
         // Has the method that is calling super received a block argument
@@ -851,13 +872,13 @@ public class RuntimeHelpers {
         return isExceptionHandled(currentException, exception1, exception2, context);
     }
 
-    private static boolean checkJavaException(Exception currentException, IRubyObject exception) {
-        if (exception instanceof RubyClass) {
-            RubyClass rubyClass = (RubyClass)exception;
+    private static boolean checkJavaException(Throwable currentThrowable, IRubyObject throwable) {
+        if (throwable instanceof RubyClass) {
+            RubyClass rubyClass = (RubyClass)throwable;
             JavaClass javaClass = (JavaClass)rubyClass.fastGetInstanceVariable("@java_class");
             if (javaClass != null) {
                 Class cls = javaClass.javaClass();
-                if (cls.isInstance(currentException)) {
+                if (cls.isInstance(currentThrowable)) {
                     return true;
                 }
             }
@@ -865,12 +886,12 @@ public class RuntimeHelpers {
         return false;
     }
     
-    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject[] exceptions, ThreadContext context) {
-        if (currentException instanceof RaiseException) {
-            return isExceptionHandled(((RaiseException)currentException).getException(), exceptions, context);
+    public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject[] throwables, ThreadContext context) {
+        if (currentThrowable instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwables, context);
         } else {
-            for (int i = 0; i < exceptions.length; i++) {
-                if (checkJavaException(currentException, exceptions[0])) {
+            for (int i = 0; i < throwables.length; i++) {
+                if (checkJavaException(currentThrowable, throwables[0])) {
                     return context.getRuntime().getTrue();
                 }
             }
@@ -879,11 +900,11 @@ public class RuntimeHelpers {
         }
     }
 
-    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject exception, ThreadContext context) {
-        if (currentException instanceof RaiseException) {
-            return isExceptionHandled(((RaiseException)currentException).getException(), exception, context);
+    public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject throwable, ThreadContext context) {
+        if (currentThrowable instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwable, context);
         } else {
-            if (checkJavaException(currentException, exception)) {
+            if (checkJavaException(currentThrowable, throwable)) {
                 return context.getRuntime().getTrue();
             }
 
@@ -891,14 +912,14 @@ public class RuntimeHelpers {
         }
     }
 
-    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject exception0, IRubyObject exception1, ThreadContext context) {
-        if (currentException instanceof RaiseException) {
-            return isExceptionHandled(((RaiseException)currentException).getException(), exception0, exception1, context);
+    public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject throwable0, IRubyObject throwable1, ThreadContext context) {
+        if (currentThrowable instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwable0, throwable1, context);
         } else {
-            if (checkJavaException(currentException, exception0)) {
+            if (checkJavaException(currentThrowable, throwable0)) {
                 return context.getRuntime().getTrue();
             }
-            if (checkJavaException(currentException, exception1)) {
+            if (checkJavaException(currentThrowable, throwable1)) {
                 return context.getRuntime().getTrue();
             }
 
@@ -906,17 +927,17 @@ public class RuntimeHelpers {
         }
     }
 
-    public static IRubyObject isJavaExceptionHandled(Exception currentException, IRubyObject exception0, IRubyObject exception1, IRubyObject exception2, ThreadContext context) {
-        if (currentException instanceof RaiseException) {
-            return isExceptionHandled(((RaiseException)currentException).getException(), exception0, exception1, exception2, context);
+    public static IRubyObject isJavaExceptionHandled(Throwable currentThrowable, IRubyObject throwable0, IRubyObject throwable1, IRubyObject throwable2, ThreadContext context) {
+        if (currentThrowable instanceof RaiseException) {
+            return isExceptionHandled(((RaiseException)currentThrowable).getException(), throwable0, throwable1, throwable2, context);
         } else {
-            if (checkJavaException(currentException, exception0)) {
+            if (checkJavaException(currentThrowable, throwable0)) {
                 return context.getRuntime().getTrue();
             }
-            if (checkJavaException(currentException, exception1)) {
+            if (checkJavaException(currentThrowable, throwable1)) {
                 return context.getRuntime().getTrue();
             }
-            if (checkJavaException(currentException, exception2)) {
+            if (checkJavaException(currentThrowable, throwable2)) {
                 return context.getRuntime().getTrue();
             }
 
@@ -924,12 +945,12 @@ public class RuntimeHelpers {
         }
     }
 
-    public static void storeExceptionInErrorInfo(Exception currentException, ThreadContext context) {
+    public static void storeExceptionInErrorInfo(Throwable currentThrowable, ThreadContext context) {
         IRubyObject exception = null;
-        if (currentException instanceof RaiseException) {
-            exception = ((RaiseException)currentException).getException();
+        if (currentThrowable instanceof RaiseException) {
+            exception = ((RaiseException)currentThrowable).getException();
         } else {
-            exception = JavaUtil.convertJavaToUsableRubyObject(context.getRuntime(), currentException);
+            exception = JavaUtil.convertJavaToUsableRubyObject(context.getRuntime(), currentThrowable);
         }
         context.setErrorInfo(exception);
     }
@@ -1132,22 +1153,43 @@ public class RuntimeHelpers {
     
     public static RubyHash constructHash(Ruby runtime, IRubyObject key1, IRubyObject value1) {
         RubyHash hash = RubyHash.newHash(runtime);
-        hash.fastASet(key1, value1);
+        hash.fastASetCheckString(runtime, key1, value1);
         return hash;
     }
     
     public static RubyHash constructHash(Ruby runtime, IRubyObject key1, IRubyObject value1, IRubyObject key2, IRubyObject value2) {
         RubyHash hash = RubyHash.newHash(runtime);
-        hash.fastASet(key1, value1);
-        hash.fastASet(key2, value2);
+        hash.fastASetCheckString(runtime, key1, value1);
+        hash.fastASetCheckString(runtime, key2, value2);
         return hash;
     }
     
     public static RubyHash constructHash(Ruby runtime, IRubyObject key1, IRubyObject value1, IRubyObject key2, IRubyObject value2, IRubyObject key3, IRubyObject value3) {
         RubyHash hash = RubyHash.newHash(runtime);
-        hash.fastASet(key1, value1);
-        hash.fastASet(key2, value2);
-        hash.fastASet(key3, value3);
+        hash.fastASetCheckString(runtime, key1, value1);
+        hash.fastASetCheckString(runtime, key2, value2);
+        hash.fastASetCheckString(runtime, key3, value3);
+        return hash;
+    }
+    
+    public static RubyHash constructHash19(Ruby runtime, IRubyObject key1, IRubyObject value1) {
+        RubyHash hash = RubyHash.newHash(runtime);
+        hash.fastASetCheckString19(runtime, key1, value1);
+        return hash;
+    }
+    
+    public static RubyHash constructHash19(Ruby runtime, IRubyObject key1, IRubyObject value1, IRubyObject key2, IRubyObject value2) {
+        RubyHash hash = RubyHash.newHash(runtime);
+        hash.fastASetCheckString19(runtime, key1, value1);
+        hash.fastASetCheckString19(runtime, key2, value2);
+        return hash;
+    }
+    
+    public static RubyHash constructHash19(Ruby runtime, IRubyObject key1, IRubyObject value1, IRubyObject key2, IRubyObject value2, IRubyObject key3, IRubyObject value3) {
+        RubyHash hash = RubyHash.newHash(runtime);
+        hash.fastASetCheckString19(runtime, key1, value1);
+        hash.fastASetCheckString19(runtime, key2, value2);
+        hash.fastASetCheckString19(runtime, key3, value3);
         return hash;
     }
     
@@ -1207,20 +1249,20 @@ public class RuntimeHelpers {
     }
 
     public static IRubyObject setLastLine(Ruby runtime, ThreadContext context, IRubyObject value) {
-        return context.getCurrentFrame().setLastLine(value);
+        return context.getCurrentScope().setLastLine(value);
     }
 
     public static IRubyObject getLastLine(Ruby runtime, ThreadContext context) {
-        return context.getCurrentFrame().getLastLine();
+        return context.getCurrentScope().getLastLine(runtime);
     }
 
     public static IRubyObject setBackref(Ruby runtime, ThreadContext context, IRubyObject value) {
         if (!value.isNil() && !(value instanceof RubyMatchData)) throw runtime.newTypeError(value, runtime.getMatchData());
-        return context.getCurrentFrame().setBackRef(value);
+        return context.getCurrentScope().setBackRef(value);
     }
 
     public static IRubyObject getBackref(Ruby runtime, ThreadContext context) {
-        IRubyObject backref = context.getCurrentFrame().getBackRef();
+        IRubyObject backref = context.getCurrentScope().getBackRef(runtime);
         if (backref instanceof RubyMatchData) ((RubyMatchData)backref).use();
         return backref;
     }
@@ -1399,7 +1441,11 @@ public class RuntimeHelpers {
     }
 
     private static DynamicMethod constructSingletonMethod(MethodFactory factory, RubyClass rubyClass, String javaName, int arity, StaticScope scope, Object scriptObject, CallConfiguration callConfig) {
-        return factory.getCompiledMethodLazily(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig);
+        if (RubyInstanceConfig.LAZYHANDLES_COMPILE) {
+            return factory.getCompiledMethodLazily(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig);
+        } else {
+            return factory.getCompiledMethod(rubyClass, javaName, Arity.createArity(arity), Visibility.PUBLIC, scope, scriptObject, callConfig);
+        }
     }
 
     private static StaticScope creatScopeForClass(ThreadContext context, String[] scopeNames, int required, int optional, int rest) {
@@ -1588,4 +1634,18 @@ public class RuntimeHelpers {
     public static Block getBlock(Ruby runtime, ThreadContext context, IRubyObject self, Node node, Block aBlock) {
         return RuntimeHelpers.getBlockFromBlockPassBody(runtime, node.interpret(runtime, context, self, aBlock), aBlock);
     }
+
+    /**
+     * Equivalent to rb_equal in MRI
+     *
+     * @param context
+     * @param a
+     * @param b
+     * @return
+     */
+    public static IRubyObject rbEqual(ThreadContext context, IRubyObject a, IRubyObject b) {
+        Ruby runtime = context.getRuntime();
+        if (a == b) return runtime.getTrue();
+        return a.callMethod(context, "==", b);
+    }
 }
