@@ -35,6 +35,7 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby.parser;
 
+import org.jruby.CompatVersion;
 import org.jruby.ast.AndNode;
 import org.jruby.ast.ArgsPreOneArgNode;
 import org.jruby.ast.ArgsPreTwoArgNode;
@@ -66,6 +67,7 @@ import org.jruby.ast.CallNode;
 import org.jruby.ast.CallOneArgNode;
 import org.jruby.ast.CallOneArgBlockNode;
 import org.jruby.ast.CallOneArgBlockPassNode;
+import org.jruby.ast.CallOneArgFixnumNode;
 import org.jruby.ast.CallSpecialArgNode;
 import org.jruby.ast.CallSpecialArgBlockNode;
 import org.jruby.ast.CallSpecialArgBlockPassNode;
@@ -90,6 +92,7 @@ import org.jruby.ast.DAsgnNode;
 import org.jruby.ast.DRegexpNode;
 import org.jruby.ast.DStrNode;
 import org.jruby.ast.DotNode;
+import org.jruby.ast.EncodingNode;
 import org.jruby.ast.EvStrNode;
 import org.jruby.ast.FCallManyArgsBlockNode;
 import org.jruby.ast.FCallManyArgsBlockPassNode;
@@ -150,13 +153,17 @@ import org.jruby.ast.TrueNode;
 import org.jruby.ast.WhenNode;
 import org.jruby.ast.WhenOneArgNode;
 import org.jruby.ast.YieldNode;
+import org.jruby.ast.YieldOneNode;
+import org.jruby.ast.YieldThreeNode;
+import org.jruby.ast.YieldTwoNode;
+import org.jruby.ast.ZYieldNode;
 import org.jruby.ast.types.ILiteralNode;
 import org.jruby.ast.types.INameNode;
 import org.jruby.common.IRubyWarnings;
 import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.lexer.yacc.ISourcePosition;
 import org.jruby.lexer.yacc.ISourcePositionHolder;
-import org.jruby.lexer.yacc.IDESourcePosition;
+import org.jruby.lexer.yacc.RubyYaccLexer;
 import org.jruby.lexer.yacc.SyntaxException;
 import org.jruby.lexer.yacc.Token;
 import org.jruby.lexer.yacc.SyntaxException.PID;
@@ -170,6 +177,8 @@ import org.jruby.util.IdUtil;
 public class ParserSupport {
     // Parser states:
     protected StaticScope currentScope;
+
+    protected RubyYaccLexer lexer;
     
     // Is the parser current within a singleton (value is number of nested singletons)
     private int inSingleton;
@@ -189,7 +198,8 @@ public class ParserSupport {
     
     public void allowDubyExtension(ISourcePosition position) {
         if (!configuration.isDubyExtensionsEnabled()) {
-            throw new SyntaxException(PID.DUBY_EXTENSIONS_OFF, position, "Duby extensions not configured");
+            throw new SyntaxException(PID.DUBY_EXTENSIONS_OFF, position,
+                    lexer.getCurrentLine(), "Duby extensions not configured");
         }
     }
     
@@ -270,11 +280,9 @@ public class ParserSupport {
         case Tokens.k__FILE__:
             return new FileNode(token.getPosition(), ByteList.create(token.getPosition().getFile()));
         case Tokens.k__LINE__:
-            return new FixnumNode(token.getPosition(), token.getPosition().getEndLine()+1);
+            return new FixnumNode(token.getPosition(), token.getPosition().getStartLine()+1);
         case Tokens.k__ENCODING__:
-            // ENEBO: 1.8 will never see this, but 1.9 can
-            // TODO: Replace with proper encoding support
-            return new FixnumNode(token.getPosition(), token.getPosition().getEndLine()+1);            
+            return new EncodingNode(token.getPosition());
         case Tokens.tIDENTIFIER:
             return currentScope.declare(token.getPosition(), (String) token.getValue());
         case Tokens.tCONSTANT:
@@ -292,8 +300,8 @@ public class ParserSupport {
     }
     
     protected void getterIdentifierError(ISourcePosition position, String identifier) {
-        throw new SyntaxException(PID.BAD_IDENTIFIER, position, "identifier " +
-                identifier + " is not valid", identifier);
+        throw new SyntaxException(PID.BAD_IDENTIFIER, position, lexer.getCurrentLine(),
+                "identifier " + identifier + " is not valid", identifier);
     }
     
     public AssignableNode assignable(Token lhs, Node value) {
@@ -301,22 +309,29 @@ public class ParserSupport {
 
         switch (lhs.getType()) {
             case Tokens.kSELF:
-                throw new SyntaxException(PID.CANNOT_CHANGE_SELF, lhs.getPosition(), "Can't change the value of self");
+                throw new SyntaxException(PID.CANNOT_CHANGE_SELF, lhs.getPosition(),
+                        lexer.getCurrentLine(), "Can't change the value of self");
             case Tokens.kNIL:
-                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(), "Can't assign to nil", "nil");
+                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(),
+                        lexer.getCurrentLine(), "Can't assign to nil", "nil");
             case Tokens.kTRUE:
-                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(), "Can't assign to true", "true");
+                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(),
+                        lexer.getCurrentLine(), "Can't assign to true", "true");
             case Tokens.kFALSE:
-                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(), "Can't assign to false", "false");
+                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(),
+                        lexer.getCurrentLine(), "Can't assign to false", "false");
             case Tokens.k__FILE__:
-                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(), "Can't assign to __FILE__", "__FILE__");
+                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(),
+                        lexer.getCurrentLine(), "Can't assign to __FILE__", "__FILE__");
             case Tokens.k__LINE__:
-                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(), "Can't assign to __LINE__", "__LINE__");
+                throw new SyntaxException(PID.INVALID_ASSIGNMENT, lhs.getPosition(),
+                        lexer.getCurrentLine(), "Can't assign to __LINE__", "__LINE__");
             case Tokens.tIDENTIFIER:
-                return currentScope.assign(value != NilImplicitNode.NIL ? union(lhs, value) : lhs.getPosition(), (String) lhs.getValue(), makeNullNil(value));
+                return currentScope.assign(lhs.getPosition(), (String) lhs.getValue(), makeNullNil(value));
             case Tokens.tCONSTANT:
                 if (isInDef() || isInSingle()) {
-                    throw new SyntaxException(PID.DYNAMIC_CONSTANT_ASSIGNMENT, lhs.getPosition(), "dynamic constant assignment");
+                    throw new SyntaxException(PID.DYNAMIC_CONSTANT_ASSIGNMENT, lhs.getPosition(),
+                            lexer.getCurrentLine(), "dynamic constant assignment");
                 }
                 return new ConstDeclNode(lhs.getPosition(), (String) lhs.getValue(), null, value);
             case Tokens.tIVAR:
@@ -330,8 +345,8 @@ public class ParserSupport {
                 return new GlobalAsgnNode(lhs.getPosition(), (String) lhs.getValue(), value);
         }
 
-        throw new SyntaxException(PID.BAD_IDENTIFIER, lhs.getPosition(), "identifier " + 
-                (String) lhs.getValue() + " is not valid", lhs.getValue());
+        throw new SyntaxException(PID.BAD_IDENTIFIER, lhs.getPosition(), lexer.getCurrentLine(),
+                "identifier " + (String) lhs.getValue() + " is not valid", lhs.getValue());
     }
 
     /**
@@ -346,27 +361,6 @@ public class ParserSupport {
         return node instanceof NewlineNode ? node : new NewlineNode(position, node); 
     }
     
-    public ISourcePosition union(ISourcePositionHolder first, ISourcePositionHolder second) {
-        while (first instanceof NewlineNode) {
-            first = ((NewlineNode) first).getNextNode();
-        }
-
-        while (second instanceof NewlineNode) {
-            second = ((NewlineNode) second).getNextNode();
-        }
-        
-        if (second == null) return first.getPosition();
-        if (first == null) return second.getPosition();
-        
-        return first.getPosition().union(second.getPosition());
-    }
-    
-    public ISourcePosition union(ISourcePosition first, ISourcePosition second) {
-		if (first.getStartOffset() < second.getStartOffset()) return first.union(second); 
-
-		return second.union(first);
-	}
-    
     public Node addRootNode(Node topOfAST, ISourcePosition position) {
         position = topOfAST != null ? topOfAST.getPosition() : position;
 
@@ -407,7 +401,6 @@ public class ParserSupport {
 
         // Assumption: tail is never a list node
         ((ListNode) head).addAll(tail);
-        head.setPosition(union(head, tail));
         return head;
     }
 
@@ -429,8 +422,9 @@ public class ParserSupport {
         
         checkExpression(firstNode);
         checkExpression(secondNode);
-        
-        return new CallOneArgNode(union(firstNode.getPosition(), secondNode.getPosition()), firstNode, operator, new ArrayNode(secondNode.getPosition(), secondNode));
+
+        return new_call_one_arg(firstNode.getPosition(), firstNode, operator, secondNode);
+//        return new CallOneArgNode(firstNode.getPosition(), firstNode, operator, new ArrayNode(secondNode.getPosition(), secondNode));
     }
 
     public Node getMatchNode(Node firstNode, Node secondNode) {
@@ -504,7 +498,6 @@ public class ParserSupport {
         checkExpression(rhs);
         if (lhs instanceof AssignableNode) {
     	    ((AssignableNode) lhs).setValueNode(rhs);
-    	    lhs.setPosition(union(lhs, rhs));
         } else if (lhs instanceof IArgumentNode) {
             IArgumentNode invokableNode = (IArgumentNode) lhs;
             
@@ -517,7 +510,8 @@ public class ParserSupport {
     public Node ret_args(Node node, ISourcePosition position) {
         if (node != null) {
             if (node instanceof BlockPassNode) {
-                throw new SyntaxException(PID.DYNAMIC_CONSTANT_ASSIGNMENT, position, "Dynamic constant assignment.");
+                throw new SyntaxException(PID.DYNAMIC_CONSTANT_ASSIGNMENT, position,
+                        lexer.getCurrentLine(), "Dynamic constant assignment.");
             } else if (node instanceof ArrayNode && ((ArrayNode)node).size() == 1) {
                 node = ((ArrayNode)node).get(0);
             } else if (node instanceof SplatNode) {
@@ -716,7 +710,8 @@ public class ParserSupport {
 	 **/
     private boolean checkAssignmentInCondition(Node node) {
         if (node instanceof MultipleAsgnNode) {
-            throw new SyntaxException(PID.MULTIPLE_ASSIGNMENT_IN_CONDITIONAL, node.getPosition(), "Multiple assignment in conditional.");
+            throw new SyntaxException(PID.MULTIPLE_ASSIGNMENT_IN_CONDITIONAL, node.getPosition(),
+                    lexer.getCurrentLine(), "Multiple assignment in conditional.");
         } else if (node instanceof LocalAsgnNode || node instanceof DAsgnNode || node instanceof GlobalAsgnNode || node instanceof InstAsgnNode) {
             Node valueNode = ((AssignableNode) node).getValueNode();
             if (valueNode instanceof ILiteralNode || valueNode instanceof NilNode || valueNode instanceof TrueNode || valueNode instanceof FalseNode) {
@@ -772,7 +767,7 @@ public class ParserSupport {
             warningUnlessEOption(ID.REGEXP_LITERAL_IN_CONDITION, node, "regex literal in condition");
             
             return new MatchNode(node.getPosition(), node);
-        } 
+        }
 
         return node;
     }
@@ -811,12 +806,16 @@ public class ParserSupport {
         return new ArrayNode(position, makeNullNil(firstNode));
     }
 
+    public ISourcePosition position(ISourcePositionHolder one, ISourcePositionHolder two) {
+        return one == null ? two.getPosition() : one.getPosition();
+    }
+
     public AndNode newAndNode(ISourcePosition position, Node left, Node right) {
         checkExpression(left);
         
         if (left == null && right == null) return new AndNode(position, makeNullNil(left), makeNullNil(right));
         
-        return new AndNode(union(left, right), makeNullNil(left), makeNullNil(right));
+        return new AndNode(position(left, right), makeNullNil(left), makeNullNil(right));
     }
 
     public OrNode newOrNode(ISourcePosition position, Node left, Node right) {
@@ -824,7 +823,7 @@ public class ParserSupport {
 
         if (left == null && right == null) return new OrNode(position, makeNullNil(left), makeNullNil(right));
         
-        return new OrNode(union(left, right), makeNullNil(left), makeNullNil(right));
+        return new OrNode(position(left, right), makeNullNil(left), makeNullNil(right));
     }
 
     /**
@@ -914,7 +913,8 @@ public class ParserSupport {
         if (node instanceof ArrayNode && ((ArrayNode) node).size() == 1) { 
             return ((ListNode) node).get(0);
         } else if (node instanceof BlockPassNode) {
-            throw new SyntaxException(PID.BLOCK_ARG_UNEXPECTED, node.getPosition(), "Block argument should not be given.");
+            throw new SyntaxException(PID.BLOCK_ARG_UNEXPECTED, node.getPosition(),
+                    lexer.getCurrentLine(), "Block argument should not be given.");
         }
         return node;
     }
@@ -954,7 +954,7 @@ public class ParserSupport {
     }
     
     private Node new_call_noargs(Node receiver, Token name, IterNode iter) {
-        ISourcePosition position = union(receiver, name);
+        ISourcePosition position = position(receiver, name);
         
         if (receiver == null) receiver = NilImplicitNode.NIL;
         
@@ -967,19 +967,20 @@ public class ParserSupport {
         if (args instanceof BlockPassNode) {
             // Block and block pass passed in at same time....uh oh
             if (iter != null) {
-                throw new SyntaxException(PID.BLOCK_ARG_AND_BLOCK_GIVEN, iter.getPosition(), "Both block arg and actual block given.");
+                throw new SyntaxException(PID.BLOCK_ARG_AND_BLOCK_GIVEN, iter.getPosition(),
+                        lexer.getCurrentLine(), "Both block arg and actual block given.");
             }
 
             return new_call_blockpass(receiver, name, (BlockPassNode) args);
         }
 
-        if (iter != null) return new CallSpecialArgBlockNode(union(receiver, args), receiver,(String) name.getValue(), args, (IterNode) iter);
+        if (iter != null) return new CallSpecialArgBlockNode(position(receiver, args), receiver,(String) name.getValue(), args, (IterNode) iter);
 
-        return new CallSpecialArgNode(union(receiver, args), receiver, (String) name.getValue(), args);
+        return new CallSpecialArgNode(position(receiver, args), receiver, (String) name.getValue(), args);
     }
     
     private Node new_call_blockpass(Node receiver, Token operation, BlockPassNode blockPass) {
-        ISourcePosition position = union(receiver, blockPass);
+        ISourcePosition position = position(receiver, blockPass);
         String name = (String) operation.getValue();
         Node args = blockPass.getArgsNode();
         
@@ -999,6 +1000,33 @@ public class ParserSupport {
                 return new CallManyArgsBlockPassNode(position, receiver, name, args, blockPass);
         } 
     }
+    
+    private boolean isNumericOperator(String name) {
+        if (name.length() == 1) {
+            switch (name.charAt(0)) {
+                case '+': case '-': case '*': case '/': case '<': case '>':
+                    return true;
+            }
+        } else if (name.length() == 2) {
+            switch (name.charAt(0)) {
+            case '<': case '>': case '=':
+                switch (name.charAt(1)) {
+                case '=': case '<':
+                    return true;
+                }
+            }
+        }
+        
+        return false;
+    }
+
+    private Node new_call_one_arg(ISourcePosition position, Node receiver, String name, Node first) {
+        if (first instanceof FixnumNode && isNumericOperator(name)) {
+            return new CallOneArgFixnumNode(position, receiver, name, new ArrayNode(position, first));
+        }
+
+        return new CallOneArgNode(position, receiver, name, new ArrayNode(position, first));
+    }
 
     public Node new_call(Node receiver, Token name, Node argsNode, Node iter) {
         if (argsNode == null) return new_call_noargs(receiver, name, (IterNode) iter);
@@ -1008,28 +1036,39 @@ public class ParserSupport {
 
         switch (args.size()) {
             case 0:
-                if (iter != null) return new CallNoArgBlockNode(union(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
+                if (iter != null) return new CallNoArgBlockNode(position(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
                     
-                return new CallNoArgNode(union(receiver, args), receiver, args, (String) name.getValue());
+                return new CallNoArgNode(position(receiver, args), receiver, args, (String) name.getValue());
             case 1:
-                if (iter != null) return new CallOneArgBlockNode(union(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
-                
-                return new CallOneArgNode(union(receiver, args), receiver, (String) name.getValue(), args);
-            case 2:
-                if (iter != null) return new CallTwoArgBlockNode(union(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
-                
-                return new CallTwoArgNode(union(receiver, args), receiver, (String) name.getValue(), args);
-            case 3:
-                if (iter != null) return new CallThreeArgBlockNode(union(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
-                
-                return new CallThreeArgNode(union(receiver, args), receiver, (String) name.getValue(), args);
-            default:
-                if (iter != null) return new CallManyArgsBlockNode(union(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
+                if (iter != null) return new CallOneArgBlockNode(position(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
 
-                return new CallManyArgsNode(union(receiver, args), receiver, (String) name.getValue(), args);
+                return new CallOneArgNode(position(receiver, args), receiver, (String) name.getValue(), args);
+            case 2:
+                if (iter != null) return new CallTwoArgBlockNode(position(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
+                
+                return new CallTwoArgNode(position(receiver, args), receiver, (String) name.getValue(), args);
+            case 3:
+                if (iter != null) return new CallThreeArgBlockNode(position(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
+                
+                return new CallThreeArgNode(position(receiver, args), receiver, (String) name.getValue(), args);
+            default:
+                if (iter != null) return new CallManyArgsBlockNode(position(receiver, args), receiver, (String) name.getValue(), args, (IterNode) iter);
+
+                return new CallManyArgsNode(position(receiver, args), receiver, (String) name.getValue(), args);
         }
     }
 
+    public Node new_aref(Node receiver, Token name, Node argsNode) {
+        if (argsNode instanceof ArrayNode) {
+            ArrayNode args = (ArrayNode) argsNode;
+
+            if (args.size() == 1 && args.get(0) instanceof FixnumNode) {
+                return new CallOneArgFixnumNode(position(receiver, args), receiver, "[]", args);
+            }
+        }
+        return new_call(receiver, name, argsNode, null);
+    }
+
     public Colon2Node new_colon2(ISourcePosition position, Node leftNode, String name) {
         if (IdUtil.isConstant(name)) {
             if (leftNode == null) return new Colon2ImplicitNode(position, name);
@@ -1051,33 +1090,34 @@ public class ParserSupport {
     
     private Node new_fcall_simpleargs(Token operation, ArrayNode args, Node iter) {
         String name = (String) operation.getValue();
+        ISourcePosition position = position(operation, args);
             
         switch (args.size()) {
             case 0:  // foo()
-                if (iter != null) return new FCallNoArgBlockNode(union(operation, args), name, args, (IterNode) iter);
+                if (iter != null) return new FCallNoArgBlockNode(position, name, args, (IterNode) iter);
                     
-                return new FCallNoArgNode(union(operation, args), args, name);
+                return new FCallNoArgNode(position, args, name);
             case 1:
-                if (iter != null) return new FCallOneArgBlockNode(union(operation, args), name, args, (IterNode) iter);
+                if (iter != null) return new FCallOneArgBlockNode(position, name, args, (IterNode) iter);
                 
-                return new FCallOneArgNode(union(operation, args), name, args);
+                return new FCallOneArgNode(position, name, args);
             case 2:
-                if (iter != null) return new FCallTwoArgBlockNode(union(operation, args), name, args, (IterNode) iter);
+                if (iter != null) return new FCallTwoArgBlockNode(position, name, args, (IterNode) iter);
                 
-                return new FCallTwoArgNode(union(operation, args), name, args);
+                return new FCallTwoArgNode(position, name, args);
             case 3:
-                if (iter != null) return new FCallThreeArgBlockNode(union(operation, args), name, args, (IterNode) iter);
+                if (iter != null) return new FCallThreeArgBlockNode(position, name, args, (IterNode) iter);
                 
-                return new FCallThreeArgNode(union(operation, args), name, args);
+                return new FCallThreeArgNode(position, name, args);
             default:
-                if (iter != null) return new FCallManyArgsBlockNode(union(operation, args), name, args, (IterNode) iter);
+                if (iter != null) return new FCallManyArgsBlockNode(position, name, args, (IterNode) iter);
 
-                return new FCallManyArgsNode(union(operation, args), name, args);
+                return new FCallManyArgsNode(position, name, args);
         }
     }
     
     private Node new_fcall_blockpass(Token operation, BlockPassNode blockPass) {
-        ISourcePosition position = union(operation, blockPass);
+        ISourcePosition position = position(operation, blockPass);
         String name = (String) operation.getValue();
         Node args = blockPass.getArgsNode();
         
@@ -1104,16 +1144,17 @@ public class ParserSupport {
         if (args instanceof BlockPassNode) {
             if (iter == null) return new_fcall_blockpass(operation, (BlockPassNode) args);
 
-            throw new SyntaxException(PID.BLOCK_ARG_AND_BLOCK_GIVEN, iter.getPosition(), "Both block arg and actual block given.");
+            throw new SyntaxException(PID.BLOCK_ARG_AND_BLOCK_GIVEN, iter.getPosition(),
+                    lexer.getCurrentLine(), "Both block arg and actual block given.");
         }
 
-        if (iter != null) new FCallSpecialArgBlockNode(union(operation, args), (String) operation.getValue(), args, (IterNode) iter);
-        return new FCallSpecialArgNode(union(operation, args), (String) operation.getValue(), args);
+        if (iter != null) new FCallSpecialArgBlockNode(position(operation, args), (String) operation.getValue(), args, (IterNode) iter);
+        return new FCallSpecialArgNode(position(operation, args), (String) operation.getValue(), args);
     }
 
     public Node new_super(Node args, Token operation) {
         if (args != null && args instanceof BlockPassNode) {
-            return new SuperNode(union(operation, args), ((BlockPassNode) args).getArgsNode(), args);
+            return new SuperNode(position(operation, args), ((BlockPassNode) args).getArgsNode(), args);
         }
         return new SuperNode(operation.getPosition(), args);
     }
@@ -1184,20 +1225,24 @@ public class ParserSupport {
     public void setWarnings(IRubyWarnings warnings) {
         this.warnings = warnings;
     }
+
+    public void setLexer(RubyYaccLexer lexer) {
+        this.lexer = lexer;
+    }
     
     public Node literal_concat(ISourcePosition position, Node head, Node tail) { 
         if (head == null) return tail;
         if (tail == null) return head;
         
         if (head instanceof EvStrNode) {
-            head = new DStrNode(union(head.getPosition(), position)).add(head);
+            head = new DStrNode(head.getPosition()).add(head);
         } 
 
         if (tail instanceof StrNode) {
             if (head instanceof StrNode) {
-        	    return new StrNode(union(head, tail), (StrNode) head, (StrNode) tail);
+        	    return new StrNode(head.getPosition(), (StrNode) head, (StrNode) tail);
             } 
-            head.setPosition(union(head, tail));
+            head.setPosition(head.getPosition());
             return ((ListNode) head).add(tail);
         	
         } else if (tail instanceof DStrNode) {
@@ -1216,8 +1261,6 @@ public class ParserSupport {
             if(((StrNode) head).getValue().length() == 0) {
                 head = new DStrNode(head.getPosition());
             } else {
-                // All first element StrNode's do not include syntacical sugar.
-                head.getPosition().adjustStartOffset(-1);
                 head = new DStrNode(head.getPosition()).add(head);
             }
         }
@@ -1246,11 +1289,12 @@ public class ParserSupport {
         
         if (node != null) {
             if (node instanceof BlockPassNode) {
-                throw new SyntaxException(PID.BLOCK_ARG_UNEXPECTED, node.getPosition(), "Block argument should not be given.");
+                throw new SyntaxException(PID.BLOCK_ARG_UNEXPECTED, node.getPosition(),
+                        lexer.getCurrentLine(), "Block argument should not be given.");
             }
 
-            // FIXME: This does not seem to be in 1.9
-            if (node instanceof ArrayNode && ((ArrayNode)node).size() == 1) {
+            if (node instanceof ArrayNode && configuration.getVersion() == CompatVersion.RUBY1_8 &&
+                    ((ArrayNode)node).size() == 1) {
                 node = ((ArrayNode)node).get(0);
                 state = false;
             }
@@ -1259,9 +1303,28 @@ public class ParserSupport {
                 state = true;
             }
         } else {
-            state = false;
+            return new ZYieldNode(position);
         }
-        
+
+        if (state && node instanceof ArrayNode) {
+            ArrayNode args = (ArrayNode) node;
+
+            switch (args.size()) {
+                case 0:
+                    return new ZYieldNode(position);
+                case 1:
+                    return new YieldOneNode(position, args);
+                case 2:
+                    return new YieldTwoNode(position, args);
+                case 3:
+                    return new YieldThreeNode(position, args);
+            }
+        }
+
+        if (node instanceof FixnumNode) {
+            return new YieldOneNode(position, (FixnumNode) node);
+        }
+
         return new YieldNode(position, node, state);
     }
     
@@ -1285,9 +1348,10 @@ public class ParserSupport {
         
         return floatNode;
     }
-    
+
+    // FIXME: Remove this from grammars.
     public ISourcePosition createEmptyArgsNodePosition(ISourcePosition pos) {
-        return new IDESourcePosition(pos.getFile(), pos.getStartLine(), pos.getEndLine(), pos.getEndOffset() - 1, pos.getEndOffset() - 1);
+        return pos;
     }
     
     public Node unwrapNewlineNode(Node node) {
@@ -1307,9 +1371,8 @@ public class ParserSupport {
             return null;
         }
         String name = getCurrentScope().getLocalScope().getVariables()[index];
-        // FIXME: We should not need to specify IDESourcePosition here...
-        ISourcePosition position = new IDESourcePosition(token.getPosition().getFile(), token.getPosition().getStartLine(), token.getPosition().getEndLine(), token.getPosition().getStartOffset(), token.getPosition().getEndOffset() + name.length());
-        return new ArgumentNode(position, name);
+
+        return new ArgumentNode(token.getPosition(), name);
     }
 
     public Node new_args(ISourcePosition position, ListNode pre, ListNode optional, RestArgNode rest,
