@@ -41,7 +41,9 @@ public final class ClassIndex {
     public static final int CONVERTER = 23;
     public static final int GENERATOR = 24;
     public static final int YIELDER = 25;
-    public static final int MAX_CLASSES = 26;
+    public static final int FILE = 26;
+    public static final int MATCH = 27;
+    public static final int MAX_CLASSES = 28;
 
     /** Creates a new instance of ClassIndex */
     private ClassIndex() {
