@@ -45,7 +45,7 @@ public class CompiledBlock extends BlockBody {
     protected final CompiledBlockCallback callback;
     protected final boolean hasMultipleArgsHead;
     protected final Arity arity;
-    protected final StaticScope scope;
+    protected StaticScope scope;
     
     public static Block newCompiledClosure(ThreadContext context, IRubyObject self, Arity arity,
             StaticScope scope, CompiledBlockCallback callback, boolean hasMultipleArgsHead, int argumentType) {
@@ -78,14 +78,17 @@ public class CompiledBlock extends BlockBody {
         return yield(context, null, binding, type);
     }
 
+    @Override
     public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, Binding binding, Block.Type type) {
         return yield(context, arg0, binding, type);
     }
 
+    @Override
     public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, Binding binding, Block.Type type) {
         return yield(context, context.getRuntime().newArrayNoCopyLight(arg0, arg1), null, null, true, binding, type);
     }
 
+    @Override
     public IRubyObject yieldSpecific(ThreadContext context, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Binding binding, Block.Type type) {
         return yield(context, context.getRuntime().newArrayNoCopyLight(arg0, arg1, arg2), null, null, true, binding, type);
     }
@@ -201,6 +204,10 @@ public class CompiledBlock extends BlockBody {
         return scope;
     }
 
+    public void setStaticScope(StaticScope newScope) {
+        this.scope = newScope;
+    }
+
     public Block cloneBlock(Binding binding) {
         binding = binding.clone();
         
