@@ -416,6 +416,15 @@ public final class ThreadContext {
     public int getFrameCount() {
         return frameIndex + 1;
     }
+
+    public Frame[] getFrames(int delta) {
+        int top = frameIndex + delta;
+        Frame[] frames = new Frame[top + 1];
+        for (int i = 0; i <= top; i++) {
+            frames[i] = frameStack[i].duplicateForBacktrace();
+        }
+        return frames;
+    }
     
     public String getFrameName() {
         return getCurrentFrame().getName();
@@ -895,7 +904,7 @@ public final class ThreadContext {
     
     public static IRubyObject createRubyCompiledBacktrace(Ruby runtime, StackTraceElement[] stackTrace) {
         RubyArray traceArray = RubyArray.newArray(runtime);
-        for (int i = 17; i < stackTrace.length; i++) {
+        for (int i = 0; i < stackTrace.length; i++) {
             StackTraceElement element = stackTrace[i];
             int index = element.getMethodName().indexOf("$RUBY$");
             if (index < 0) continue;
@@ -1238,8 +1247,11 @@ public final class ThreadContext {
         popFrame();
     }
     
-    public void preRunThread(Frame currentFrame) {
-        pushFrame(currentFrame);
+    public void preRunThread(Frame[] currentFrames) {
+        for (Frame frame : currentFrames) {
+            pushFrame(frame);
+        }
+        setFileAndLine(getCurrentFrame());
     }
     
     public void preTrace() {
