@@ -232,6 +232,13 @@ public interface IRubyObject {
      */
     IRubyObject checkArrayType();
 
+    /**
+     * Convert the object to the specified Java class, if possible.
+     *
+     * @param cls The target type to which the object should be converted.
+     */
+    Object toJava(Class cls);
+
     /**
      * RubyMethod dup.
      * @return
@@ -280,6 +287,7 @@ public interface IRubyObject {
      * @return the object wrapped.
      */
     Object dataGetStruct();
+    Object dataGetStructChecked();
     
     /**
      *
@@ -324,12 +332,12 @@ public interface IRubyObject {
      * 
      * @param variables the variables to be set for object 
      */
-    void syncVariables(List<Variable<IRubyObject>> variables);
+    void syncVariables(List<Variable<Object>> variables);
     
     /**
      * @return a list of all variables (ivar/cvar/constant/internal)
      */
-    List<Variable<IRubyObject>> getVariableList();
+    List<Variable<Object>> getVariableList();
 
     //
     // INSTANCE VARIABLE METHODS
@@ -349,4 +357,7 @@ public interface IRubyObject {
     List<String> getVariableNameList();
 
     void copySpecialInstanceVariables(IRubyObject clone);
+
+    public Object getVariable(int index);
+    public void setVariable(int index, Object value);
 }
