@@ -42,7 +42,7 @@ public class JavaCompiledScript implements Library {
 
     public void load(Ruby runtime, boolean wrap) {
         try {
-            Script script = CompiledScriptLoader.loadScriptFromFile(runtime, resource.getURL().openStream(), resource.getName());
+            Script script = CompiledScriptLoader.loadScriptFromFile(runtime, resource.getInputStream(), resource.getName());
             if (script == null) {
                 // we're depending on the side effect of the load, which loads the class but does not turn it into a script
                 // I don't like it, but until we restructure the code a bit more, we'll need to quietly let it by here.
