@@ -28,15 +28,14 @@
 package org.jruby.util;
 
 import java.io.File;
-
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Enumeration;
 import java.util.List;
-
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
 
-import java.util.Enumeration;
+import org.jruby.ext.posix.JavaSecuredFile;
 
 /**
  * This class exists as a counterpart to the dir.c file in 
@@ -565,7 +564,7 @@ public class Dir {
             }
 
             if (bytes[begin] == '/' || (DOSISH && begin+2<end && bytes[begin+1] == ':' && isdirsep(bytes[begin+2]))) {
-                if (new File(newStringFromUTF8(bytes, begin, end - begin)).exists()) {
+                if (new JavaSecuredFile(newStringFromUTF8(bytes, begin, end - begin)).exists()) {
                     status = func.call(bytes, begin, end, arg);
                 }
             } else if (isJarFilePath(bytes, begin, end)) {
@@ -577,7 +576,7 @@ public class Dir {
                     }
                 }
 
-                st = new File(newStringFromUTF8(bytes, begin+5, ix-5));
+                st = new JavaSecuredFile(newStringFromUTF8(bytes, begin+5, ix-5));
                 String jar = newStringFromUTF8(bytes, begin+ix+1, end-(ix+1));
                 try {
                     JarFile jf = new JarFile(st);
@@ -589,7 +588,7 @@ public class Dir {
                     }
                 } catch(Exception e) {}
             } else if ((end - begin) > 0) { // Length check is a hack.  We should not be reeiving "" as a filename ever. 
-                if (new File(cwd, newStringFromUTF8(bytes, begin, end - begin)).exists()) {
+                if (new JavaSecuredFile(cwd, newStringFromUTF8(bytes, begin, end - begin)).exists()) {
                     status = func.call(bytes, begin, end - begin, arg);
                 }
             }
@@ -613,7 +612,7 @@ public class Dir {
                     JarFile jf = null;
 
                     if(dir[0] == '/'  || (DOSISH && 2<dir.length && dir[1] == ':' && isdirsep(dir[2]))) {
-                        st = new File(newStringFromUTF8(dir));
+                        st = new JavaSecuredFile(newStringFromUTF8(dir));
                     } else if(isJarFilePath(dir, 0, dir.length)) {
                         int ix = dir.length;
                         for(int i = 0;i<dir.length;i++) {
@@ -623,7 +622,7 @@ public class Dir {
                             }
                         }
 
-                        st = new File(newStringFromUTF8(dir, 5, ix-5));
+                        st = new JavaSecuredFile(newStringFromUTF8(dir, 5, ix-5));
                         jar = newStringFromUTF8(dir, ix+1, dir.length-(ix+1));
                         try {
                             jf = new JarFile(st);
@@ -635,7 +634,7 @@ public class Dir {
                             jf = null;
                         }
                     } else {
-                        st = new File(cwd, newStringFromUTF8(dir));
+                        st = new JavaSecuredFile(cwd, newStringFromUTF8(dir));
                     }
 
                     if((jf != null && ("".equals(jar) || (jf.getJarEntry(jar) != null && jf.getJarEntry(jar).isDirectory()))) || st.isDirectory()) {
@@ -668,9 +667,9 @@ public class Dir {
                                 buf.append( BASE(base) ? SLASH : EMPTY );
                                 buf.append(getBytesInUTF8(dirp[i]));
                                 if (buf.bytes[0] == '/' || (DOSISH && 2<buf.realSize && buf.bytes[1] == ':' && isdirsep(buf.bytes[2]))) {
-                                    st = new File(newStringFromUTF8(buf.bytes, buf.begin, buf.realSize));
+                                    st = new JavaSecuredFile(newStringFromUTF8(buf.bytes, buf.begin, buf.realSize));
                                 } else {
-                                    st = new File(cwd, newStringFromUTF8(buf.bytes, buf.begin, buf.realSize));
+                                    st = new JavaSecuredFile(cwd, newStringFromUTF8(buf.bytes, buf.begin, buf.realSize));
                                 }
                                 if(st.isDirectory() && !".".equals(dirp[i]) && !"..".equals(dirp[i])) {
                                     int t = buf.realSize;
@@ -768,9 +767,9 @@ public class Dir {
                     for (ByteList b : link) {
                         if (status == 0) {
                             if(b.bytes[0] == '/'  || (DOSISH && 2<b.realSize && b.bytes[1] == ':' && isdirsep(b.bytes[2]))) {
-                                st = new File(newStringFromUTF8(b.bytes, 0, b.realSize));
+                                st = new JavaSecuredFile(newStringFromUTF8(b.bytes, 0, b.realSize));
                             } else {
-                                st = new File(cwd, newStringFromUTF8(b.bytes, 0, b.realSize));
+                                st = new JavaSecuredFile(cwd, newStringFromUTF8(b.bytes, 0, b.realSize));
                             }
 
                             if(st.isDirectory()) {
