@@ -36,8 +36,6 @@ package org.jruby.test;
 import junit.framework.Test;
 import junit.framework.TestSuite;
 
-import org.jruby.ast.executable.YARVMachineTest;
-import org.jruby.ast.visitor.rewriter.VisitorTestSuite;
 import org.jruby.ext.posix.JavaFileStatTest;
 import org.jruby.javasupport.TestJava;
 import org.jruby.javasupport.TestJavaClass;
@@ -46,6 +44,7 @@ import org.jruby.runtime.EventHookTest;
 import org.jruby.util.JRubyThreadContextTest;
 import org.jruby.util.PlatformTest;
 import org.jruby.util.ShellLauncherTest;
+import org.jruby.util.TimeOutputFormatterTest;
 
 /**
  *
@@ -74,9 +73,7 @@ public class MainTestSuite extends TestSuite {
         suite.addTestSuite(TestRubyException.class);
         suite.addTestSuite(TestAdoptedThreading.class);
         suite.addTestSuite(TestRubyArray.class);
-        suite.addTestSuite(YARVMachineTest.class);
         suite.addTestSuite(TestRaiseException.class);
-        suite.addTest(VisitorTestSuite.suite());
         suite.addTestSuite(PlatformTest.class);
         suite.addTestSuite(ShellLauncherTest.class);
         suite.addTestSuite(TestRbConfigLibrary.class);
@@ -87,6 +84,8 @@ public class MainTestSuite extends TestSuite {
         suite.addTestSuite(TestCodeCache.class);
         suite.addTestSuite(TestJavaReentrantExceptions.class);
         suite.addTestSuite(EventHookTest.class);
+        suite.addTestSuite(TestMethodFactories.class);
+        suite.addTestSuite(TimeOutputFormatterTest.class);
         return suite;
     }
 }
