@@ -49,8 +49,10 @@ public class TestKernel extends TestRubyBase {
         super(name);
     }
 
-    public void setUp() {
-        runtime = Ruby.getDefaultInstance();
+    @Override
+    protected void setUp() throws Exception {
+        super.setUp();
+        runtime = Ruby.newInstance();
         runtime.getLoadService().init(new ArrayList());
     }
 
@@ -66,7 +68,7 @@ public class TestKernel extends TestRubyBase {
         assertEquals("failed to load the file test/loadTest", "0", eval("require 'test/loadTest'"));
         assertEquals("incorrectly reloaded the file test/loadTest", "", eval("require 'test/loadTest'"));
 
-        assertEquals("incorrect value for $\" variable", "test/loadTest.rb", eval("print $\".sort"));
+        assertEquals("incorrect value for $\" variable", "test/loadTest.rb", eval("print $\"[-1]"));
     }
 
     public void testPrintf() throws Exception {
@@ -87,7 +89,7 @@ public class TestKernel extends TestRubyBase {
             fail("Expected a SystemExit to be thrown by calling exit.");
         } catch (RaiseException re) {
         	RubyException raisedException = re.getException();
-        	if (raisedException.isKindOf(runtime.getClass("SystemExit"))) {
+        	if (runtime.getClass("SystemExit").isInstance(raisedException)) {
 	            IRubyObject status = raisedException.callMethod(runtime.getCurrentContext(), "status");
 	            assertEquals(expectedStatus, status);
         	} else {
@@ -95,9 +97,4 @@ public class TestKernel extends TestRubyBase {
         	}
         }
     }
-        
-    public void tearDown() {
-        super.tearDown();
-    }
-
 }
